@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qcstring.h>
@@ -25,14 +25,15 @@
 #include <qimage.h>
 
 #include <kgenericfactory.h>
-#include <koFilter.h>
-#include <koFilterChain.h>
-#include <koStore.h>
+#include <KoFilter.h>
+#include <KoFilterChain.h>
+#include <KoStore.h>
 
 #include "pngexport.h"
 #include "vdocument.h"
 #include "vselection.h"
 #include "vkopainter.h"
+#include "vlayer.h"
 
 #include <kdebug.h>
 
@@ -66,8 +67,16 @@ PngExport::convert( const QCString& from, const QCString& to )
 	VDocument doc;
 	doc.load( docNode );
 
-	// select all objects:
-	doc.selection()->append();
+	VLayerListIterator layerItr( doc.layers() );
+	VLayer *currentLayer;
+
+	for( ; currentLayer = layerItr.current(); ++layerItr )
+	{
+		if( currentLayer->state() == VObject::normal || currentLayer->state() == VObject::normal_locked || currentLayer->state() == VObject::selected )
+		{
+			doc.selection()->append(currentLayer->objects());
+		}
+	}
 
 	// get the bounding box of all selected objects:
 	const KoRect& rect = doc.selection()->boundingBox();
