@@ -13,15 +13,15 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include "stylefactory.h"
 
 #include <qcolor.h>
 #include <qdatetime.h>
-#include <koUnit.h>
+#include <KoUnit.h>
 #include <kdebug.h>
 
 StyleFactory::StyleFactory()
@@ -657,11 +657,24 @@ bool PageMasterStyle::operator==( const PageMasterStyle & pageMasterStyle ) cons
 
 PageStyle::PageStyle( StyleFactory * styleFactory, QDomElement & e, const uint index )
 {
-    // some defaults that won't be overwritten because they
-    // are not available in KPresenter
-    m_bg_visible = "true";
-    m_bg_objects_visible = "true";
-
+    QDomElement backMaster = e.namedItem( "BACKMASTER" ).toElement();
+    if( !backMaster.isNull())
+    {
+        int tmp=0;
+	if(backMaster.hasAttribute("displayBackground"))
+		tmp = backMaster.attribute("displayBackground").toInt();
+	 m_bg_visible = (tmp==1) ? "true" : "false";	
+	 tmp = 0;
+         if(backMaster.hasAttribute("displayMasterPageObject"))
+	 	tmp = backMaster.attribute("displayMasterPageObject").toInt();
+	 m_bg_objects_visible = (tmp==1) ? "true" : "false";	 
+    }
+    else
+    {
+    	m_bg_visible = "true";
+    	m_bg_objects_visible = "true";
+    }
+    
     m_name = QString( "dp%1" ).arg( index );
 
     // check if this is an empty page tag
