@@ -13,22 +13,23 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
-#if !defined MAGICKEXPORT_H_
+#ifndef MAGICKEXPORT_H_
 #define MAGICKEXPORT_H_
 
-#include <koFilter.h>
+#include <KoFilter.h>
 
 class MagickExport : public KoFilter {
-	Q_OBJECT
+    Q_OBJECT
 
 public:
-	MagickExport(KoFilter *parent, const char *name, const QStringList&);
-	virtual ~MagickExport();
+    MagickExport(KoFilter *parent, const char *name, const QStringList&);
+    virtual ~MagickExport();
 
 public:
-	virtual KoFilter::ConversionStatus convert(const QCString& from, const QCString& to);
+    virtual KoFilter::ConversionStatus convert(const QCString& from, const QCString& to);
 };
 
 #endif // MAGICKEXPORT_H_
