@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, US
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA
 */
 
 #ifndef SWINDER_EXCEL_H
@@ -168,16 +168,12 @@ public:
   const char* functionName() const;  // for non external function
   unsigned functionParams() const;
   
-  // only when id is Ref or Ref3d
+  // only when id is Ref
   UString ref( unsigned row, unsigned col ) const;
 
-  // only when id is Area or Aread3d
+  // only when id is Area
   UString area( unsigned row, unsigned col ) const;
 
-  // only when id is Ref3d or Area3d
-  // if -1, means it is invalid
-  int sheetIndex() const;
-
   // only when id is Attr
   unsigned attr() const;
 
@@ -1187,6 +1183,38 @@ private:
    EOFRecord& operator=( const EOFRecord& );
 };
 
+class FilepassRecord : public Record
+{
+public:
+
+  static const unsigned int id;
+
+  unsigned int rtti(){
+    return this->id;
+  }
+
+  /**
+   * Creates a new FILEPASS record.
+   */
+  FilepassRecord();
+
+  /**
+   * Destroy the record.
+   */
+  virtual ~FilepassRecord();
+  
+  virtual void setData( unsigned size, const unsigned char* data );
+
+  virtual const char* name(){ return "FILEPASS"; }
+
+  virtual void dump( std::ostream& out ) const;
+
+private:
+   // no copy or assign
+   FilepassRecord( const FilepassRecord& );
+   FilepassRecord& operator=( const FilepassRecord& );
+};
+
 /**
   \brief Font information.
   
@@ -3066,6 +3094,7 @@ private:
   void handleDateMode( DateModeRecord* record );
   void handleDimension( DimensionRecord* record );
   void handleExternName( ExternNameRecord* record );
+  void handleFilepass( FilepassRecord* record );
   void handleFormat( FormatRecord* record );
   void handleFormula( FormulaRecord* record );
   void handleFont( FontRecord* record );
