@@ -1,5 +1,3 @@
-// 
-
 /* This file is part of the KDE project
    Copyright (C) 2001, 2002, 2003, 2004 Nicolas GOUTTE <goutte@kde.org>
 
@@ -15,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 /*
@@ -42,9 +40,9 @@
 #include <klocale.h>
 #include <kzip.h>
 
-#include <koPageLayout.h>
-#include <koPictureKey.h>
-#include <koPicture.h>
+#include <KoPageLayout.h>
+#include <KoPictureKey.h>
+#include <KoPicture.h>
 
 #include <KWEFStructures.h>
 #include <KWEFUtil.h>
@@ -360,7 +358,8 @@ void OOWriterWorker::writeStartOfFile(const QString& type)
 void OOWriterWorker::writeFontDeclaration(void)
 {
     zipWriteData( " <office:font-decls>\n");
-    for (QMap<QString,QString>::ConstIterator it=m_fontNames.begin(); it!=m_fontNames.end(); it++)
+	QMap<QString,QString>::ConstIterator end(m_fontNames.end());
+    for (QMap<QString,QString>::ConstIterator it=m_fontNames.begin(); it!=end; ++it)
     {
         const bool space=(it.key().find(' ')>=0); // Does the font has at least a space in its name
         const QString fontName(escapeOOText(it.key()));
@@ -500,7 +499,7 @@ void OOWriterWorker::writeMetaXml(void)
 
     // Tell who we are in case that we have a bug in our filter output!
     zipWriteData("  <meta:generator>KWord's OOWriter Export Filter");
-    zipWriteData(QString("$Revision: 379357 $").mid(10).remove('$')); // has a leading and a trailing space.
+    zipWriteData(QString("$Revision: 515673 $").mid(10).remove('$')); // has a leading and a trailing space.
 
     zipWriteData("</meta:generator>\n");
 
@@ -1598,7 +1597,7 @@ void OOWriterWorker::processNote( const VariableData& variable )
 }
 
 void OOWriterWorker::processVariable ( const QString&,
-    const TextFormatting& formatLayout,
+    const TextFormatting& /*formatLayout*/,
     const FormatData& formatData)
 {
     if (0==formatData.variable.m_type)
@@ -1931,7 +1930,8 @@ QString OOWriterWorker::layoutToParagraphStyle(const LayoutData& layoutOrigin,
     {
         props += "\n    <style:tab-stops>\n";
         TabulatorList::ConstIterator it;
-        for (it=layout.tabulatorList.begin();it!=layout.tabulatorList.end();it++)
+		TabulatorList::ConstIterator end(layout.tabulatorList.end());
+        for (it=layout.tabulatorList.begin();it!=end;++it)
         {
             props+="     <style:tab-stop style:position=\"";
             props += QString::number((*it).m_ptpos);
