@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef __VSTROKE_H__
@@ -27,10 +27,12 @@
 #include "vdashpattern.h"
 #include "vgradient.h"
 #include "vpattern.h"
-
+#include <koffice_export.h>
 
 class QDomElement;
 class VObject;
+class KoGenStyle;
+class KoStyleStack;
 
 
 /**
@@ -43,7 +45,7 @@ class VObject;
  * style and no dashes.
  */
 
-class VStroke
+class KARBONBASE_EXPORT VStroke
 {
 public:
 	enum VStrokeType
@@ -107,11 +109,15 @@ public:
 	const VDashPattern& dashPattern() const { return m_dashPattern; }
 
 	void save( QDomElement& element ) const;
+	void saveOasis( KoGenStyle &style ) const;
 	void load( const QDomElement& element );
+	void loadOasis( const KoStyleStack &stack );
 
 
 	VStroke& operator=( const VStroke& stroke );
 
+	void transform( const QWMatrix& m );
+
 private:
 	VObject			*m_parent;
 
