@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qlabel.h>
@@ -27,8 +27,8 @@
 
 #include <klocale.h>
 #include <kiconloader.h>
-#include <koMainWindow.h>
-#include <koFilterManager.h>
+#include <KoMainWindow.h>
+#include <KoFilterManager.h>
 #include <kfiledialog.h>
 
 #include "karbon_part.h"
@@ -73,12 +73,11 @@ ClipartChooser::startDrag()
 }
 
 VStyleDocker::VStyleDocker( KarbonPart* part, KarbonView* parent, const char* /*name*/ )
-	: VDocker( parent->shell() ), m_part ( part ), m_view( parent )
+	: QWidget(), m_part ( part ), m_view( parent )
 {
-	setCaption( i18n( "Style Settings" ) );
+	setCaption( i18n( "Resources" ) );
 
-	mainWidget = new QWidget( this );
-	mTabWidget = new QTabWidget( mainWidget );
+	mTabWidget = new QTabWidget( this );
 
 	//Pattern
 	KoPatternChooser *pPatternChooser = new KoPatternChooser( KarbonFactory::rServer()->patterns(), mTabWidget );
@@ -91,13 +90,11 @@ VStyleDocker::VStyleDocker( KarbonPart* part, KarbonView* parent, const char* /*
 	ClipartWidget *pClipartWidget = new ClipartWidget( KarbonFactory::rServer()->cliparts(), part, mTabWidget );
 	mTabWidget->addTab( pClipartWidget, i18n( "Clipart" ) );
 
-	QVBoxLayout *mainWidgetLayout = new QVBoxLayout( mainWidget, 2 );
+	QVBoxLayout *mainWidgetLayout = new QVBoxLayout( this, 2 );
 	mainWidgetLayout->addWidget( mTabWidget );
 	mainWidgetLayout->activate();
-	mainWidget->setMinimumHeight( 174 );
-	mainWidget->setMinimumWidth( 194 );
-
-	setWidget( mainWidget );
+	setMinimumHeight( 174 );
+	setMinimumWidth( 194 );
 }
 
 VStyleDocker::~VStyleDocker()
@@ -136,7 +133,6 @@ ClipartWidget::ClipartWidget( QPtrList<VClipartIconItem>* clipartItems, KarbonPa
 	m_buttonGroup->insert( m_addClipartButton = new QToolButton( m_buttonGroup ) );
 	m_buttonGroup->insert( m_importClipartButton = new QToolButton( m_buttonGroup ) );
 	m_buttonGroup->insert( m_deleteClipartButton = new QToolButton( m_buttonGroup ) );
-	m_clipartChooser->setFixedSize( 180, 120 );
 	m_addClipartButton->setIconSet( SmallIcon( "14_layer_newlayer" ) );
 	m_addClipartButton->setTextLabel( i18n( "Add" ) );
 	m_importClipartButton->setIconSet( SmallIcon( "fileimport" ) );
