@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 
@@ -27,15 +27,15 @@
 #include <kdebug.h>
 #include <klocale.h>
 #include <ktempfile.h>
-#include <koTemplateChooseDia.h>
-#include <koStoreDevice.h>
-#include <koOasisStyles.h>
-#include <kooasiscontext.h>
-#include <koxmlwriter.h>
-#include <koxmlns.h>
-#include <kodom.h>
-#include <koOasisSettings.h>
-#include "kovariable.h"
+#include <KoTemplateChooseDia.h>
+#include <KoStoreDevice.h>
+#include <KoOasisStyles.h>
+#include <KoOasisLoadingContext.h>
+#include <KoXmlWriter.h>
+#include <KoXmlNS.h>
+#include <KoDom.h>
+#include <KoOasisSettings.h>
+#include <KoMainWindow.h>
 
 #include "karbon_factory.h"
 #include "karbon_part.h"
@@ -50,18 +50,19 @@
 #include "vlayer.h"
 #include "vdocumentdocker.h"
 #include "vtoolcontroller.h"
-#include "koApplication.h"
+#include "KoApplication.h"
+#include "vtool.h"
+#include "commands/vtransformcmd.h"
 
 // Make sure an appropriate DTD is available in www/koffice/DTD if changing this value
 // static const char * CURRENT_DTD_VERSION = "1.2";
 
 KarbonPart::KarbonPart( QWidget* parentWidget, const char* widgetName,
 						QObject* parent, const char* name, bool singleViewMode )
-		: KarbonPartBase( parentWidget, widgetName, parent, name, singleViewMode )
+		: KoDocument( parentWidget, widgetName, parent, name, singleViewMode )
 {
-	m_toolController = new VToolController( const_cast<KarbonPart *>( this ) );
-	m_toolController->init();
 	setInstance( KarbonFactory::instance(), false );
+	setTemplateType( "karbon_template" );
 	m_bShowStatusBar = true;
 	dcop = 0L;
 
@@ -77,6 +78,19 @@ KarbonPart::KarbonPart( QWidget* parentWidget, const char* widgetName,
 
 	//if( name )
         dcopObject();
+
+	// set as default paper
+	m_pageLayout.format = KoPageFormat::defaultFormat();
+	m_pageLayout.orientation = PG_PORTRAIT;
+	m_pageLayout.ptWidth = MM_TO_POINT( KoPageFormat::width( m_pageLayout.format, m_pageLayout.orientation ) );
+	m_pageLayout.ptHeight = MM_TO_POINT( KoPageFormat::height( m_pageLayout.format, m_pageLayout.orientation ) );
+	m_doc.setWidth( m_pageLayout.ptWidth );
+	m_doc.setHeight( m_pageLayout.ptHeight );
+	// enable selection drawing
+	m_doc.selection()->showHandle();
+	m_doc.selection()->setSelectObjects();
+	m_doc.selection()->setState( VObject::selected );
+	m_doc.selection()->selectNodes();
 }
 
 KarbonPart::~KarbonPart()
@@ -108,26 +122,18 @@ KarbonPart::initDoc(InitDocFlags flags, QWidget* parentWidget)
 {
         if (flags==KoDocument::InitDocEmpty)
         {
-                initUnit();
                 return true;
         }
 	QString file;
 	KoTemplateChooseDia::ReturnType result;
 
         KoTemplateChooseDia::DialogType dlgtype;
-        //if (flags != KoDocument::InitDocFileNew)
+        if (flags != KoDocument::InitDocFileNew)
             dlgtype = KoTemplateChooseDia::Everything;
-      //  else
-        //    dlgtype = KoTemplateChooseDia::OnlyTemplates;
+        else
+            dlgtype = KoTemplateChooseDia::OnlyTemplates;
 
 	result = KoTemplateChooseDia::choose( KarbonFactory::instance(), file, dlgtype, "karbon_template", parentWidget );
-	// set as default paper
-        m_pageLayout.format = KoPageFormat::defaultFormat();
-        m_pageLayout.orientation = PG_PORTRAIT;
-        m_pageLayout.ptWidth = MM_TO_POINT( KoPageFormat::width( m_pageLayout.format, m_pageLayout.orientation ) );
-        m_pageLayout.ptHeight = MM_TO_POINT( KoPageFormat::height( m_pageLayout.format, m_pageLayout.orientation ) );
-	m_doc.setWidth( m_pageLayout.ptWidth );
-	m_doc.setHeight( m_pageLayout.ptHeight );
 
 	if( result == KoTemplateChooseDia::Template )
 	{
@@ -135,13 +141,11 @@ KarbonPart::initDoc(InitDocFlags flags, QWidget* parentWidget)
 		bool ok = loadNativeFormat( file );
 		if ( !ok )
 			showLoadingErrorDialog();
-		initUnit();
 		setEmpty();
 		return ok;
 	}
 	else if( result == KoTemplateChooseDia::Empty )
 	{
-		initUnit();
 		return true;
 	}
 	else if( result == KoTemplateChooseDia::File )
@@ -157,7 +161,6 @@ KoView*
 KarbonPart::createViewInstance( QWidget* parent, const char* name )
 {
 	KarbonView *result = new KarbonView( this, parent, name );
-	toolController()->setActiveView( result );
 	return result;
 }
 
@@ -168,6 +171,24 @@ KarbonPart::removeView( KoView *view )
 	KoDocument::removeView( view );
 }
 
+double getAttribute(QDomElement &element, const char *attributeName, double defaultValue)
+{
+	QString value;
+	if ( ( value = element.attribute( attributeName ) ) != QString::null )
+		return value.toDouble();
+	else
+		return defaultValue;
+}
+
+int getAttribute(QDomElement &element, const char *attributeName, int defaultValue)
+{
+	QString value;
+	if ( ( value = element.attribute( attributeName ) ) != QString::null )
+		return value.toInt();
+	else
+		return defaultValue;
+}
+
 bool
 KarbonPart::loadXML( QIODevice*, const QDomDocument& document )
 {
@@ -183,8 +204,46 @@ KarbonPart::loadXML( QIODevice*, const QDomDocument& document )
 
 	success = m_doc.loadXML( doc );
 
-	m_pageLayout.ptWidth	= m_doc.width();
-	m_pageLayout.ptHeight	= m_doc.height();
+	//m_pageLayout = KoPageLayout::standardLayout();
+
+	// <PAPER>
+	QDomElement paper = doc.namedItem( "PAPER" ).toElement();
+	if ( !paper.isNull() )
+	{
+		m_pageLayout.format = static_cast<KoFormat>( getAttribute( paper, "format", 0 ) );
+		m_pageLayout.orientation = static_cast<KoOrientation>( getAttribute( paper, "orientation", 0 ) );
+
+		if( m_pageLayout.format == PG_CUSTOM )
+		{
+			m_pageLayout.ptWidth	= m_doc.width();
+			m_pageLayout.ptHeight	= m_doc.height();
+		}
+		else
+		{
+			m_pageLayout.ptWidth = getAttribute( paper, "width", 0.0 );
+			m_pageLayout.ptHeight = getAttribute( paper, "height", 0.0 );
+		}
+	}
+	else
+	{
+		m_pageLayout.ptWidth = getAttribute( doc, "width", 595.277);
+		m_pageLayout.ptHeight = getAttribute( doc, "height", 841.891 );
+	}
+
+	kdDebug() << " ptWidth=" << m_pageLayout.ptWidth << endl;
+	kdDebug() << " ptHeight=" << m_pageLayout.ptHeight << endl;
+        QDomElement borders = paper.namedItem( "PAPERBORDERS" ).toElement();
+        if( !borders.isNull() )
+       	{
+            if( borders.hasAttribute( "ptLeft" ) )
+                m_pageLayout.ptLeft = borders.attribute( "ptLeft" ).toDouble();
+            if( borders.hasAttribute( "ptTop" ) )
+                m_pageLayout.ptTop = borders.attribute( "ptTop" ).toDouble();
+            if( borders.hasAttribute( "ptRight" ) )
+                m_pageLayout.ptRight = borders.attribute( "ptRight" ).toDouble();
+            if( borders.hasAttribute( "ptBottom" ) )
+                m_pageLayout.ptBottom = borders.attribute( "ptBottom" ).toDouble();
+	}
 
 	setUnit( m_doc.unit() );
 
@@ -194,16 +253,38 @@ KarbonPart::loadXML( QIODevice*, const QDomDocument& document )
 QDomDocument
 KarbonPart::saveXML()
 {
-	return m_doc.saveXML();
+	QDomDocument doc = m_doc.saveXML();
+	QDomElement me = doc.documentElement();
+	QDomElement paper = doc.createElement( "PAPER" );
+	me.appendChild( paper );
+	paper.setAttribute( "format", static_cast<int>( m_pageLayout.format ) );
+	paper.setAttribute( "pages", pageCount() );
+	paper.setAttribute( "width", m_pageLayout.ptWidth );
+	paper.setAttribute( "height", m_pageLayout.ptHeight );
+	paper.setAttribute( "orientation", static_cast<int>( m_pageLayout.orientation ) );
+
+	QDomElement paperBorders = doc.createElement( "PAPERBORDERS" );
+	paperBorders.setAttribute( "ptLeft", m_pageLayout.ptLeft );
+	paperBorders.setAttribute( "ptTop", m_pageLayout.ptTop );
+	paperBorders.setAttribute( "ptRight", m_pageLayout.ptRight );
+	paperBorders.setAttribute( "ptBottom", m_pageLayout.ptBottom );
+	paper.appendChild(paperBorders);
+
+	return doc;
 }
 
 bool
 KarbonPart::loadOasis( const QDomDocument &doc, KoOasisStyles &styles, const QDomDocument &settings, KoStore *store )
 {
-	kdDebug(38000) << "Start loading OASIS document..." << endl;
+	kdDebug(38000) << "Start loading OASIS document..." << doc.toString() << endl;
 
 	QDomElement contents = doc.documentElement();
+	kdDebug(38000) << "Start loading OASIS document..." << contents.text() << endl;
+	kdDebug(38000) << "Start loading OASIS contents..." << contents.lastChild().localName() << endl;
+	kdDebug(38000) << "Start loading OASIS contents..." << contents.lastChild().namespaceURI() << endl;
+	kdDebug(38000) << "Start loading OASIS contents..." << contents.lastChild().isElement() << endl;
 	QDomElement body( KoDom::namedItemNS( contents, KoXmlNS::office, "body" ) );
+	kdDebug(38000) << "Start loading OASIS document..." << body.text() << endl;
 	if( body.isNull() )
 	{
 		kdDebug(38000) << "No office:body found!" << endl;
@@ -229,22 +310,30 @@ KarbonPart::loadOasis( const QDomDocument &doc, KoOasisStyles &styles, const QDo
 
 	QString masterPageName = "Standard"; // use default layout as fallback
 	QDomElement *master = styles.masterPages()[ masterPageName ];
+	if ( !master ) //last test...
+		master = styles.masterPages()[ "Default" ];
 	Q_ASSERT( master );
-	QDomElement *style =master ? styles.styles()[master->attributeNS( KoXmlNS::style, "page-layout-name", QString::null )] : 0;
+	const QDomElement *style = master ? styles.findStyle( master->attributeNS( KoXmlNS::style, "page-layout-name", QString::null ) ) : 0;
 	if( style )
 	{
-        	m_pageLayout.loadOasis( *style );
+		m_pageLayout.loadOasis( *style );
 		m_doc.setWidth( m_pageLayout.ptWidth );
 		m_doc.setHeight( m_pageLayout.ptHeight );
 	}
 	else
 		return false;
 
-	KoVariableFormatCollection varFormatColl;
-	KoVariableCollection varColl( new KoVariableSettings(), &varFormatColl );
-	KoOasisContext context( this, varColl, styles, store );
+	KoOasisLoadingContext context( this, styles, store );
 	m_doc.loadOasis( page, context );
-        loadOasisSettings( settings );
+	// do y-mirroring here
+	QWMatrix mat;
+	mat.scale( 1, -1 );
+	mat.translate( 0, -m_doc.height() );
+	VTransformCmd trafo( 0L, mat );
+	trafo.visit( m_doc );
+
+	loadOasisSettings( settings );
+
 	return true;
 }
 
@@ -258,7 +347,7 @@ KarbonPart::loadOasisSettings( const QDomDocument&settingsDoc )
     if ( !viewSettings.isNull() )
     {
         setUnit(KoUnit::unit(viewSettings.parseConfigItemString("unit")));
-        //todo add other config here.
+        // FIXME: add other config here.
     }
 }
 
@@ -277,7 +366,7 @@ KarbonPart::saveOasis( KoStore *store, KoXmlWriter *manifestWriter )
     QString layoutName = mainStyles.lookup( pageLayout, "PL" );
     KoGenStyle masterPage( KoGenStyle::STYLE_MASTER );
     masterPage.addAttribute( "style:page-layout-name", layoutName );
-    mainStyles.lookup( masterPage, "Standard", false );
+    mainStyles.lookup( masterPage, "Default", KoGenStyles::DontForceNumbering );
 
     KTempFile contentTmpFile;
     contentTmpFile.setAutoDelete( true );
@@ -301,15 +390,6 @@ KarbonPart::saveOasis( KoStore *store, KoXmlWriter *manifestWriter )
 
     docWriter->endElement(); // office:automatic-styles
 
-    styles = mainStyles.styles( KoGenStyle::STYLE_MASTER );
-    it = styles.begin();
-    docWriter->startElement("office:master-styles");
-
-    for( ; it != styles.end(); ++it)
-        (*it).style->writeStyle( docWriter, mainStyles, "style:master-page", (*it).name, "");
-
-    docWriter->endElement(); // office:master-styles
-
     // And now we can copy over the contents from the tempfile to the real one
     tmpFile->close();
     docWriter->addCompleteElement( tmpFile );
@@ -353,6 +433,15 @@ KarbonPart::saveOasis( KoStore *store, KoXmlWriter *manifestWriter )
 
     styleWriter->endElement(); // office:automatic-styles
 
+    styles = mainStyles.styles( KoGenStyle::STYLE_MASTER );
+    it = styles.begin();
+    styleWriter->startElement("office:master-styles");
+
+    for( ; it != styles.end(); ++it)
+        (*it).style->writeStyle( styleWriter, mainStyles, "style:master-page", (*it).name, "");
+
+    styleWriter->endElement(); // office:master-styles
+
     styleWriter->endElement(); // Root element
     styleWriter->endDocument();
     delete styleWriter;
@@ -537,51 +626,25 @@ KarbonPart::initConfig()
 		m_maxRecentFiles = config->readNumEntry( "NbRecentFile", 10 );
 		setShowStatusBar( config->readBoolEntry( "ShowStatusBar" , true ) );
 		setBackupFile( config->readNumEntry( "BackupFile", true ) );
+		m_doc.saveAsPath( config->readBoolEntry( "SaveAsPath", true ) );
 	}
 	int undos = 30;
 	if( config->hasGroup( "Misc" ) )
 	{
 		config->setGroup( "Misc" );
 		undos = config->readNumEntry( "UndoRedo", -1 );
+		QString defaultUnit = "cm";
+
+		if( KGlobal::locale()->measureSystem() == KLocale::Imperial )
+			defaultUnit = "in";
+
+		setUnit( KoUnit::unit( config->readEntry( "Units", defaultUnit ) ) );
+		m_doc.setUnit( unit() );
 	}
 	if( undos != -1 )
 		setUndoRedoLimit( undos );
 }
 
-void
-KarbonPart::initUnit()
-{
-	//load unit config after we load file.
-	//load it for new file or empty file
-	KConfig* config = KarbonPart::instance()->config();
-
-
-	if (KGlobal::locale()->measureSystem() == KLocale::Imperial) {
-		setUnit( KoUnit::U_INCH );
-	} else {
-		setUnit( KoUnit::U_CM );
-	}
-
-	if(config->hasGroup("Misc") ) {
-	        config->setGroup( "Misc" );
-		if ( config->hasKey( "Units" ) ) {
-		         setUnit( KoUnit::unit( config->readEntry("Units") ) );
-		}
-	}
-}
-
-void
-KarbonPart::setUnit( KoUnit::Unit _unit )
-{
-	m_doc.setUnit( _unit );
-	QPtrListIterator<KoView> itr( views() );
-
-	for( ; itr.current(); ++itr )
-	{
-		static_cast<KarbonView*>( itr.current() )->setUnit( _unit );
-	}
-}
-
 bool
 KarbonPart::mergeNativeFormat( const QString &file )
 {
@@ -593,5 +656,24 @@ KarbonPart::mergeNativeFormat( const QString &file )
 	return result;
 }
 
+void
+KarbonPart::addShell( KoMainWindow *shell )
+{
+	connect( shell, SIGNAL( documentSaved() ), m_commandHistory, SLOT( documentSaved() ) );
+	KoDocument::addShell( shell );
+}
+
+
+void
+KarbonPart::slotUnitChanged( KoUnit::Unit /*unit*/ )
+{
+#if 0
+	// VDocument has its own storage of the unit...
+	m_doc.setUnit( unit );
+	if( m_toolController->activeTool() )
+		m_toolController->activeTool()->refreshUnit();
+#endif
+}
+
 #include "karbon_part.moc"
 
