@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef __VKOPAINTER_H__
@@ -25,7 +25,7 @@
 #include "vpainter.h"
 #include <qwmatrix.h>
 #include <qptrlist.h>
-
+#include <koffice_export.h>
 class QPainter;
 struct _ArtVpath;
 struct _ArtBpath;
@@ -35,7 +35,7 @@ class VGradient;
 class VPattern;
 class KoRect;
 
-class VKoPainter : public VPainter
+class KARBONBASE_EXPORT VKoPainter : public VPainter
 {
 public:
 	VKoPainter( QPaintDevice *target, unsigned int w = 0, unsigned int h = 0, bool = true );
