@@ -13,20 +13,22 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 
 #include "vellipse.h"
 #include "vtransformcmd.h"
 #include <klocale.h>
-#include <koUnit.h>
-#include <koStore.h>
-#include <koxmlwriter.h>
+#include <KoUnit.h>
+#include <KoStore.h>
+#include <KoXmlWriter.h>
+#include <KoXmlNS.h>
 #include <vglobal.h>
 #include <vdocument.h>
 #include <qdom.h>
+#include <core/vfill.h>
 
 VEllipse::VEllipse( VObject* parent, VState state ) : VPath( parent, state )
 {
@@ -53,9 +55,9 @@ VEllipse::init()
 	// to radials
 	int nsegs;
 	if( m_startAngle < m_endAngle )
-	    nsegs = floor( ( m_endAngle - m_startAngle ) / 90.0 );
+	    nsegs = int( floor( ( m_endAngle - m_startAngle ) / 90.0 ) );
 	else
-	    nsegs = 4 - ceil( ( m_startAngle - m_endAngle ) / 90.0 );
+	    nsegs = 4 - int( ceil( ( m_startAngle - m_endAngle ) / 90.0 ) );
 	double startAngle = m_startAngle - 90.0;
 	if( startAngle < 0 ) startAngle += 360.0;
 	startAngle = VGlobal::pi_2 * ( startAngle / 90.0 );
@@ -96,8 +98,9 @@ VEllipse::init()
 	m.translate( m_center.x(), m_center.y() );
 	m.scale( 2.0 * m_rx, 2.0 * m_ry );
 
+	// only tranform the path data
 	VTransformCmd cmd( 0L, m );
-	cmd.visit( *this );
+	cmd.VVisitor::visitVPath( *this );
 
 	m_matrix.reset();
 }
@@ -112,7 +115,8 @@ VEllipse::name() const
 void
 VEllipse::save( QDomElement& element ) const
 {
-	if( document()->saveAsPath() )
+	VDocument *doc = document();
+	if( doc && doc->saveAsPath() )
 	{
 		VPath::save( element );
 		return;
@@ -123,8 +127,13 @@ VEllipse::save( QDomElement& element ) const
 		QDomElement me = element.ownerDocument().createElement( "ELLIPSE" );
 		element.appendChild( me );
 
-		VObject::save( me );
-
+		// save fill/stroke untransformed
+		VPath path( *this );
+		VTransformCmd cmd( 0L, m_matrix.invert() );
+		cmd.visit( path );
+		path.VObject::save( me );
+		//VObject::save( me );
+		
 		me.setAttribute( "cx", m_center.x() );
 		me.setAttribute( "cy", m_center.y() );
 
@@ -150,7 +159,7 @@ VEllipse::save( QDomElement& element ) const
 }
 
 void
-VEllipse::saveOasis( KoStore *store, KoXmlWriter *docWriter, KoGenStyles &mainStyles ) const
+VEllipse::saveOasis( KoStore *store, KoXmlWriter *docWriter, KoGenStyles &mainStyles, int &index ) const
 {
 	docWriter->startElement( "draw:ellipse" );
 
@@ -175,9 +184,13 @@ VEllipse::saveOasis( KoStore *store, KoXmlWriter *docWriter, KoGenStyles &mainSt
 		docWriter->addAttribute( "draw:end-angle", m_endAngle );
 	}
 
-	VObject::saveOasis( store, docWriter, mainStyles );
+	VObject::saveOasis( store, docWriter, mainStyles, index );
 
-	QString transform = buildSvgTransform();
+	QWMatrix tmpMat;
+	tmpMat.scale( 1, -1 );
+	tmpMat.translate( 0, -document()->height() );
+	
+	QString transform = buildOasisTransform( m_matrix*tmpMat );
 	if( !transform.isEmpty() )
 		docWriter->addAttribute( "draw:transform", transform );
 
@@ -185,33 +198,61 @@ VEllipse::saveOasis( KoStore *store, KoXmlWriter *docWriter, KoGenStyles &mainSt
 }
 
 bool
-VEllipse::loadOasis( const QDomElement &element, KoOasisContext &context )
+VEllipse::loadOasis( const QDomElement &element, KoOasisLoadingContext &context )
 {
 	setState( normal );
 
-	m_rx = KoUnit::parseValue( element.attribute( "svg:rx" ) );
-	m_ry = KoUnit::parseValue( element.attribute( "svg:ry" ) );
+	if( element.tagName() == "ellipse" )
+	{
+		if( element.hasAttributeNS( KoXmlNS::svg, "rx" ) )
+			m_rx = KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "rx", QString::null ) );
+		else 
+			m_rx = 0.5 * KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "width", QString::null ) );
 
-	m_center.setX( KoUnit::parseValue( element.attribute( "svg:cx" ) ) );
-	m_center.setY( KoUnit::parseValue( element.attribute( "svg:cy" ) ) );
+		if( element.hasAttributeNS( KoXmlNS::svg, "ry" ) )
+			m_ry = KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "ry", QString::null ) );
+		else 
+			m_ry = 0.5 * KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "height", QString::null ) );
 
-	m_startAngle = element.attribute( "draw:start-angle" ).toDouble();
-	m_endAngle = element.attribute( "draw:end-angle" ).toDouble();
+	}
+	else if( element.tagName() == "circle" )
+	{
+		if( element.hasAttributeNS( KoXmlNS::svg, "r" ) )
+			m_rx = m_ry = KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "r", QString::null ) );
+		else 
+			m_rx = m_ry = 0.5 * KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "width", QString::null ) );
+	}
 
-	if( element.attribute( "draw:kind" ) == "cut" )
+	if( element.hasAttributeNS( KoXmlNS::svg, "cx" ) )
+		m_center.setX( KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "cx", QString::null ) ) );
+	else
+		m_center.setX( m_rx + KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "x", QString::null ) ) );
+
+	if( element.hasAttributeNS( KoXmlNS::svg, "cy" ) )
+		m_center.setY( KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "cy", QString::null ) ) );
+	else
+		m_center.setY( m_ry + KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "y", QString::null ) ) );
+
+	m_startAngle = element.attributeNS( KoXmlNS::draw, "start-angle", QString::null ).toDouble();
+	m_endAngle = element.attributeNS( KoXmlNS::draw, "end-angle", QString::null ).toDouble();
+
+	QString kind = element.attributeNS( KoXmlNS::draw, "kind", QString::null );
+	if( kind == "cut" )
 		m_type = cut;
-	else if( element.attribute( "draw:kind" ) == "section" )
+	else if( kind == "section" )
 		m_type = section;
-	else if( element.attribute( "draw:kind" ) == "arc" )
+	else if( kind == "arc" )
 		m_type = arc;
 	else
 		m_type = full;
 
 	init();
 
-	QString trafo = element.attribute( "draw:transform" );
+	transformByViewbox( element, element.attributeNS( KoXmlNS::svg, "viewBox", QString::null ) );
+
+	QString trafo = element.attributeNS( KoXmlNS::draw, "transform", QString::null );
 	if( !trafo.isEmpty() )
-		transform( trafo );
+		transformOasis( trafo );
 
 	return VObject::loadOasis( element, context );
 }
@@ -251,3 +292,8 @@ VEllipse::load( const QDomElement& element )
 		transform( trafo );
 }
 
+VPath* 
+VEllipse::clone() const
+{
+	return new VEllipse( *this );
+}
