@@ -13,20 +13,22 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 
 #include "vrectangle.h"
 #include <klocale.h>
-#include <koUnit.h>
-#include <koStore.h>
-#include <koxmlwriter.h>
+#include <KoUnit.h>
+#include <KoStore.h>
+#include <KoXmlWriter.h>
+#include <KoXmlNS.h>
 #include <qdom.h>
 #include <kdebug.h>
 #include "vglobal.h"
 #include <vdocument.h>
+#include "vtransformcmd.h"
 
 VRectangle::VRectangle( VObject* parent, VState state )
 	: VPath( parent, state )
@@ -101,7 +103,8 @@ VRectangle::name() const
 void
 VRectangle::save( QDomElement& element ) const
 {
-	if( document()->saveAsPath() )
+	VDocument *doc = document();
+	if( doc && doc->saveAsPath() )
 	{
 		VPath::save( element );
 		return;
@@ -112,7 +115,12 @@ VRectangle::save( QDomElement& element ) const
 		QDomElement me = element.ownerDocument().createElement( "RECT" );
 		element.appendChild( me );
 
-		VObject::save( me );
+		// save fill/stroke untransformed
+		VPath path( *this );
+		VTransformCmd cmd( 0L, m_matrix.invert() );
+		cmd.visit( path );
+		path.VObject::save( me );
+		//VObject::save( me );
 
 		me.setAttribute( "x", m_topLeft.x() );
 		me.setAttribute( "y", m_topLeft.y() );
@@ -130,26 +138,30 @@ VRectangle::save( QDomElement& element ) const
 }
 
 void
-VRectangle::saveOasis( KoStore *store, KoXmlWriter *docWriter, KoGenStyles &mainStyles ) const
+VRectangle::saveOasis( KoStore *store, KoXmlWriter *docWriter, KoGenStyles &mainStyles, int &index ) const
 {
 	// different rx/ry is not supported by oasis, so act like it is a normal path
 	if( m_rx != 0. && m_ry != 0. && m_rx != m_ry )
-		return VPath::saveOasis( store, docWriter, mainStyles );
+		return VPath::saveOasis( store, docWriter, mainStyles, index );
 
 	docWriter->startElement( "draw:rect" );
 
 	//save all into pt
 	docWriter->addAttributePt( "svg:x",      m_topLeft.x() );
-	docWriter->addAttributePt( "svg:y",      m_topLeft.y() );
+	docWriter->addAttributePt( "svg:y",      m_topLeft.y()-m_height );
 	docWriter->addAttributePt( "svg:width",  m_width );
 	docWriter->addAttributePt( "svg:height", m_height );
 
 	if( m_rx != 0. && m_ry != 0. && m_rx == m_ry )
 		docWriter->addAttributePt( "draw:corner-radius", m_rx );
 
-	VObject::saveOasis( store, docWriter, mainStyles );
+	VObject::saveOasis( store, docWriter, mainStyles, index );
 
-	QString transform = buildSvgTransform();
+	QWMatrix tmpMat;
+	tmpMat.scale( 1, -1 );
+	tmpMat.translate( 0, -document()->height() );
+	
+	QString transform = buildOasisTransform( m_matrix*tmpMat );
 	if( !transform.isEmpty() )
 		docWriter->addAttribute( "draw:transform", transform );
 
@@ -157,24 +169,25 @@ VRectangle::saveOasis( KoStore *store, KoXmlWriter *docWriter, KoGenStyles &main
 }
 
 bool
-VRectangle::loadOasis( const QDomElement &element, KoOasisContext &context )
+VRectangle::loadOasis( const QDomElement &element, KoOasisLoadingContext &context )
 {
 	setState( normal );
 
-	m_width  = KoUnit::parseValue( element.attribute( "svg:width" ), 10.0 );
-	m_height = KoUnit::parseValue( element.attribute( "svg:height" ), 10.0 );
+	m_width  = KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "width", QString::null ), 10.0 );
+	m_height = KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "height", QString::null ), 10.0 );
 
-	m_topLeft.setX( KoUnit::parseValue( element.attribute( "svg:x" ) ) );
-	m_topLeft.setY( KoUnit::parseValue( element.attribute( "svg:y" ) ) );
+	m_topLeft.setX( KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "x", QString::null ) ) );
+	m_topLeft.setY( m_height + KoUnit::parseValue( element.attributeNS( KoXmlNS::svg, "y", QString::null ) ) );
 
-	m_rx  = KoUnit::parseValue( element.attribute( "svg:rx" ) );
-	m_ry  = KoUnit::parseValue( element.attribute( "svg:ry" ) );
+	m_rx = m_ry = KoUnit::parseValue( element.attributeNS( KoXmlNS::draw, "corner-radius", QString::null ) );
 
 	init();
 
-	QString trafo = element.attribute( "draw:transform" );
+	transformByViewbox( element, element.attributeNS( KoXmlNS::svg, "viewBox", QString::null ) );
+
+	QString trafo = element.attributeNS( KoXmlNS::draw, "transform", QString::null );
 	if( !trafo.isEmpty() )
-		transform( trafo );
+		transformOasis( trafo );
 
 	return VObject::loadOasis( element, context );
 }
@@ -205,3 +218,8 @@ VRectangle::load( const QDomElement& element )
 		transform( trafo );
 }
 
+VPath* 
+VRectangle::clone() const
+{
+	return new VRectangle( *this );
+}
