@@ -13,16 +13,17 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef __VSINUS_H__
 #define __VSINUS_H__
 
 #include "vcomposite.h"
+#include <koffice_export.h>
 
-class VSinus : public VPath
+class KARBONBASE_EXPORT VSinus : public VPath
 {
 public:
 	VSinus( VObject* parent, VState state = edit );
@@ -34,6 +35,8 @@ public:
 	virtual void save( QDomElement& element ) const;
 	virtual void load( const QDomElement& element );
 
+	virtual VPath* clone() const;
+
 protected:
 	void init();
 
