@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 
@@ -22,8 +22,8 @@
 #include <qlabel.h>
 
 #include <klocale.h>
-#include "koUnitWidgets.h"
-#include <koUnit.h>
+#include "KoUnitWidgets.h"
+#include <KoUnit.h>
 #include <kcombobox.h>
 #include <knuminput.h>
 
@@ -47,12 +47,10 @@ VEllipseOptionsWidget::VEllipseOptionsWidget( KarbonPart *part, QWidget *parent,
 	connect( m_type, SIGNAL( activated( int ) ), this, SLOT( typeChanged( int ) ) );
 
 	// add width/height-input:
-	new QLabel( i18n( "Width:" ), group );
-	m_width = new KDoubleSpinBox( 0.0, 1000.0, 0.5, 10.0, 2, group );
-	//m_width = new KoBuggyUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
-	new QLabel( i18n( "Height:" ), group );
-	m_height = new KDoubleSpinBox( 0.0, 1000.0, 0.5, 10.0, 2, group );
-	//m_height = new KoBuggyUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
+	m_widthLabel = new QLabel( i18n( "Width:" ), group );
+	m_width = new KoUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
+	m_heightLabel = new QLabel( i18n( "Height:" ), group );
+	m_height = new KoUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
 
 	new QLabel( i18n( "Start angle:" ), group );
 	m_startAngle = new KIntSpinBox( group );
@@ -115,27 +113,27 @@ VEllipseOptionsWidget::height() const
 void
 VEllipseOptionsWidget::setWidth( double value )
 {
-	m_width->setValue( value );
+	m_width->changeValue( value );
 }
 
 void
 VEllipseOptionsWidget::setHeight( double value )
 {
-	m_height->setValue( value );
+	m_height->changeValue( value );
 }
 
 void
 VEllipseOptionsWidget::refreshUnit ()
 {
-	m_width->setSuffix( KoUnit::unitName( m_part->unit() ) );
-	m_height->setSuffix( KoUnit::unitName( m_part->unit() ) );
+	m_width->setUnit( m_part->unit() );
+	m_height->setUnit( m_part->unit() );
 }
 
-VEllipseTool::VEllipseTool( KarbonPart *part )
-	: VShapeTool( part, i18n( "Insert Ellipse" ) )
+VEllipseTool::VEllipseTool( KarbonView *view )
+	: VShapeTool( view, "tool_ellipse" )
 {
 	// create config dialog:
-	m_optionsWidget = new VEllipseOptionsWidget( part );
+	m_optionsWidget = new VEllipseOptionsWidget( view->part() );
 	registerTool( this );
 
 	m_startAngle = m_endAngle = 0;
@@ -158,8 +156,8 @@ VEllipseTool::shape( bool interactive ) const
 {
 	if( interactive )
 	{
-		double d1 = KoUnit::fromUserValue( m_optionsWidget->width(), view()->part()->unit() ) / 2.0;
-		double d2 = KoUnit::fromUserValue( m_optionsWidget->height(), view()->part()->unit() ) / 2.0;
+		double d1 = m_optionsWidget->width() / 2.0;
+		double d2 = m_optionsWidget->height() / 2.0;
 		return
 			new VEllipse(
 				0L,
@@ -263,5 +261,19 @@ VEllipseTool::showDialog() const
 	return m_optionsWidget->exec() == QDialog::Accepted;
 }
 
+void
+VEllipseTool::setup( KActionCollection *collection )
+{
+	m_action = static_cast<KRadioAction *>(collection -> action( name() ) );
+
+	if( m_action == 0 )
+	{
+		m_action = new KRadioAction( i18n( "Ellipse Tool" ), "14_ellipse", Qt::SHIFT+Qt::Key_H, this, SLOT( activate() ), collection, name() );
+		m_action->setToolTip( i18n( "Ellipse" ) );
+		m_action->setExclusiveGroup( "shapes" );
+		//m_ownAction = true;
+	}
+}
+
 #include "vellipsetool.moc"
 
