@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qlabel.h>
@@ -27,21 +27,20 @@
 #include <karbon_part.h>
 #include <shapes/vrectangle.h>
 #include "vrectangletool.h"
-#include <koUnitWidgets.h>
+#include <KoUnitWidgets.h>
 
 
-VRectangleTool::VRectangleOptionsWidget::VRectangleOptionsWidget( KarbonPart*part, QWidget* parent, const char* name )
-	: KDialogBase( parent, name, true, i18n( "Insert Rectangle" ), Ok | Cancel ), m_part(part)
+VRectangleTool::VRectangleOptionsWidget::VRectangleOptionsWidget( KarbonPart *part, QWidget* parent, const char* name )
+	: KDialogBase( parent, name, true, i18n( "Insert Rectangle" ), Ok | Cancel ), m_part( part )
 {
 	QGroupBox *group = new QGroupBox( 2, Qt::Horizontal, i18n( "Properties" ), this );
 	// add width/height-input:
-	new QLabel( i18n( "Width:" ), group );
-	m_width = new KDoubleSpinBox( 0.0, 1000.0, 0.5, 10.0, 2, group );
-	//m_width = new KoBuggyUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
+	m_widthLabel = new QLabel( i18n( "Width:" ), group );
+	m_width = new KoUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
+
+	m_heightLabel = new QLabel( i18n( "Height:" ), group );
+	m_height = new KoUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
 
-	new QLabel( i18n( "Height:" ), group );
-	m_height = new KDoubleSpinBox( 0.0, 1000.0, 0.5, 10.0, 2, group );
-	//m_height = new KoBuggyUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
 	refreshUnit();
 
 	group->setInsideMargin( 4 );
@@ -78,15 +77,15 @@ VRectangleTool::VRectangleOptionsWidget::setHeight( double value )
 void
 VRectangleTool::VRectangleOptionsWidget::refreshUnit()
 {
-	m_width->setSuffix( KoUnit::unitName( m_part->unit() ) );
-	m_height->setSuffix( KoUnit::unitName( m_part->unit() ) );
+	m_width->setUnit( m_part->unit() );
+	m_height->setUnit( m_part->unit() );
 }
 
-VRectangleTool::VRectangleTool( KarbonPart *part )
-	: VShapeTool( part, i18n( "Insert Rectangle" ) )
+VRectangleTool::VRectangleTool( KarbonView *view )
+	: VShapeTool( view, "tool_rectangle" )
 {
 	// Create config dialog:
-	m_optionWidget = new VRectangleOptionsWidget( part );
+	m_optionWidget = new VRectangleOptionsWidget( view->part() );
 	registerTool( this );
 }
 
@@ -110,8 +109,8 @@ VRectangleTool::shape( bool interactive ) const
 			new VRectangle(
 				0L,
 				m_p,
-				KoUnit::fromUserValue( m_optionWidget->width(), view()->part()->unit() ),
-				KoUnit::fromUserValue( m_optionWidget->height(), view()->part()->unit() ) );
+				m_optionWidget->width(),
+				m_optionWidget->height() );
 	}
 	else
 		return
@@ -128,3 +127,17 @@ VRectangleTool::showDialog() const
 	return m_optionWidget->exec() == QDialog::Accepted;
 }
 
+void
+VRectangleTool::setup( KActionCollection *collection )
+{
+	m_action = static_cast<KRadioAction *>(collection -> action( name() ) );
+
+	if( m_action == 0 )
+	{
+		m_action = new KRadioAction( i18n( "Rectangle Tool" ), "14_rectangle", Qt::Key_Plus+Qt::Key_F9, this, SLOT( activate() ), collection, name() );
+		m_action->setToolTip( i18n( "Rectangle" ) );
+		m_action->setExclusiveGroup( "shapes" );
+		//m_ownAction = true;
+	}
+}
+
