@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 
@@ -29,7 +29,7 @@
 #include <karbon_part.h>
 #include <shapes/vsinus.h>
 #include "vsinustool.h"
-#include "koUnitWidgets.h"
+#include "KoUnitWidgets.h"
 
 
 VSinusTool::VSinusOptionsWidget::VSinusOptionsWidget( KarbonPart *part, QWidget* parent, const char* name )
@@ -38,12 +38,10 @@ VSinusTool::VSinusOptionsWidget::VSinusOptionsWidget( KarbonPart *part, QWidget*
 	QGroupBox *group = new QGroupBox( 2, Qt::Horizontal, i18n( "Properties" ), this );
 
 	// add width/height-input:
-	new QLabel( i18n( "Width:" ), group );
-	m_width = new KDoubleSpinBox( 0.0, 1000.0, 0.5, 10.0, 2, group );
-	//m_width = new KoBuggyUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
-	new QLabel( i18n( "Height:" ), group );
-	m_height = new KDoubleSpinBox( 0.0, 1000.0, 0.5, 10.0, 2, group );
-	//m_height = new KoBuggyUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
+	m_widthLabel = new QLabel( i18n( "Width:" ), group );
+	m_width = new KoUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
+	m_heightLabel = new QLabel( i18n( "Height:" ), group );
+	m_height = new KoUnitDoubleSpinBox( group, 0.0, 1000.0, 0.5, 100.0, KoUnit::U_MM );
 
 	refreshUnit();
 
@@ -79,13 +77,13 @@ VSinusTool::VSinusOptionsWidget::periods() const
 void
 VSinusTool::VSinusOptionsWidget::setWidth( double value )
 {
-	m_width->setValue( value );
+	m_width->changeValue( value );
 }
 
 void
 VSinusTool::VSinusOptionsWidget::setHeight( double value )
 {
-	m_height->setValue( value );
+	m_height->changeValue( value );
 }
 
 void
@@ -97,15 +95,15 @@ VSinusTool::VSinusOptionsWidget::setPeriods( uint value )
 void
 VSinusTool::VSinusOptionsWidget::refreshUnit ()
 {
-	m_width->setSuffix( KoUnit::unitName( m_part->unit() ) );
-	m_height->setSuffix( KoUnit::unitName( m_part->unit() ) );
+	m_width->setUnit( m_part->unit() );
+	m_height->setUnit( m_part->unit() );
 }
 
-VSinusTool::VSinusTool( KarbonPart *part )
-	: VShapeTool( part, i18n( "Insert Sinus" ) )
+VSinusTool::VSinusTool( KarbonView *view )
+	: VShapeTool( view, "tool_sinus" )
 {
 	// create config widget:
-	m_optionsWidget = new VSinusOptionsWidget( part );
+	m_optionsWidget = new VSinusOptionsWidget( view->part() );
 	m_optionsWidget->setPeriods( 1 );
 	registerTool( this );
 }
@@ -129,8 +127,8 @@ VSinusTool::shape( bool interactive ) const
 			new VSinus(
 				0L,
 				m_p,
-				KoUnit::fromUserValue( m_optionsWidget->width(), view()->part()->unit() ),
-				KoUnit::fromUserValue( m_optionsWidget->height(), view()->part()->unit() ),
+				m_optionsWidget->width(),
+				m_optionsWidget->height(),
 				m_optionsWidget->periods() );
 	else
 		return
@@ -148,3 +146,17 @@ VSinusTool::showDialog() const
 	return m_optionsWidget->exec() == QDialog::Accepted;
 }
 
+void
+VSinusTool::setup( KActionCollection *collection )
+{
+	m_action = static_cast<KRadioAction *>(collection -> action( name() ) );
+
+	if( m_action == 0 )
+	{
+		m_action = new KRadioAction( i18n( "Sinus Tool" ), "14_sinus", Qt::SHIFT+Qt::Key_S, this, SLOT( activate() ), collection, name() );
+		m_action->setToolTip( i18n( "Sinus" ) );
+		m_action->setExclusiveGroup( "shapes" );
+		//m_ownAction = true;
+	}
+}
+
