@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 /* vselecttoolbar.h */
@@ -24,7 +24,7 @@
 
 #include <ktoolbar.h>
 
-class KoBuggyUnitDoubleSpinBox;
+class KoUnitDoubleSpinBox;
 class KarbonView;
 
 class VSelectToolBar : public KToolBar
@@ -42,10 +42,10 @@ public slots:
 	void slotHeightChanged( double );
 
 private:
-	KoBuggyUnitDoubleSpinBox *m_x;
-	KoBuggyUnitDoubleSpinBox *m_y;
-	KoBuggyUnitDoubleSpinBox *m_width;
-	KoBuggyUnitDoubleSpinBox *m_height;
+	KoUnitDoubleSpinBox *m_x;
+	KoUnitDoubleSpinBox *m_y;
+	KoUnitDoubleSpinBox *m_width;
+	KoUnitDoubleSpinBox *m_height;
 	KarbonView			*m_view;
 };
 
