@@ -3,7 +3,7 @@
    */
 
 /****************************************************************************
- ** Copyright (C) 2001-2003 Klarlvdalens Datakonsult AB.  All rights reserved.
+ ** Copyright (C) 2001-2003 Klarälvdalens Datakonsult AB.  All rights reserved.
  **
  ** This file is part of the KDChart library.
  **
@@ -39,7 +39,7 @@
   This exception is thrown when it is attempted to create a chart of
   non-registered user-defined type.
   */
-class KDChartUnknownTypeException
+class KDCHART_EXPORT KDChartUnknownTypeException
 {
     public:
         KDChartUnknownTypeException( const QString& type ) :
