@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qdom.h>
@@ -26,13 +26,18 @@
 #include <klibloader.h>
 #include <kparts/componentfactory.h>
 #include <ktrader.h>
+#include <kiconloader.h>
+#include <kpopupmenu.h>
 
 #include "widgetfactory.h"
 #include "widgetlibrary.h"
 #include "libactionwidget.h"
+#include "container.h"
 #include "form.h"
+#include "formIO.h"
 
 namespace KFormDesigner {
+//! @internal
 class WidgetLibraryPrivate
 {
 	public:
@@ -182,20 +187,27 @@ WidgetLibrary::lookupFactories()
 		KService::Ptr ptr = (*it);
 		KService::Ptr* existingService = (d->services)[ptr->library().latin1()];
 		if (existingService) {
-			kdWarning() << "WidgetLibrary::scan(): factory '" << ptr->name()
+			kdWarning() << "WidgetLibrary::lookupFactories(): factory '" << ptr->name()
 				<< "' already found (library="<< (*existingService)->library()
 				<<")! skipping this one: library=" << ptr->library() << endl;
 			continue;
 		}
-		kdDebug() << "WidgetLibrary::scan(): found factory: " << ptr->name() << endl;
+		kdDebug() << "WidgetLibrary::lookupFactories(): found factory: " << ptr->name() << endl;
 
 		QCString groupName = ptr->property("X-KFormDesigner-FactoryGroup").toCString();
 		if (!groupName.isEmpty() && !d->supportedFactoryGroups[groupName]) {
-			kdDebug() << "WidgetLibrary::scan(): factory group '" << groupName
+			kdDebug() << "WidgetLibrary::lookupFactories(): factory group '" << groupName
 				<< "' is unsupported by this application (library=" << ptr->library() << ")"<< endl;
 			continue;
 		}
-		//FIXME: check if this name matches the filter...
+		const uint factoryVersion = ptr->property("X-KFormDesigner-WidgetFactoryVersion").toUInt();
+		if (KFormDesigner::version()!=factoryVersion) {
+			kdWarning() << QString("WidgetLibrary::lookupFactories(): factory '%1'" 
+				" has version '%2' but required Widget Factory version is '%3'\n"
+				" -- skipping this factory!").arg(ptr->library()).arg(factoryVersion)
+				.arg(KFormDesigner::version()) << endl;
+			continue;
+		}
 		d->services.insert(ptr->library().latin1(), new KService::Ptr( ptr ));
 	}
 }
@@ -214,6 +226,7 @@ WidgetLibrary::loadFactories()
 				<< (*it.current())->library() << endl;
 			continue;
 		}
+		f->m_library = this;
 		f->m_showAdvancedProperties = d->showAdvancedProperties; //inherit this flag from the library
 		d->factories.insert( f->name(), f );
 		//collect information about classes to be hidden
@@ -294,7 +307,6 @@ WidgetLibrary::addCreateWidgetActions(KActionCollection *parent,  QObject *recei
 	ActionList actions;
 	for (QAsciiDictIterator<WidgetInfo> it(d->widgets); it.current(); ++it)
 	{
-//		kdDebug() << "WidgetLibrary::createActions():" << it.current()->className() << endl;
 		LibActionWidget *a = new LibActionWidget(it.current(), parent);
 		connect(a, SIGNAL(prepareInsert(const QCString &)), receiver, slot);
 		actions.append(a);
@@ -312,24 +324,26 @@ WidgetLibrary::addCustomWidgetActions(KActionCollection *parent)
 }
 
 QWidget*
-WidgetLibrary::createWidget(const QCString &classname, QWidget *parent, const char *name, Container *c)
+WidgetLibrary::createWidget(const QCString &classname, QWidget *parent, const char *name, Container *c,
+	int options)
 {
 	loadFactories();
 	WidgetInfo *wclass = d->widgets[classname];
 	if(!wclass)
 		return 0;
 
-	QWidget *widget = wclass->factory()->create(wclass->className(), parent, name, c);
-	if (widget)
-		return widget;
-	//try to instantiate from inherited class
-	if (wclass->inheritedClass())
-		return wclass->inheritedClass()->factory()->create(wclass->className(), parent, name, c);
-	return 0;
+	QWidget *widget = wclass->factory()->createWidget(wclass->className(), parent, name, c, options);
+	if (!widget) {
+		//try to instantiate from inherited class
+		if (wclass->inheritedClass())
+			widget = wclass->inheritedClass()->factory()->createWidget(
+				wclass->className(), parent, name, c, options);
+	}
+	return widget;
 }
 
 bool
-WidgetLibrary::createMenuActions(const QCString &c, QWidget *w, QPopupMenu *menu, 
+WidgetLibrary::createMenuActions(const QCString &c, QWidget *w, QPopupMenu *menu,
 	KFormDesigner::Container *container)
 {
 	loadFactories();
@@ -445,16 +459,14 @@ WidgetLibrary::textForWidgetName(const QCString &name, const QCString &className
 }
 
 QCString
-WidgetLibrary::checkAlternateName(const QCString &classname)
+WidgetLibrary::classNameForAlternate(const QCString &classname)
 {
 	loadFactories();
 	if(d->widgets.find(classname))
 		return classname;
 
-//	WidgetInfo *wi =  d->alternateWidgets[classname];
 	WidgetInfo *wi =  d->widgets[classname];
 	if (wi) {
-//		kdDebug() << "WidgetLibrary::alternateName() : The name " << classname << " will be replaced with " << wi->className() << endl;
 		return wi->className();
 	}
 
@@ -474,14 +486,14 @@ WidgetLibrary::includeFileName(const QCString &classname)
 }
 
 QString
-WidgetLibrary::icon(const QCString &classname)
+WidgetLibrary::iconName(const QCString &classname)
 {
 	loadFactories();
 	WidgetInfo *wi = d->widgets.find(classname);
 	if(wi)
 		return wi->pixmap();
 
-	return QString("form");
+	return QString::fromLatin1("unknown_widget");
 }
 
 bool
@@ -526,26 +538,33 @@ bool WidgetLibrary::advancedPropertiesVisible() const
 }
 
 bool
-WidgetLibrary::isPropertyVisible(const QCString &classname, QWidget *w, 
-	const QCString &property, bool multiple)
+WidgetLibrary::isPropertyVisible(const QCString &classname, QWidget *w,
+	const QCString &property, bool multiple, bool isTopLevel)
 {
-	if (dynamic_cast<FormWidget*>(w)) {
+	if (isTopLevel) {
 		// no focus policy for top-level form widget...
 		if (!d->showAdvancedProperties && property == "focusPolicy")
 			return false;
 	}
-	if (!d->showAdvancedProperties && d->advancedProperties[ property ])
-		return false;
 
 	loadFactories();
 	WidgetInfo *wi = d->widgets.find(classname);
 	if (!wi)
 		return false;
-	if (!wi->factory()->isPropertyVisible(classname, w, property, multiple))
+	if (!d->showAdvancedProperties && d->advancedProperties[ property ]) {
+		//this is advanced property, should we hide it?
+		if (wi->factory()->internalProperty(classname, "forceShowAdvancedProperty:"+property).isEmpty()
+			&& (!wi->inheritedClass() || wi->inheritedClass()->factory()->internalProperty(classname, "forceShowAdvancedProperty:"+property).isEmpty()))
+		{
+			return false; //hide it
+		}
+	}
+
+	if (!wi->factory()->isPropertyVisible(classname, w, property, multiple, isTopLevel))
 		return false;
 	//try from inherited class
-	if (wi->inheritedClass() 
-		&& !wi->inheritedClass()->factory()->isPropertyVisible(wi->className(), w, property, multiple))
+	if (wi->inheritedClass()
+		&& !wi->inheritedClass()->factory()->isPropertyVisible(wi->className(), w, property, multiple, isTopLevel))
 		return false;
 
 	return true;
@@ -582,7 +601,7 @@ WidgetLibrary::factoryForClassName(const char* classname)
 
 QString WidgetLibrary::propertyDescForName(WidgetInfo *winfo, const QCString& propertyName)
 {
-	if (!winfo->factory())
+	if (!winfo || !winfo->factory())
 		return QString::null;
 	QString desc( winfo->factory()->propertyDescForName(propertyName) );
 	if (!desc.isEmpty())
@@ -590,11 +609,11 @@ QString WidgetLibrary::propertyDescForName(WidgetInfo *winfo, const QCString& pr
 	if (winfo->m_parentFactoryName.isEmpty())
 		return QString::null;
 
-	//try in parent factory, if exists	
+	//try in parent factory, if exists
 	WidgetFactory *parentFactory = d->factories[winfo->m_parentFactoryName];
 	if (!parentFactory)
 		return QString::null;
-	
+
 	return parentFactory->propertyDescForName(propertyName);
 }
 
@@ -608,15 +627,15 @@ QString WidgetLibrary::propertyDescForValue(WidgetInfo *winfo, const QCString& n
 	if (winfo->m_parentFactoryName.isEmpty())
 		return QString::null;
 
-	//try in parent factory, if exists	
+	//try in parent factory, if exists
 	WidgetFactory *parentFactory = d->factories[winfo->m_parentFactoryName];
 	if (!parentFactory)
 		return QString::null;
-	
+
 	return parentFactory->propertyDescForValue(name);
 }
 
-void WidgetLibrary::setPropertyOptions( KexiPropertyBuffer& buf, const WidgetInfo& winfo, QWidget* w )
+void WidgetLibrary::setPropertyOptions( WidgetPropertySet& buf, const WidgetInfo& winfo, QWidget* w )
 {
 	if (!winfo.factory())
 		return;
@@ -629,4 +648,74 @@ void WidgetLibrary::setPropertyOptions( KexiPropertyBuffer& buf, const WidgetInf
 	parentFactory->setPropertyOptions(buf, winfo, w);
 }
 
+WidgetFactory* WidgetLibrary::factory(const char* factoryName) const
+{
+	return d->factories[factoryName];
+}
+
+QString WidgetLibrary::internalProperty(const QCString& classname, const QCString& property)
+{
+	loadFactories();
+	WidgetInfo *wclass = d->widgets[classname];
+	if(!wclass)
+		return QString::null;
+	QString value( wclass->factory()->internalProperty(classname, property) );
+	if (value.isEmpty() && wclass->inheritedClass())
+		return wclass->inheritedClass()->factory()->internalProperty(classname, property);
+	return value;
+}
+
+WidgetFactory::CreateWidgetOptions WidgetLibrary::showOrientationSelectionPopup(
+	const QCString &classname, QWidget* parent, const QPoint& pos)
+{
+	loadFactories();
+	WidgetInfo *wclass = d->widgets[classname];
+	if(!wclass)
+		return WidgetFactory::AnyOrientation;
+
+	//get custom icons and strings
+	QPixmap iconHorizontal, iconVertical;
+	QString iconName( wclass->factory()->internalProperty(classname, "orientationSelectionPopup:horizontalIcon") );
+	if (iconName.isEmpty() && wclass->inheritedClass())
+		iconName = wclass->inheritedClass()->factory()->internalProperty(classname, "orientationSelectionPopup:horizontalIcon");
+	if (!iconName.isEmpty())
+		iconHorizontal = SmallIcon(iconName);
+
+	iconName = wclass->factory()->internalProperty(classname, "orientationSelectionPopup:verticalIcon");
+	if (iconName.isEmpty() && wclass->inheritedClass())
+		iconName = wclass->inheritedClass()->factory()->internalProperty(classname, "orientationSelectionPopup:verticalIcon");
+	if (!iconName.isEmpty())
+		iconVertical = SmallIcon(iconName);
+
+	QString textHorizontal = wclass->factory()->internalProperty(classname, "orientationSelectionPopup:horizontalText");
+	if (textHorizontal.isEmpty() && wclass->inheritedClass())
+		iconName = wclass->inheritedClass()->factory()->internalProperty(classname, "orientationSelectionPopup:horizontalText");
+	if (textHorizontal.isEmpty()) //default
+		textHorizontal = i18n("Insert Horizontal Widget", "Insert Horizontal");
+
+	QString textVertical = wclass->factory()->internalProperty(classname, "orientationSelectionPopup:verticalText");
+	if (textVertical.isEmpty() && wclass->inheritedClass())
+		iconName = wclass->inheritedClass()->factory()->internalProperty(classname, "orientationSelectionPopup:verticalText");
+	if (textVertical.isEmpty()) //default
+		textVertical = i18n("Insert Vertical Widget", "Insert Vertical");
+
+	KPopupMenu* popup = new KPopupMenu(parent, "orientationSelectionPopup");
+	popup->insertTitle(SmallIcon(wclass->pixmap()), i18n("Insert Widget: %1").arg(wclass->name()));
+	popup->insertItem(iconHorizontal, textHorizontal, 1);
+	popup->insertItem(iconVertical, textVertical, 2);
+	popup->insertSeparator();
+	popup->insertItem(SmallIcon("button_cancel"), i18n("Cancel"), 3);
+	WidgetFactory::CreateWidgetOptions result;
+	switch (popup->exec(pos)) {
+	case 1:
+		result = WidgetFactory::HorizontalOrientation; break;
+	case 2:
+		result = WidgetFactory::VerticalOrientation; break;
+	default:
+		result = WidgetFactory::AnyOrientation; //means "cancelled"
+	}
+	delete popup;
+	return result;
+}
+
 #include "widgetlibrary.moc"
