@@ -16,26 +16,26 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef _KEXI_VERSION_
 #define _KEXI_VERSION_
 
-#ifdef OOPL_VERSION /* user-friendly version info */
-# include "oopl_global.h"
+#ifdef CUSTOM_VERSION /* user-friendly version info */
+# include "custom_global.h"
 #else /* default */
 # define KEXI_APP_NAME "Kexi"
 #endif
 
 #ifndef KEXI_VERSION_STRING
-# define KEXI_VERSION_STRING "0.8.1"
+# define KEXI_VERSION_STRING "1.0 rc1"
 #endif
 
 #define KEXI_VERSION_MAJOR 0
-#define KEXI_VERSION_MINOR 8
-#define KEXI_VERSION_RELEASE 1
+#define KEXI_VERSION_MINOR 9
+#define KEXI_VERSION_RELEASE 98
 
 #define KEXI_MAKE_VERSION( a,b,c ) (((a) << 16) | ((b) << 8) | (c))
 
@@ -84,13 +84,13 @@ namespace Kexi
 }
 
 /*
- * this is the version a part has to be only increase it if the 
- * interface isn't binary compatible anymore
- 
- * @todo
+ * This is the version a part has to be only increase it if the 
+ * interface isn't binary compatible anymore.
+ * 
+ * Note: update X-Kexi-PartVersion values in kexi*handler.desktop 
+ * files everytime you are increasing this value.
  */
  
-#define KEXI_PART_VERSION 1
+#define KEXI_PART_VERSION 2
 
 #endif /* _KEXI_VERSION_ */
-
