@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2003-2004 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2003-2006 Jaroslaw Staniek <js@iidea.pl>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef KEXIDB_CONNECTION_H
@@ -36,8 +36,9 @@
 #include <kexidb/queryschema.h>
 #include <kexidb/transaction.h>
 #include <kexidb/driver.h>
+#include <kexidb/preparedstatement.h>
 
-#include <tristate.h>
+#include <kexiutils/tristate.h>
 
 namespace KexiDB {
 
@@ -47,9 +48,13 @@ typedef QValueVector<QVariant> RowData;
 class Cursor;
 class ConnectionPrivate;
 class RowEditBuffer;
+class DatabaseProperties;
 
-//! Database connection API, allowing queries and updates.
-/*! This class represents a database connection established with given database source.
+/*! @short Provides database connection, allowing queries and data modification.
+
+ This class represents a database connection established within a data source.
+ It supports data queries and modification by creating client-side database cursors.
+ Database transactions are supported.
 */
 class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 {
@@ -64,7 +69,7 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		virtual ~Connection();
 
 		/*! \return parameters that were used to create this connection. */
-		ConnectionData* data() { return m_data; }
+		ConnectionData* data() const { return m_data; }
 
 		/*! \return the driver used for this connection. */
 		Driver* driver() const { return m_driver; }
@@ -75,12 +80,17 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		bool connect();
 
 		/*! \return true, if connection is properly established. */
-		bool isConnected() { return m_is_connected; }
+		bool isConnected() const;
 
 		/*! \return true, both if connection is properly established
 		 and any database within this connection is properly used
 		 with useDatabase(). */
-		bool isDatabaseUsed();
+		bool isDatabaseUsed() const;
+
+		/*! \return true for read only connection. Used especially for file-based drivers. 
+		 Can be reimplemented in a driver to provide real read-only flag of the connection 
+		 (SQlite3 dirver does this). */
+		virtual bool isReadOnly() const;
 
 		/*! Reimplemented from Object: also clears sql string.
 		 @sa recentSQLString() */
@@ -110,6 +120,10 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 false is returned. 
 		 For file-based drivers, \a dbName should be equal to the database
 		 filename (the same as specified for ConnectionData).
+
+		 See doc/dev/kexidb_issues.txt document, chapter "Table schema, query schema, etc. storage"
+		 for database schema documentation (detailed description of kexi__* 'system' tables).
+
 		 \sa useDatabase() */
 		bool createDatabase( const QString &dbName );
 
@@ -120,8 +134,11 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 to recognize database Kexi-specific format. Set \a kexiCompatible to false
 		 if you're using native database (one that have no Kexi System tables).
 		 For file-based drivers, \a dbName should be equal to filename
-		 (the same as specified for ConnectionData). */
-		bool useDatabase( const QString &dbName, bool kexiCompatible = true );
+		 (the same as specified for ConnectionData). 
+		 \return true on success, false on failure. 
+		 If user has cancelled this action and \a cancelled is not 0, *cancelled is set to true. */
+		bool useDatabase( const QString &dbName, bool kexiCompatible = true, bool *cancelled = 0, 
+			MessageHandler* msgHandler = 0 );
 
 		/*! 
 		\brief Closes currently used database for this connection.
@@ -143,12 +160,19 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		*/
 		bool dropDatabase( const QString &dbName = QString::null );
 
+		 /*! \return names of all the \a objecttype (see \a ObjectTypes in global.h)
+		 schemas stored in currently used database. KexiDB::AnyObjectType can be passed 
+		 as \a objType to get names of objects of any type.
+		 If \a ok is not null then variable pointed by it will be set to the result. 
+		 On error, the functions can return incomplete list. */
+		QStringList objectNames(int objType = KexiDB::AnyObjectType, bool* ok = 0);
+
 		/*! \return names of all table schemas stored in currently 
 		 used database. If \a also_system_tables is true, 
 		 internal KexiDB system table names (kexi__*) are also returned.
 		 \sa kexiDBSystemTableNames() */
 		QStringList tableNames(bool also_system_tables = false);
-		
+
 		/*! \return list of internal KexiDB system table names 
 		 (kexi__*). This does not mean that these tables can be found
 		 in currently opened database. Just static list of table 
@@ -165,6 +189,10 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		int versionMajor() const;
 		int versionMinor() const;
 
+		/*! \return DatabaseProperties obejct allowing to read and write global database properties 
+		 for this connection. */
+		DatabaseProperties& databaseProperties();
+
 		/*! \return ids of all table schema names stored in currently 
 		 used database. These ids can be later used as argument for tableSchema().
 		 This is a shortcut for objectIds(TableObjectType).
@@ -367,6 +395,17 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 used database.  \sa querySchema( int queryId ) */
 		QuerySchema* querySchema( const QString& queryName );
 
+		/*! Sets \a queryName query obsolete by moving it out of the query sets, so it will not be 
+		 accessible by querySchema( const QString& queryName ). The existing query object is not 
+		 destroyed, to avoid problems when it's referenced. In this case, 
+		 a new query schema will be retrieved directly from the backend.
+
+		 For now it's used in KexiQueryDesignerGuiEditor::storeLayout(). 
+		 This solves the problem when user has changed a query schema but already form still uses
+		 previously instantiated query schema. 
+		 \return true if there is such query. Otherwise the method does nothing. */
+		bool setQuerySchemaObsolete( const QString& queryName );
+
 //js: MOVED TO Driver		QString valueToSQL( const Field::Type ftype, const QVariant& v ) const;
 //		QString valueToSQL( const Field *field, const QVariant& v ) const;
 
@@ -374,28 +413,37 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 This is convenient method when we need only first record from query result,
 		 or when we know that query result has only one record.
 		 Adds a LIMIT clause to the query, \a sql should not include one already.
-		 \return true if query was successfully executed and first record has been found. */
-		bool querySingleRecord(const QString& sql, RowData &data);
+		 \return true if query was successfully executed and first record has been found,
+		 false on data retrieving failure, and cancelled if there's no single record available. */
+		tristate querySingleRecord(const QString& sql, RowData &data);
+
+		/*! Like tristate querySingleRecord(const QString& sql, RowData &data)
+		 but uses QuerySchema object. */
+		tristate querySingleRecord(QuerySchema& query, RowData &data);
 
 		/*! Executes \a sql query and stores first record's field's (number \a column) string value 
 		 inside \a value. For efficiency it's recommended that a query defined by \a sql
 		 should have just one field (SELECT one_field FROM ....). 
 		 Adds a LIMIT clause to the query, so \a sql should not include one already.
-		 \return true if query was successfully executed and first record has been found.
+		 \return true if query was successfully executed and first record has been found,
+		 false on data retrieving failure, and cancelled if there's no single record available.
 		 \sa queryStringList() */
-		bool querySingleString(const QString& sql, QString &value, uint column = 0);
+		tristate querySingleString(const QString& sql, QString &value, uint column = 0);
 
 		/*! Convenience function: executes \a sql query and stores first 
 		 record's field's (number \a column) value inside \a number. \sa querySingleString(). 
-		 Note: "LIMIT 1" is appended to \a sql statement */
-		bool querySingleNumber(const QString& sql, int &number, uint column = 0);
+		 Note: "LIMIT 1" is appended to \a sql statement 
+		 \return true if query was successfully executed and first record has been found,
+		 false on data retrieving failure, and cancelled if there's no single record available. */
+		tristate querySingleNumber(const QString& sql, int &number, uint column = 0);
 
 		/*! Executes \a sql query and stores first record's first field's string value 
 		 inside \a list. The list is initially cleared.
 		 For efficiency it's recommended that a query defined by \a sql
 		 should have just one field (SELECT one_field FROM ....). 
-		 \return true if all values were fetched successfuly
-		 On errors, the list is not cleared, it may contain few retrieved values. */
+		 \return true if all values were fetched successfuly,
+		 false on data retrieving failure. Returning empty list can be still a valid result.
+		 On errors, the list is not cleared, it may contain a few retrieved values. */
 		bool queryStringList(const QString& sql, QStringList& list, uint column = 0);
 
 		/*! \return true if there is at least one record returned in \a sql query.
@@ -408,7 +456,7 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		/*! \return true if there is at least one record in \a table. */
 		bool isEmpty( TableSchema& table, bool &success );
 
-//TODO perhaps use Q_ULLONG here?
+//! @todo perhaps use Q_ULLONG here?
 		/*! \return number of records in \a sql query.
 		 Does not fetch any records. -1 is returned on query execution errors (>0 otherwise).
 		 Note: real executed query is: "SELECT COUNT() FROM (\a sql) LIMIT 1" 
@@ -469,7 +517,7 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 No error is raised if the table does not exist physically 
 		 - its schema is removed even in this case.
 		*/
-//TODO(js): update any structure (e.g. query) that depend on this table!
+//! @todo (js): update any structure (e.g. query) that depend on this table!
 		tristate dropTable( TableSchema* tableSchema );
 
 		/*! It is a convenience function, does exactly the same as 
@@ -478,8 +526,8 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 
 		/*! Alters \a tableSchema using \a newTableSchema in memory and on the db backend. 
 		 \return true on success, cancelled if altering was cancelled. */
-//TODO(js): implement real altering
-//TODO(js): update any structure (e.g. query) that depend on this table!
+//! @todo (js): implement real altering
+//! @todo (js): update any structure (e.g. query) that depend on this table!
 		tristate alterTable( TableSchema& tableSchema, TableSchema& newTableSchema);
 
 		/*! Alters table's described \a tableSchema name to \a newName. 
@@ -570,13 +618,19 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 Simply, method internally fetches last inserted record and returns selected 
 		 field's value. Requirements: field must be of integer type, there must be a
 		 record inserted in current database session (whatever this means).
-		 On error -1ULL is returned.
+		 On error (Q_ULLONG)-1 is returned.
+		 Last inserted record is identified by magical row identifier, usually called 
+		 ROWID (PostgreSQL has it as well as SQLite; 
+		 see DriverBehaviour::ROW_ID_FIELD_RETURNS_LAST_AUTOINCREMENTED_VALUE). 
+		 ROWID's value will be assigned back to \a ROWID if this pointer is not null.
 		*/
-		Q_ULLONG lastInsertedAutoIncValue(const QString& aiFieldName, const QString& tableName);
+		Q_ULLONG lastInsertedAutoIncValue(const QString& aiFieldName, const QString& tableName, 
+			Q_ULLONG* ROWID = 0);
 		
-		/*! \overload int lastInsertedAutoIncValue(const QString&, const QString&)
+		/*! \overload int lastInsertedAutoIncValue(const QString&, const QString&, Q_ULLONG*)
 		*/
-		Q_ULLONG lastInsertedAutoIncValue(const QString& aiFieldName, const TableSchema& table);
+		Q_ULLONG lastInsertedAutoIncValue(const QString& aiFieldName, 
+			const TableSchema& table, Q_ULLONG* ROWID = 0);
 
 		/*! Executes query \a statement, but without returning resulting 
 		 rows (used mostly for functional queries). 
@@ -589,12 +643,11 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 Note: The statement string can be specific for this connection's driver database, 
 		 and thus not reusable in general.
 		*/
-		QString selectStatement( QuerySchema& querySchema, int idEscaping = Driver::EscapeDriver|Driver::EscapeAsNecessary ) const;
-
-		/*! \return sql string of actually executed SQL statement,
-		 usually using drv_executeSQL(). If there was error during executing SQL statement, 
-		 before, that string is returned instead. */
-		const QString recentSQLString() const { return m_errorSql.isEmpty() ? m_sql : m_errorSql; }
+		inline QString selectStatement( QuerySchema& querySchema, 
+			int idEscaping = Driver::EscapeDriver|Driver::EscapeAsNecessary ) const
+		{
+			return selectStatement(querySchema, false, idEscaping);
+		}
 
 		/*! Stores object's schema data (id, name, caption, help text)
 		 described by \a sdata on the backend. 
@@ -608,13 +661,14 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		bool storeObjectSchemaData( SchemaData &sdata, bool newObject );
 
 		/*! Added for convenience. 
-		 \sa setupObjectSchemaData( const KexiDB::RowData &data, SchemaData &sdata ) */
-		bool loadObjectSchemaData( int objectID, SchemaData &sdata );
+		 \sa setupObjectSchemaData( const KexiDB::RowData &data, SchemaData &sdata ).
+		 \return true on success, false on failure and cancelled when such object couldn't */
+		tristate loadObjectSchemaData( int objectID, SchemaData &sdata );
 
 		/*! Finds object schema data for object of type \a objectType and name \a objectName.
 		 If the object is found, resulted schema is stored in \a sdata and true is returned,
 		 otherwise false is returned. */
-		bool loadObjectSchemaData( int objectType, const QString& objectName, SchemaData &sdata );
+		tristate loadObjectSchemaData( int objectType, const QString& objectName, SchemaData &sdata );
 
 		/*! Loads (potentially large) data block (e.g. xml form's representation), referenced by objectID
 		 and puts it to \a dataString. The can be block indexed with optional \a dataID.
@@ -666,6 +720,41 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 
 		tristate closeAllTableSchemaChangeListeners(TableSchema& tableSchema);
 
+		/*! @internal Removes \a tableSchema from internal structures and 
+		 destroys it. Does not make any change at the backend. */
+		void removeTableSchemaInternal(KexiDB::TableSchema *tableSchema);
+
+		/*! @internal. Inserts internal table to Connection's structures, so it can be found by name.
+		 This method is used for example in KexiProject to insert information about "kexi__blobs" 
+		 table schema. Use createTable() to physically create table. After createTable() 
+		 calling insertInternalTableSchema() is not required.
+		 Also used internally by newKexiDBSystemTableSchema(const QString& tsname) */
+		void insertInternalTableSchema(TableSchema *tableSchema);
+
+//! @todo move this somewhere to low level class (MIGRATION?)
+		/*! LOW LEVEL METHOD. For reimplemenation: returns true if table 
+		 with name \a tableName exists in the database.
+		 \return false if it does not exist or error occured.
+		 The lookup is case insensitive. */
+		virtual bool drv_containsTable( const QString &tableName ) = 0;
+
+		/*! Creates table using \a tableSchema information.
+		 \return true on success. Default implementation 
+		 builds a statement using createTableStatement() and calls drv_executeSQL()
+		 Note for driver developers: reimplement this only if you want do to 
+		 this in other way.
+
+		 Moved to public for KexiMigrate.
+		 @todo fix this after refatoring
+		 */
+		virtual bool drv_createTable( const TableSchema& tableSchema );
+
+		/*! */
+		virtual PreparedStatement::Ptr prepareStatement(PreparedStatement::StatementType type, 
+			FieldList& fields) = 0;
+
+		bool isInternalTableSchema(const QString& tableName);
+
 	protected:
 		/*! Used by Driver */
 		Connection( Driver *driver, ConnectionData &conn_data );
@@ -702,20 +791,13 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 Default implementation puts empty list into \a list and returns true. */
 		virtual bool drv_getDatabasesList( QStringList &list );
 
-//TODO: move this somewhere to low level class (MIGRATION?)
+//! @todo move this somewhere to low level class (MIGRATION?)
 		/*! LOW LEVEL METHOD. For reimplemenation: loads low-level list of table names
 		 available for this connection. The names are in lower case.
 		 The method should return true only if there was no error on getting database names 
 		 list from the server. */
 		virtual bool drv_getTablesList( QStringList &list ) = 0;
 
-//TODO: move this somewhere to low level class (MIGRATION?)
-		/*! LOW LEVEL METHOD. For reimplemenation: returns true if table 
-		 with name \a tableName exists in the database.
-		 \return false if it does not exist or error occured.
-		 The lookup is case insensitive. */
-		virtual bool drv_containsTable( const QString &tableName ) = 0;
-
 		/*! For optional reimplemenation: asks server if database \a dbName exists.
 		 This method is used internally in databaseExists(). The default  implementation
 		 calls databaseNames and checks if that list contains \a dbName. If you need to
@@ -733,8 +815,10 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		/*! For reimplemenation: creates new database using connection */
 		virtual bool drv_createDatabase( const QString &dbName = QString::null ) = 0;
 
-		/*! For reimplemenation: opens existing database using connection */
-		virtual bool drv_useDatabase( const QString &dbName = QString::null ) = 0;
+		/*! For reimplemenation: opens existing database using connection 
+		 \return true on success, false on failure and cancelled if user has cancelled this action. */
+		virtual bool drv_useDatabase( const QString &dbName = QString::null, bool *cancelled = 0, 
+			MessageHandler* msgHandler = 0 ) = 0;
 
 		/*! For reimplemenation: closes previously opened database 
 			using connection. */
@@ -770,7 +854,7 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 
 
 		/*! \return "SELECT ..." statement's string needed for executing query 
-		 defined by "select * from <table_name>" where <table_name> is \a tableSchema's name.
+		 defined by "select * from table_name" where <i>table_name</i> is \a tableSchema's name.
 		 This method's variant can be useful when there is no appropriate QuerySchema defined.
 
 		 Note: The statement string can be specific for this connection's driver database, 
@@ -778,13 +862,10 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		*/
 		QString selectStatement( TableSchema& tableSchema ) const;
 
-		/*! Creates table using \a tableSchema information.
-		 \return true on success. Default implementation 
-		 builds a statement using createTableStatement() and calls drv_executeSQL()
-		 Note for driver developers: reimplement this only if you want do to 
-		 this in other way.
-		 */
-		virtual bool drv_createTable( const TableSchema& tableSchema );
+		/*! Like selectStatement( QuerySchema& querySchema, int idEscaping = Driver::EscapeDriver|Driver::EscapeAsNecessary ) const
+		 but also retrieves ROWID information, if \a alsoRetrieveROWID is true.
+		 Used by cursors. */
+		QString selectStatement( QuerySchema& querySchema, bool alsoRetrieveROWID, int idEscaping = Driver::EscapeDriver|Driver::EscapeAsNecessary ) const;
 
 		/*! 
 		 Creates table named by \a tableSchemaName. Schema object must be on
@@ -871,21 +952,30 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		virtual bool drv_alterTableName(TableSchema& tableSchema, const QString& newName, bool replace = false);
 
 		/*! Internal, for handling autocommited transactions:
-		 begins transaction is one is supported.
-		 If driver only supports single transaction,
-		 and there is already transaction started, it is commited before
-		 starting a new one. \return true if new transaction started
+		 begins transaction if one is supported.
+		 \return true if new transaction started
 		 successfully or no transactions are supported at all by the driver
 		 or if autocommit option is turned off.
-		 Newly created transaction (or null on error)
-		 is passed to \a trans parameter.
+		 A handle to a newly created transaction (or null on error) is passed 
+		 to \a tg parameter.
+
+		 Special case when used database driver has only single transaction support 
+		 (Driver::SingleTransactions): 
+		 and there is already transaction started, it is commited before
+		 starting a new one, but only if this transaction has been started inside Connection object.
+		 (i.e. by beginAutoCommitTransaction()). Otherwise, a new transaction will not be started, 
+		 but true will be returned immediately.
 		*/
-		bool beginAutoCommitTransaction(Transaction& trans);
+		bool beginAutoCommitTransaction(TransactionGuard& tg);
 		
 		/*! Internal, for handling autocommited transactions:
 		 Commits transaction prevoiusly started with beginAutoCommitTransaction().
 		 \return true on success or when no transactions are supported 
 		 at all by the driver.
+
+		 Special case when used database driver has only single transaction support 
+		 (Driver::SingleTransactions): if \a trans has been started outside Connection object 
+		 (i.e. not by beginAutoCommitTransaction()), the transaction will not be commited.
 		*/
 		bool commitAutoCommitTransaction(const Transaction& trans);
 		
@@ -893,6 +983,10 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 Rollbacks transaction prevoiusly started with beginAutoCommitTransaction().
 		 \return true on success or when no transactions are supported 
 		 at all by the driver.
+
+		 Special case when used database driver has only single transaction support 
+		 (Driver::SingleTransactions): \a trans will not be rolled back 
+		 if it has been started outside this Connection object.
 		*/
 		bool rollbackAutoCommitTransaction(const Transaction& trans);
 
@@ -923,11 +1017,11 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 Used internally by querySchema() methods. */
 		QuerySchema* setupQuerySchema( const RowData &data );
 
-		bool updateRow(QuerySchema &query, RowData& data, RowEditBuffer& buf);
+		bool updateRow(QuerySchema &query, RowData& data, RowEditBuffer& buf, bool useROWID = false);
 
-		bool insertRow(QuerySchema &query, RowData& data, RowEditBuffer& buf);
+		bool insertRow(QuerySchema &query, RowData& data, RowEditBuffer& buf, bool getROWID = false);
 
-		bool deleteRow(QuerySchema &query, RowData& data);
+		bool deleteRow(QuerySchema &query, RowData& data, bool useROWID = false);
 
 		bool deleteAllRows(QuerySchema &query);
 
@@ -958,7 +1052,7 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 - we assume these are valid identifiers for all drivers.
 		*/
 		inline QString escapeIdentifier(const QString& id, 
-		    int drvEscaping = Driver::EscapeDriver|Driver::EscapeAsNecessary ) const {
+			int drvEscaping = Driver::EscapeDriver|Driver::EscapeAsNecessary ) const {
 			return m_driver->escapeIdentifier(id, drvEscaping);
 		}
 		
@@ -971,6 +1065,13 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		 else, sets appropriate error with a message and returns false. */
 		bool checkIfColumnExists(Cursor *cursor, uint column);
 
+		/*! @internal used by querySingleRecord() methods. */
+		tristate querySingleRecordInternal(RowData &data, const QString* sql, QuerySchema* query);
+
+		/*! @internal used by Driver::createConnection(). 
+		 Only works if connection is not yet established. */
+		void setReadOnly(bool set);
+
 		QGuardedPtr<ConnectionData> m_data;
 		QString m_name;
 		QString m_usedDatabase; //!< database name that is opened now
@@ -982,28 +1083,21 @@ class KEXI_DB_EXPORT Connection : public QObject, public KexiDB::Object
 		QDict<QuerySchema> m_queries_byname;
 
 		//! used just for removing system TableSchema objects on db close.
-		QPtrList<TableSchema> m_kexiDBSystemtables;
+		QPtrDict<TableSchema> m_kexiDBSystemTables;
 
 		//! cursors created for this connection
 		QPtrDict<KexiDB::Cursor> m_cursors;
-//		ConnectionInternal* m_internal;
-
-		//! used to store of actually executed SQL statement
-		QString m_sql, m_errorSql;
 
 	friend class KexiDB::Driver;
 	friend class KexiDB::Cursor;
 	friend class KexiDB::TableSchema; //!< for removeMe()
+	friend class KexiDB::DatabaseProperties; //!< for setError()
 	friend class ConnectionPrivate;
 
 		ConnectionPrivate *d;
 	private:
 		Driver *m_driver;
-		bool m_is_connected : 1;
-		bool m_autoCommit : 1;
 		bool m_destructor_started : 1; //!< helper: true if destructor is started
-		
-		QString m_availableDatabaseName; //! used by anyAvailableDatabaseName()
 };
 
 } //namespace KexiDB
