@@ -13,14 +13,16 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <kexidb/queryschema.h>
 #include <kexidb/driver.h>
 #include <kexidb/connection.h>
 #include <kexidb/expression.h>
+#include <kexidb/parser/sqlparser.h>
+#include <kexidb/utils.h>
 
 #include <assert.h>
 
@@ -37,6 +39,7 @@ using namespace KexiDB;
 
 //=======================================
 namespace KexiDB {
+//! @internal
 class QuerySchemaPrivate
 {
 	public:
@@ -46,6 +49,7 @@ class QuerySchemaPrivate
 		 , maxIndexWithAlias(-1)
 		 , visibility(64)
 		 , fieldsExpanded(0)
+		 , orderByColumnList(0)
 		 , autoincFields(0)
 		 , fieldsOrder(0)
 		 , pkeyFieldsOrder(0)
@@ -67,6 +71,7 @@ class QuerySchemaPrivate
 		~QuerySchemaPrivate()
 		{
 			delete fieldsExpanded;
+			delete orderByColumnList;
 			delete autoincFields;
 			delete fieldsOrder;
 			delete pkeyFieldsOrder;
@@ -162,7 +167,7 @@ class QuerySchemaPrivate
 					//missing
 					for (;;) { //find 1st unused
 						colNum++;
-						columnAlias = (i18n("short for expression (only latin letters, please)", "expr") 
+						columnAlias = (i18n("short for 'expression' word (only latin letters, please)", "expr") 
 							+ QString::number(colNum)).latin1();
 						if (!tablePositionsForAliases[columnAlias])
 							break;
@@ -196,6 +201,9 @@ class QuerySchemaPrivate
 //		Field::Vector *fieldsExpanded;
 		QueryColumnInfo::Vector *fieldsExpanded;
 
+		/*! A list of fields for ORDER BY section. @see QuerySchema::orderByColumnList(). */
+		QueryColumnInfo::Vector *orderByColumnList;
+
 		/*! A cache for autoIncrementFields(). */
 		QueryColumnInfo::List *autoincFields;
 		
@@ -206,12 +214,12 @@ class QuerySchemaPrivate
 		/*! A map for fast lookup of query fields' order.
 		 This is exactly opposite information compared to vector returned 
 		 by fieldsExpanded() */
-		QMap<QueryColumnInfo*,uint> *fieldsOrder;
+		QMap<QueryColumnInfo*,int> *fieldsOrder;
 
 //		QValueList<bool> detailedVisibility;
 
 		/*! order of PKEY fields (e.g. for updateRow() ) */
-		QValueVector<uint> *pkeyFieldsOrder;
+		QValueVector<int> *pkeyFieldsOrder;
 
 		/*! number of PKEY fields within the query */
 		uint pkeyFieldsCount;
@@ -640,6 +648,17 @@ bool QuerySchema::contains(TableSchema *table) const
 	return d->tables.findRef(table)!=-1;
 }
 
+Field* QuerySchema::findTableField(const QString &tableDotFieldName) const
+{
+	QString tableName, fieldName;
+	if (!KexiDB::splitToTableAndFieldParts(tableDotFieldName, tableName, fieldName))
+		return 0;
+	TableSchema *tableSchema = table(tableName);
+	if (!tableSchema)
+		return 0;
+	return tableSchema->field(fieldName);
+}
+
 QCString QuerySchema::columnAlias(uint position) const
 {
 	QCString *a = d->columnAlias(position);
@@ -774,10 +793,28 @@ QueryColumnInfo* QuerySchema::columnInfo(const QString& name)
 	return d->columnInfosByName[name];
 }
 
-QueryColumnInfo::Vector QuerySchema::fieldsExpanded()
+QueryColumnInfo::Vector QuerySchema::fieldsExpanded(bool unique)
 {
 	computeFieldsExpanded();
-	return *d->fieldsExpanded;
+	if (!unique)
+		return *d->fieldsExpanded;
+
+	QDict<char> columnsAlreadyFound;
+	QueryColumnInfo::Vector result( d->fieldsExpanded->count() ); //initial size is set
+//	QMapConstIterator<QueryColumnInfo*, bool> columnsAlreadyFoundIt;
+	//compute unique list
+	uint uniqueListCount = 0;
+	for (uint i=0; i<d->fieldsExpanded->count(); i++) {
+		QueryColumnInfo *ci = (*d->fieldsExpanded)[i];
+//		columnsAlreadyFoundIt = columnsAlreadyFound.find(ci);
+//		uint foundColumnIndex = -1;
+		if (!columnsAlreadyFound[ci->aliasOrName()]) {// columnsAlreadyFoundIt==columnsAlreadyFound.constEnd())
+			columnsAlreadyFound.insert(ci->aliasOrName(), (char*)1);
+			result.insert(uniqueListCount++, ci);
+		}
+	}
+	result.resize(uniqueListCount); //update result size
+	return result;
 }
 
 void QuerySchema::computeFieldsExpanded()
@@ -837,7 +874,7 @@ void QuerySchema::computeFieldsExpanded()
 	if (!d->fieldsExpanded) {
 		d->fieldsExpanded = new QueryColumnInfo::Vector( list.count() );// Field::Vector( list.count() );
 		d->fieldsExpanded->setAutoDelete(true);
-		d->fieldsOrder = new QMap<QueryColumnInfo*,uint>();
+		d->fieldsOrder = new QMap<QueryColumnInfo*,int>();
 	}
 	else {//for future:
 		d->fieldsExpanded->clear();
@@ -875,32 +912,32 @@ void QuerySchema::computeFieldsExpanded()
 //		*detailedVisibility = d->detailedVisibility;
 }
 
-QMap<QueryColumnInfo*,uint> QuerySchema::fieldsOrder()
+QMap<QueryColumnInfo*,int> QuerySchema::fieldsOrder()
 {
 	if (!d->fieldsOrder)
 		computeFieldsExpanded();
 	return *d->fieldsOrder;
 }
 
-QValueVector<uint> QuerySchema::pkeyFieldsOrder()
+QValueVector<int> QuerySchema::pkeyFieldsOrder()
 {
 	if (d->pkeyFieldsOrder)
 		return *d->pkeyFieldsOrder;
 
 	TableSchema *tbl = masterTable();
 	if (!tbl || !tbl->primaryKey())
-		return QValueVector<uint>();
+		return QValueVector<int>();
 
 	//get order of PKEY fields (e.g. for rows updating or inserting )
 	IndexSchema *pkey = tbl->primaryKey();
-	d->pkeyFieldsOrder = new QValueVector<uint>( pkey->fieldCount(), -1 );
+	d->pkeyFieldsOrder = new QValueVector<int>( pkey->fieldCount(), -1 );
 
 	const uint fCount = fieldsExpanded().count();
 	d->pkeyFieldsCount = 0;
 	for (uint i = 0; i<fCount; i++) {
 		QueryColumnInfo *fi = d->fieldsExpanded->at(i);
 		const int fieldIndex = fi->field->table()==tbl ? pkey->indexOf(fi->field) : -1;
-		if (fieldIndex!=-1/* field foung in PK */ 
+		if (fieldIndex!=-1/* field found in PK */ 
 			&& d->pkeyFieldsOrder->at(fieldIndex)==-1 /* first time */)
 		{
 			KexiDBDbg << "QuerySchema::pkeyFieldsOrder(): FIELD " << fi->field->name() 
@@ -992,11 +1029,91 @@ void QuerySchema::setWhereExpression(BaseExpr *expr)
 	d->whereExpr = expr;
 }
 
+void QuerySchema::addToWhereExpression(KexiDB::Field *field, const QVariant& value, int relation)
+{
+	int token;
+	if (value.isNull())
+		token = SQL_NULL;
+	else if (field->isIntegerType()) {
+		token = INTEGER_CONST;
+	}
+	else if (field->isFPNumericType()) {
+		token = REAL_CONST;
+	}
+	else {
+		token = CHARACTER_STRING_LITERAL;
+//! @todo date, time			
+	}
+	
+	BinaryExpr * newExpr = new BinaryExpr(
+		KexiDBExpr_Relational, 
+		new ConstExpr( token, value ),
+		relation,
+		new VariableExpr((field->table() ? (field->table()->name()+".") : QString::null)+field->name())
+	);
+	if (d->whereExpr) {
+		d->whereExpr = new BinaryExpr(
+			KexiDBExpr_Logical, 
+			d->whereExpr,
+			AND,
+			newExpr
+		);
+	}
+	else {
+		d->whereExpr = newExpr;
+	}
+}
+
+/*
+void QuerySchema::addToWhereExpression(KexiDB::Field *field, const QVariant& value)
+		switch (value.type()) {
+		case Int: case UInt: case Bool: case LongLong: case ULongLong:
+			token = INTEGER_CONST;
+			break;
+		case Double:
+			token = REAL_CONST;
+			break;
+		default:
+			token = CHARACTER_STRING_LITERAL;
+		}
+//! @todo date, time			
+				
+*/
+
 BaseExpr *QuerySchema::whereExpression() const
 {
 	return d->whereExpr;
 }
 
+void QuerySchema::setOrderByColumnList(const QStringList& columnNames)
+{
+	Q_UNUSED(columnNames);
+//! @todo implement this:
+// all field names should be fooun, exit otherwise ..........
+
+	// OK
+//TODO	if (!d->orderByColumnList)
+//TODO
+}
+
+/*! Convenience method, similar to setOrderBy(const QStringList&). */
+void QuerySchema::setOrderByColumnList(const QString& column1, const QString& column2, 
+	const QString& column3, const QString& column4, const QString& column5)
+{
+	Q_UNUSED(column1);
+	Q_UNUSED(column2);
+	Q_UNUSED(column3);
+	Q_UNUSED(column4);
+	Q_UNUSED(column5);
+//! @todo implement this, like above
+//! @todo add ORDER BY info to debugString()
+}
+
+QueryColumnInfo::Vector QuerySchema::orderByColumnList() const
+{
+	return d->orderByColumnList ? *d->orderByColumnList: QueryColumnInfo::Vector();
+}
+
 
 /*
 	new field1, Field *field2
