@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 #include <qdom.h>
 #include <qtextstream.h>
@@ -39,6 +39,7 @@
 #include "kivioglobal.h"
 #include "kivio_config.h"
 #include "polylineconnectorspawner.h"
+#include "kivio_canvas.h"
 
 #include "stencilbarbutton.h"
 
@@ -72,20 +73,20 @@
 #include <kiconloader.h>
 #include <khelpmenu.h>
 #include <kconfig.h>
-#include <koTemplateChooseDia.h>
-#include <koFilterManager.h>
-#include <koStoreDevice.h>
+#include <KoTemplateChooseDia.h>
+#include <KoFilterManager.h>
+#include <KoStoreDevice.h>
 #include "KIvioDocIface.h"
 #include <kcommand.h>
-#include <kozoomhandler.h>
-#include <koApplication.h>
+#include <KoZoomHandler.h>
+#include <KoApplication.h>
 #include <kglobal.h>
-#include <kocommandhistory.h>
-#include <koxmlwriter.h>
-#include <koGenStyles.h>
-#include <koOasisSettings.h>
-#include <kodom.h>
-#include <koxmlns.h>
+#include <KoCommandHistory.h>
+#include <KoXmlWriter.h>
+#include <KoGenStyles.h>
+#include <KoOasisSettings.h>
+#include <KoDom.h>
+#include <KoXmlNS.h>
 
 //using namespace std;
 
@@ -113,6 +114,7 @@ KivioDoc::KivioDoc( QWidget *parentWidget, const char* widgetName, QObject* pare
   m_currentFile = 0;
 
   setInstance( KivioFactory::global(), false );
+  setTemplateType("kivio_template");
 
   if ( !name )
   {
@@ -149,7 +151,7 @@ KivioDoc::KivioDoc( QWidget *parentWidget, const char* widgetName, QObject* pare
 
 
   if ( name )
-      dcopObject();
+    dcopObject();
 }
 
 DCOPObject* KivioDoc::dcopObject()
@@ -220,6 +222,29 @@ bool KivioDoc::initDoc(InitDocFlags flags, QWidget* parentWidget)
   }
 }
 
+void KivioDoc::openExistingFile( const QString& file )
+{
+  KivioPage* t = createPage();
+  m_pMap->addPage(t);
+
+  KoDocument::openExistingFile(file);
+}
+
+void KivioDoc::openTemplate( const QString& file )
+{
+  KivioPage* t = createPage();
+  m_pMap->addPage(t);
+
+  KoDocument::openTemplate(file);
+}
+
+void KivioDoc::initEmpty()
+{
+  KivioPage* t = createPage();
+  m_pMap->addPage(t);
+  KoDocument::initEmpty();
+}
+
 KoView* KivioDoc::createViewInstance( QWidget* parent, const char* name )
 {
   if (!name) {
@@ -237,7 +262,7 @@ QDomDocument KivioDoc::saveXML()
   kivio.setAttribute( "editor", "Kivio" );
   kivio.setAttribute( "mime", MIME_TYPE );
 
-  kivio.setAttribute( "units", KoUnit::unitName(m_units) );
+  kivio.setAttribute( "units", unitName() );
   gridData.save(kivio,"grid");
 
   QDomElement viewItemsElement = doc.createElement("ViewItems");
@@ -259,7 +284,9 @@ QDomDocument KivioDoc::saveXML()
   KivioStencilSpawnerSet *pSet = m_pLstSpawnerSets->first();
   while( pSet )
   {
-    kivio.appendChild( pSet->saveXML( doc ) );
+    if(checkStencilsForSpawnerSet(pSet)) {
+      kivio.appendChild(pSet->saveXML(doc));
+    }
 
     pSet = m_pLstSpawnerSets->next();
   }
@@ -360,7 +387,7 @@ bool KivioDoc::saveOasis(KoStore* store, KoXmlWriter* manifestWriter)
     settingsWriter->startElement("config:config-item-set");
     settingsWriter->addAttribute("config:name", "view-settings");
 
-    KoUnit::saveOasis( settingsWriter, units() );
+    KoUnit::saveOasis( settingsWriter, unit() );
     saveOasisSettings( *settingsWriter );
 
     settingsWriter->endElement(); // config:config-item-set
@@ -431,7 +458,7 @@ bool KivioDoc::loadOasis( const QDomDocument& doc, KoOasisStyles& oasisStyles, c
 
   loadOasisSettings( settings );
 
-  emit updateActivePage(m_pMap->firstPage());
+  emit loadingFinished();
   return true;
 }
 
@@ -444,7 +471,7 @@ void KivioDoc::loadOasisSettings( const QDomDocument&settingsDoc )
     KoOasisSettings::Items viewSettings = settings.itemSet( "view-settings" );
     if ( !viewSettings.isNull() )
     {
-        setUnits(KoUnit::unit(viewSettings.parseConfigItemString("unit")));
+        setUnit(KoUnit::unit(viewSettings.parseConfigItemString("unit")));
         //todo add other config here.
     }
 }
@@ -482,6 +509,7 @@ bool KivioDoc::loadXML( QIODevice *, const QDomDocument& doc )
     else if( name == "KivioStencilSpawnerSet" )
     {
         QString id = XmlReadString( node.toElement(), "id", "" );
+        bool hidden = (XmlReadString(node.toElement(), "hidden", "false") == "true");
 
         if( id.isEmpty() )
         {
@@ -489,7 +517,7 @@ bool KivioDoc::loadXML( QIODevice *, const QDomDocument& doc )
         }
         else
         {
-            loadStencilSpawnerSet( id );
+            loadStencilSpawnerSet(id, hidden);
         }
     }
     else if( name == "Options" )
@@ -509,20 +537,20 @@ bool KivioDoc::loadXML( QIODevice *, const QDomDocument& doc )
   int u = us.toInt(&isInt);
 
   if(!isInt) {
-    setUnits(KoUnit::unit(us));
+    setUnit(KoUnit::unit(us));
   } else {
-    setUnits(Kivio::convToKoUnit(u));
+    setUnit(Kivio::convToKoUnit(u));
   }
 
   if(kivio.hasAttribute("gridIsShow")) {
     gridData.load(kivio,"grid");
   }
 
-  emit updateActivePage(m_pMap->firstPage());
+  emit loadingFinished();
   return true;
 }
 
-bool KivioDoc::loadStencilSpawnerSet( const QString &id )
+bool KivioDoc::loadStencilSpawnerSet(const QString &id, bool hidden)
 {
   KStandardDirs *dirs = KGlobal::dirs();
   QStringList dirList = dirs->findDirs("data", "kivio/stencils");
@@ -572,7 +600,7 @@ bool KivioDoc::loadStencilSpawnerSet( const QString &id )
             {
 
               // Load the spawner set with  rootDir + "/" + fi.fileName()
-              addSpawnerSetDuringLoad( innerFI->absFilePath() );
+              addSpawnerSetDuringLoad(innerFI->absFilePath(), hidden);
               return true;
             }
           }
@@ -649,16 +677,16 @@ void KivioDoc::paintContent( QPainter& painter, const QRect& rect, bool transpar
     KoGlobal::dpiY());
   KivioScreenPainter ksp(&painter);
   ksp.painter()->translate( - zoom.zoomItX(r.x()), - zoom.zoomItY(r.y()) );
-  paintContent(ksp,rect,transparent,page, QPoint(zoom.zoomItX(r.x()), zoom.zoomItY(r.y())), &zoom, false);
+  paintContent(ksp,rect,transparent,page, QPoint(zoom.zoomItX(r.x()), zoom.zoomItY(r.y())), &zoom, false, false);
   ksp.setPainter(0L); // Important! Don't delete the QPainter!!!
 }
 
-void KivioDoc::paintContent( KivioPainter& painter, const QRect& rect, bool transparent, KivioPage* page, QPoint p0, KoZoomHandler* zoom, bool drawHandles )
+void KivioDoc::paintContent( KivioPainter& painter, const QRect& rect, bool transparent, KivioPage* page, QPoint p0, KoZoomHandler* zoom, bool drawConnectorTargets, bool drawSelection )
 {
   if ( isLoading() )
     return;
 
-  page->paintContent(painter,rect,transparent,p0,zoom, drawHandles);
+  page->paintContent(painter,rect,transparent,p0,zoom, drawConnectorTargets, drawSelection);
 }
 
 void KivioDoc::printContent( KPrinter &prn )
@@ -694,106 +722,47 @@ void KivioDoc::printContent( KPrinter &prn )
   p.stop();
 }
 
-/* TODO:
- *
- * This entire function should probably be encapsulated in some sort of
- * object or other class.
- */
-bool KivioDoc::exportPage(KivioPage *pPage,const QString &fileName, ExportPageDialog *dlg)
-{
-  KoZoomHandler zoom;
-  zoom.setZoomAndResolution(100, KoGlobal::dpiX(),
-    KoGlobal::dpiY());
-  QPixmap buffer;
-
-  if(dlg->crop())
-  {
-    if(dlg->fullPage()) {
-      buffer.resize(zoom.zoomItX(pPage->getRectForAllStencils().width()) + dlg->border()*2,
-        zoom.zoomItY(pPage->getRectForAllStencils().height()) + dlg->border()*2);
-    } else {
-      buffer.resize(zoom.zoomItX(pPage->getRectForAllSelectedStencils().width()) + dlg->border()*2,
-        zoom.zoomItY(pPage->getRectForAllSelectedStencils().height()) + dlg->border()*2);
-    }
-  }
-  else
-  {
-    buffer.resize(zoom.zoomItX(pPage->paperLayout().ptWidth) + dlg->border()*2,
-      zoom.zoomItY(pPage->paperLayout().ptHeight) + dlg->border()*2);
-  }
-
-  kdDebug(43000) << "KivioDoc::exportCurPage() to " << fileName << "\n";
-
-  KivioScreenPainter p;
-
-  buffer.fill(Qt::white);
-
-  p.start( &buffer );
-  p.setTranslation( dlg->border(), dlg->border() );
-
-  if( dlg->fullPage()==true )
-  {
-    if(dlg->crop()) {
-      p.setTranslation(-zoom.zoomItX(pPage->getRectForAllStencils().x()),
-        -zoom.zoomItY(pPage->getRectForAllStencils().y()));
-    }
-
-    pPage->printContent(p);
-  }
-  else
-  {
-    if(dlg->crop()) {
-      p.setTranslation(-zoom.zoomItX(pPage->getRectForAllSelectedStencils().x()),
-        -zoom.zoomItY(pPage->getRectForAllSelectedStencils().y()));
-    }
-
-    pPage->printSelected(p);
-  }
-
-  p.stop();
-
-
-  QFileInfo finfo(fileName);
-
-  return buffer.save( fileName, finfo.extension(false).upper().latin1(), dlg->quality());
-}
-
-
 // TODO: Fix for id system
-bool KivioDoc::setIsAlreadyLoaded( QString dirName, QString id )
+KivioStencilSpawnerSet* KivioDoc::findSpawnerSet(const QString& dirName, const QString& id)
 {
     KivioStencilSpawnerSet *pSet = m_pLstSpawnerSets->first();
     while(pSet)
     {
         if( pSet->dir() == dirName || pSet->id() == id )
         {
-            return true;
+            return pSet;
         }
 
         pSet = m_pLstSpawnerSets->next();
     }
 
-    return false;
+    return 0;
 }
 
 void KivioDoc::addSpawnerSet( const QString &dirName )
 {
   QString id = KivioStencilSpawnerSet::readId( dirName );
+  KivioStencilSpawnerSet* set = findSpawnerSet(dirName, id);
 
-  if( setIsAlreadyLoaded( dirName, id ) )
+  if(set)
   {
-      kdDebug(43000) << "KivioDoc::addSpawnerSet() - Cannot load duplicate stencil sets" << endl;
-      return;
+    // Unhide the set if it's only hidden
+    if(set->hidden()) {
+      set->setHidden(false);
+      emit sig_addSpawnerSet(set);
+    }
+
+    return;
   }
 
 
-  KivioStencilSpawnerSet* set = new KivioStencilSpawnerSet();
+  set = new KivioStencilSpawnerSet();
 
-  if( set->loadDir(dirName)==false )
+  if(!set->loadDir(dirName))
   {
-      kdDebug(43000) << "KivioDoc::addSpawnerSet() - Error loading dir set" << endl;
-      delete set;
-      return;
+    kdDebug(43000) << "KivioDoc::addSpawnerSet() - Error loading dir set" << endl;
+    delete set;
+    return;
   }
 
   // Queue set for loading stencils
@@ -810,11 +779,13 @@ void KivioDoc::addSpawnerSet( const QString &dirName )
   }
 }
 
-void KivioDoc::addSpawnerSetDuringLoad( const QString &dirName )
+void KivioDoc::addSpawnerSetDuringLoad(const QString& dirName, bool hidden)
 {
   KivioStencilSpawnerSet *set;
 
   set = new KivioStencilSpawnerSet();
+  set->setHidden(hidden);
+
   if( set->loadDir(dirName)==false )
   {
     kdDebug(43000) << "KivioDoc::addSpawnerSetDuringLoad() - Error loading dir set" << endl;
@@ -860,8 +831,8 @@ void KivioDoc::saveConfig()
 
 void KivioDoc::initConfig()
 {
-  m_units = KoUnit::unit(Kivio::Config::unit());
-  m_font = KoGlobal::defaultFont();
+  setUnit( KoUnit::unit(Kivio::Config::unit()) );
+  m_font = Kivio::Config::font();
   m_pageLayout = Kivio::Config::defaultPageLayout();
 }
 
@@ -876,102 +847,74 @@ bool KivioDoc::removeSpawnerSet( KivioStencilSpawnerSet *pSet )
  */
 void KivioDoc::slotDeleteStencilSet( DragBarButton *pBtn, QWidget *w, KivioStackBar *pBar )
 {
-    // Iterate through all spawners in the set checking if they exist in any of
-    // the pages
-    KivioIconView *pIconView = (KivioIconView *)w;
-    KivioStencilSpawnerSet *pSet = pIconView->spawnerSet();
+  KivioIconView* pIconView = static_cast<KivioIconView*>(w);
+  KivioStencilSpawnerSet* pSet = pIconView->spawnerSet();
 
-    // Start the iteration
-    KivioStencilSpawner *pSpawner = pSet->spawners()->first();
-    while( pSpawner )
-    {
-        // Check for a spawner.  If there is one, the set cannot be deleted
-        if( checkStencilsForSpawner( pSpawner )==true )
-        {
-            KMessageBox::error(NULL, i18n("Cannot delete stencil set because there are still stencils in use."),
-                i18n("Cannot Delete Stencil Set"));
-            return;
-        }
+  // Only actually remove the set if it isn't used...
+  if(!checkStencilsForSpawnerSet(pSet)) {
+    removeSpawnerSet(pSet);
+  } else {
+    pSet->setHidden(true);
+  }
 
-        pSpawner = pSet->spawners()->next();
-    }
-
-
-
-    // If we made it this far, it's ok to delete this stencil set, so do it
-//    if( KMessageBox::questionYesNo(NULL, i18n("Are you sure you want to delete this stencil set?"),
-//        i18n("Delete Stencil Set?"))==KMessageBox::Yes )
-    {
-        // Destroying the IconView does not destroy the spawner set, so we remove
-        // it here
-        removeSpawnerSet( pIconView->spawnerSet() );
-
-        // And emit the signal to kill the set (page & button)
-        emit sig_deleteStencilSet( pBtn, w, pBar );
-    }
+  // And emit the signal to kill the set (page & button)
+  emit sig_deleteStencilSet(pBtn, w, pBar);
 }
 
 /**
  * Checks if any stencils in the document use this spawner
  */
-bool KivioDoc::checkStencilsForSpawner( KivioStencilSpawner *pSpawner )
+bool KivioDoc::checkStencilsForSpawnerSet(KivioStencilSpawnerSet* spawnerSet)
 {
-    KivioPage *pPage;
-    KivioLayer *pLayer;
-    KivioStencil *pStencil;
+  KivioPage *pPage = m_pMap->firstPage();
+  KivioLayer *pLayer;
+  KivioStencil *pStencil;
 
-    // Iterate across all the pages
-    pPage = m_pMap->firstPage();
-    while( pPage )
-    {
-        pLayer = pPage->layers()->first();
-        while( pLayer )
-        {
-            pStencil = pLayer->stencilList()->first();
-            while( pStencil )
-            {
-                // If this is a group stencil, then we must check all child stencils
-                if( pStencil->groupList() && pStencil->groupList()->count() > 0 )
-                {
-                    if( checkGroupForSpawner( pStencil, pSpawner )==true )
-                        return true;
-                }
-                else if( pStencil->spawner() == pSpawner )
-                    return true;
+  // Iterate across all the pages
+  while(pPage) {
+    pLayer = pPage->layers()->first();
 
-                pStencil = pLayer->stencilList()->next();
-            }
+    while(pLayer) {
+      pStencil = pLayer->stencilList()->first();
 
-            pLayer = pPage->layers()->next();
+      while(pStencil) {
+        // If this is a group stencil, then we must check all child stencils
+        if(pStencil->groupList() && pStencil->groupList()->count() > 0) {
+          if(checkGroupForSpawnerSet(pStencil, spawnerSet)) {
+            return true;
+          }
+        } else if(pStencil->spawner()->set() == spawnerSet) {
+          return true;
         }
 
-        pPage = m_pMap->nextPage();
+        pStencil = pLayer->stencilList()->next();
+      }
+
+      pLayer = pPage->layers()->next();
     }
 
-    return false;
+    pPage = m_pMap->nextPage();
+  }
+
+  return false;
 }
 
-bool KivioDoc::checkGroupForSpawner( KivioStencil *pGroup, KivioStencilSpawner *pSpawner )
+bool KivioDoc::checkGroupForSpawnerSet(KivioStencil* pGroup, KivioStencilSpawnerSet* spawnerSet)
 {
-    KivioStencil *pStencil;
+  KivioStencil *pStencil = pGroup->groupList()->first();
 
-    pStencil = pGroup->groupList()->first();
-    while( pStencil )
-    {
-        if( pStencil->groupList() && pStencil->groupList()->count() > 0 )
-        {
-            if( checkGroupForSpawner( pStencil, pSpawner )==true )
-                return true;
-        }
-        else if( pStencil->spawner() == pSpawner )
-        {
-            return true;
-        }
-
-        pStencil = pGroup->groupList()->next();
+  while(pStencil) {
+    if(pStencil->groupList() && pStencil->groupList()->count() > 0) {
+      if(checkGroupForSpawnerSet(pStencil, spawnerSet))
+        return true;
+    } else if(pStencil->spawner()->set() == spawnerSet) {
+      return true;
     }
 
-    return false;
+    pStencil = pGroup->groupList()->next();
+  }
+
+  return false;
 }
 
 void KivioDoc::slotSelectionChanged()
@@ -1010,15 +953,6 @@ void KivioDoc::addInternalStencilSpawner(KivioStencilSpawner* spawner)
   m_pInternalSet->addSpawner(spawner);
 }
 
-void KivioDoc::setUnits(KoUnit::Unit unit)
-{
-  if (m_units == unit)
-    return;
-
-  m_units = unit;
-  emit unitsChanged(unit);
-}
-
 void KivioDoc::updateView(KivioPage* page)
 {
   emit sig_updateView(page);
@@ -1102,4 +1036,38 @@ void KivioDoc::loadStencil()
   }
 }
 
+void KivioDoc::updateGuideLines(KoView* sender)
+{
+  QValueList<double> hGuideLines;
+  QValueList<double> vGuideLines;
+  KivioView* view = static_cast<KivioView*>(sender);
+  view->canvasWidget()->guideLines().getGuideLines(hGuideLines, vGuideLines);
+  view->activePage()->setGuideLines(hGuideLines, vGuideLines);
+
+  QPtrListIterator<KoView> it(views());
+  KivioView* itView = 0;
+
+  for (; it.current(); ++it ) {
+    itView = static_cast<KivioView*>(it.current());
+    if(it.current() != sender && (itView->activePage() == view->activePage())) {
+      itView->canvasWidget()->guideLines().setGuideLines(hGuideLines, vGuideLines);
+    }
+  }
+}
+
+void KivioDoc::updateGuideLines(KivioPage* page)
+{
+  QValueList<double> hGuideLines = page->horizontalGuideLines();
+  QValueList<double> vGuideLines = page->verticalGuideLines();
+  QPtrListIterator<KoView> it(views());
+  KivioView* itView = 0;
+
+  for (; it.current(); ++it ) {
+    itView = static_cast<KivioView*>(it.current());
+    if(itView->activePage() == page) {
+      itView->canvasWidget()->guideLines().setGuideLines(hGuideLines, vGuideLines);
+    }
+  }
+}
+
 #include "kivio_doc.moc"
