@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "kivioglobal.h"
@@ -27,10 +27,15 @@
 #include <kglobal.h>
 #include <klocale.h>
 
-#include <koGenStyles.h>
+#include <KoGenStyles.h>
+#include <KoZoomHandler.h>
+#include <KoGlobal.h>
 
 #include "kivio_common.h"
 #include "kivio_settings.h"
+#include "kivio_stencil.h"
+#include "kivio_intra_stencil_data.h"
+#include "kivio_screen_painter.h"
 
 struct PaperSizeDef {
   const char* title;
@@ -1286,3 +1291,27 @@ QString Kivio::systemDefaultUnit()
 
   return defMS;
 }
+
+QPixmap Kivio::generatePixmapFromStencil(int width, int height, KivioStencil* stencil)
+{
+  KoZoomHandler zoomHandler;
+  QRect rect = zoomHandler.zoomRect(stencil->rect());
+
+  double zw = (double)(width - 2) / (double)rect.width();
+  double zh = (double)(height - 2) / (double)rect.height();
+  zoomHandler.setZoomAndResolution(qRound(kMin(zw, zh) * 100.0), KoGlobal::dpiX(), KoGlobal::dpiY());
+  rect = zoomHandler.zoomRect(stencil->rect());
+
+  QPixmap pix(width, height);
+  pix.fill(Qt::white);
+  KivioScreenPainter kpainter;
+  kpainter.start(&pix);
+  kpainter.translateBy(-rect.x() + ((width - rect.width()) / 2), -rect.y() + ((height - rect.height()) / 2));
+  KivioIntraStencilData data;
+  data.painter = &kpainter;
+  data.zoomHandler = &zoomHandler;
+  stencil->paint(&data);
+  kpainter.stop();
+
+  return pix;
+}
