@@ -2,7 +2,7 @@
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
    Copyright (C) 1999 Simon Hausmann <hausmann@kde.org>
    Copyright (C) 2000-2005 David Faure <faure@kde.org>
-   Copyright (C) 2005 Sven Lppken <sven@kde.org>
+   Copyright (C) 2005, 2006 Sven Lüppken <sven@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qcursor.h>
@@ -47,13 +47,14 @@
 #include <kparts/partmanager.h>
 #include <kaction.h>
 #include <kdeversion.h>
+#include <kaboutdata.h>
 
-#include <koQueryTrader.h>
-#include <koDocumentInfo.h>
-#include <koDocument.h>
-#include <koView.h>
-#include <koPartSelectDia.h>
-#include <koFilterManager.h>
+#include <KoQueryTrader.h>
+#include <KoDocumentInfo.h>
+#include <KoDocument.h>
+#include <KoView.h>
+#include <KoPartSelectDia.h>
+#include <KoFilterManager.h>
 #include <kiconloader.h>
 
 KoShellWindow::KoShellWindow()
@@ -63,6 +64,7 @@ KoShellWindow::KoShellWindow()
 
   m_pLayout = new QSplitter( centralWidget() );
 
+  // Setup the sidebar
   m_pSidebar = new IconSidePane( m_pLayout );
   m_pSidebar->setSizePolicy( QSizePolicy( QSizePolicy::Maximum,
                              QSizePolicy::Preferred ) );
@@ -71,6 +73,7 @@ KoShellWindow::KoShellWindow()
   m_grpDocuments = m_pSidebar->insertGroup(i18n("Documents"), true, this, SLOT(slotSidebar_Document(int)));
   m_pLayout->setResizeMode(m_pSidebar,QSplitter::FollowSizeHint);
 
+  // Setup the tabbar
   m_pFrame = new KTabWidget( m_pLayout );
   m_pFrame->setSizePolicy( QSizePolicy( QSizePolicy::Minimum,
                             QSizePolicy::Preferred ) );
@@ -88,34 +91,26 @@ KoShellWindow::KoShellWindow()
   QValueList<KoDocumentEntry> lstComponents = KoDocumentEntry::query(false,QString());
   QValueList<KoDocumentEntry>::Iterator it = lstComponents.begin();
   int id = 0;
-
+  // Get all available components
   for( ; it != lstComponents.end(); ++it )
   {
-      if (!(*it).service()->genericName().isEmpty()) //skip the unavailable part
-          id = m_pSidebar->insertItem(m_grpFile, (*it).service()->icon(), (*it).service()->genericName());
+      KService* service = (*it).service();
+      if ( !service->genericName().isEmpty() )
+      {
+          id = m_pSidebar->insertItem(m_grpFile, service->icon(), service->genericName());
+      }
       else
+      {
           continue;
+      }
 
       m_mapComponents[ id++ ] = *it;
-
-      // Build list of patterns for all supported KOffice documents...
-      QString nativeMimeType = (*it).service()->property( "X-KDE-NativeMimeType" ).toString();
-      //kdDebug() << nativeMimeType << endl;
-      if ( !nativeMimeType.isEmpty() )
-      {
-        KMimeType::Ptr mime = KMimeType::mimeType( nativeMimeType );
-        if (mime)
-        {
-          if ( !m_filter.isEmpty() )
-            m_filter += " ";
-          m_filter += mime->patterns().join(" ");
-        }
-      }
   }
-   QValueList<int> list;
-   list.append( KoShellSettings::sidebarWidth() );
-   list.append( this->width() - KoShellSettings::sidebarWidth() );
-   m_pLayout->setSizes( list );
+
+  QValueList<int> list;
+  list.append( KoShellSettings::sidebarWidth() );
+  list.append( this->width() - KoShellSettings::sidebarWidth() );
+  m_pLayout->setSizes( list );
 
   connect( this, SIGNAL( documentSaved() ),
            this, SLOT( slotNewDocumentName() ) );
@@ -267,7 +262,7 @@ void KoShellWindow::slotSidebarItemClicked( QIconViewItem *item )
     {
         // koshell isn't starting, but this is like starting a new app:
         // offer both "open existing file" and "open new file".
-        if ( doc->initDoc( KoDocument::InitDocAppStarting) )
+        if ( doc->showEmbedInitDialog( this ) )
         {
             partManager()->addPart( doc, false );
             setRootDocument( doc );
@@ -310,7 +305,7 @@ void KoShellWindow::slotKSLoadCanceled( const QString & errMsg )
 void KoShellWindow::saveAll()
 {
   KoView *currentView = (*m_activePage).m_pView;
-  for (QValueList<Page>::iterator it=m_lstPages.begin(); it != m_lstPages.end(); it++)
+  for (QValueList<Page>::iterator it=m_lstPages.begin(); it != m_lstPages.end(); ++it)
   {
     if ( (*it).m_pDoc->isModified() )
     {
@@ -373,7 +368,7 @@ void KoShellWindow::slotNewDocumentName()
 
 void KoShellWindow::updateCaption()
 {
-    kdDebug() << "KoShellWindow::updateCaption() rootDoc=" << rootDocument() << endl;
+    //kdDebug() << "KoShellWindow::updateCaption() rootDoc=" << rootDocument() << endl;
     KoMainWindow::updateCaption();
     // Let's take this opportunity for setting a correct name for the icon
     // in koolbar
@@ -382,7 +377,7 @@ void KoShellWindow::updateCaption()
     {
       if ( (*it).m_pDoc == rootDocument() )
       {
-        kdDebug() << "updateCaption called for " << rootDocument() << endl;
+        //kdDebug() << "updateCaption called for " << rootDocument() << endl;
         // Get caption from document info (title(), in about page)
         QString name;
         if ( rootDocument()->documentInfo() )
@@ -421,7 +416,7 @@ void KoShellWindow::slotSidebar_Part(int _item)
   kapp->restoreOverrideCursor();
   if (doc)
   {
-    if ( doc->initDoc( KoDocument::InitDocAppStarting) )
+    if ( doc->showEmbedInitDialog( this ) )
     {
       partManager()->addPart( doc, false );
       setRootDocument( doc );
@@ -499,6 +494,9 @@ void KoShellWindow::switchToPage( QValueList<Page>::Iterator it )
   // Fix caption and set focus to the new view
   updateCaption();
   v->setFocus();
+
+  partSpecificHelpAction->setEnabled(true);
+  partSpecificHelpAction->setText(i18n("%1 Handbook").arg((*m_activePage).m_pDoc->instance()->aboutData()->programName()));
 }
 
 void KoShellWindow::slotFileNew()
@@ -509,7 +507,7 @@ void KoShellWindow::slotFileNew()
     KoDocument* newdoc = m_documentEntry.createDoc();
     if ( !newdoc )
         return;
-    if ( !newdoc->initDoc(KoDocument::InitDocFileNew) )
+    if ( !newdoc->showEmbedInitDialog( this ) )
     {
       delete newdoc;
       return;
@@ -593,6 +591,8 @@ void KoShellWindow::closeDocument()
       setRootDocument( 0L );
       partManager()->setActivePart( 0L, 0L );
       mnuSaveAll->setEnabled(false);
+      partSpecificHelpAction->setEnabled(false);
+      partSpecificHelpAction->setText(i18n("Part Handbook"));
     }
 
     // Now delete the old view and page
@@ -663,20 +663,35 @@ QString KoShellWindow::configFile() const
 void KoShellWindow::tab_contextMenu(QWidget * w,const QPoint &p)
 {
   KPopupMenu menu;
-  int const mnuClose = menu.insertItem( i18n("Close") );
+  KIconLoader il;
+  int const mnuSave = menu.insertItem( il.loadIconSet( "filesave", KIcon::Small ), i18n("Save") );
+  int const mnuClose = menu.insertItem( il.loadIcon( "fileclose", KIcon::Small ), i18n("Close") );
+  
+  int tabnr = m_pFrame->indexOf( w );
+  Page page = m_lstPages[tabnr];
+  // disable save if there's nothing to save
+  if ( !page.m_pDoc->isModified() )
+    menu.setItemEnabled( mnuSave, false );
+  
+  // show menu
   int const choice = menu.exec(p);
 
-  if ( choice == mnuClose )
+  if( choice == mnuClose )
   {
     const int index = m_pFrame->currentPageIndex();
-    m_pFrame->setCurrentPage( m_pFrame->indexOf(w) );
+    m_pFrame->setCurrentPage( tabnr );
     slotFileClose();
     if ( index > m_pFrame->currentPageIndex() )
-        m_pFrame->setCurrentPage(index-1);
+      m_pFrame->setCurrentPage(index-1);
     else
-        m_pFrame->setCurrentPage(index);
+      m_pFrame->setCurrentPage(index);
+  }
+  else if ( choice == mnuSave )
+  {
+      page.m_pView->shell()->slotFileSave();
   }
 }
+
 void KoShellWindow::slotConfigureKeys()
 {
   KoView *view = rootView();
@@ -689,27 +704,29 @@ void KoShellWindow::slotConfigureKeys()
   dlg.configure();
 }
 
-/*
-void KoShellWindow::slotFilePrint()
-{
-  assert( m_activePage != m_lstPages.end() );
-
-  return (*m_activePage).m_vView->printDlg();
-}
-*/
-
 void KoShellWindow::createShellGUI( bool  )
 {
 	guiFactory()->addClient( m_client );
 }
 
+void KoShellWindow::showPartSpecificHelp()
+{
+  if((m_activePage == m_lstPages.end()) || ((*m_activePage).m_pDoc == 0))
+    return;
+
+  kapp->invokeHelp("", (*m_activePage).m_pDoc->instance()->aboutData()->appName(), "");
+}
+
 
 ///////////////////
 KoShellGUIClient::KoShellGUIClient( KoShellWindow *window ) : KXMLGUIClient()
 {
-	setXMLFile( "koshellui.rc", true, true );
-    window->mnuSaveAll = new KAction(i18n("Save All"), 0, window, SLOT(saveAll() ), actionCollection(), "save_all");
-    window->mnuSaveAll->setEnabled(false);
+  setXMLFile( "koshellui.rc", true, true );
+  window->mnuSaveAll = new KAction( i18n("Save All"), 0, window, SLOT( saveAll() ), actionCollection(), "save_all" );
+  window->mnuSaveAll->setEnabled(false);
+  window->partSpecificHelpAction = new KAction(i18n("Part Handbook"), "contents", 0, window, SLOT(showPartSpecificHelp()),
+                                               actionCollection(), "partSpecificHelp");
+  window->partSpecificHelpAction->setEnabled(false);
 }
 
 #include "koshell_shell.moc"
