@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 
@@ -27,7 +27,7 @@
 #include "kspread_canvas.h"
 #include "kspread_cell.h"
 #include "kspread_doc.h"
-#include "kspread_selection.h"
+#include "selection.h"
 #include "kspread_sheet.h"
 #include "kspread_style.h"
 #include "kspread_style_manager.h"
@@ -39,13 +39,15 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 
-#include <koGlobal.h>
+#include <KoGlobal.h>
 
 #include <qgroupbox.h>
 #include <qlabel.h>
 #include <qlayout.h>
 
-KSpreadConditionalWidget::KSpreadConditionalWidget( QWidget* parent, const char* name, WFlags fl )
+using namespace KSpread;
+
+ConditionalWidget::ConditionalWidget( QWidget* parent, const char* name, WFlags fl )
     : QWidget( parent, name, fl )
 {
   QGridLayout * Form1Layout = new QGridLayout( this, 1, 1, 11, 6, "Form1Layout");
@@ -192,11 +194,11 @@ KSpreadConditionalWidget::KSpreadConditionalWidget( QWidget* parent, const char*
   connect( m_condition_3, SIGNAL( highlighted( const QString & ) ), this, SLOT( slotTextChanged3( const QString & ) ) );
 }
 
-KSpreadConditionalWidget::~KSpreadConditionalWidget()
+ConditionalWidget::~ConditionalWidget()
 {
 }
 
-void KSpreadConditionalWidget::slotTextChanged1( const QString & text )
+void ConditionalWidget::slotTextChanged1( const QString & text )
 {
   if ( text == i18n( "<none>" ) )
   {
@@ -221,7 +223,7 @@ void KSpreadConditionalWidget::slotTextChanged1( const QString & text )
   }
 }
 
-void KSpreadConditionalWidget::slotTextChanged2( const QString & text )
+void ConditionalWidget::slotTextChanged2( const QString & text )
 {
   if ( text == i18n( "<none>" ) )
   {
@@ -246,7 +248,7 @@ void KSpreadConditionalWidget::slotTextChanged2( const QString & text )
   }
 }
 
-void KSpreadConditionalWidget::slotTextChanged3( const QString & text )
+void ConditionalWidget::slotTextChanged3( const QString & text )
 {
   if ( text == i18n( "<none>" ) )
   {
@@ -271,16 +273,15 @@ void KSpreadConditionalWidget::slotTextChanged3( const QString & text )
 }
 
 /**
- * Dialog
+ * ConditionalDialog
+ * Sets conditional cell formattings.
  */
-
-
-KSpreadConditionalDlg::KSpreadConditionalDlg( KSpreadView * parent, const char * name,
+ConditionalDialog::ConditionalDialog( View * parent, const char * name,
                                               const QRect & marker )
   : KDialogBase( parent, name, true, "", KDialogBase::Ok | KDialogBase::Cancel,
                  KDialogBase::Ok, false ),
     m_view( parent ),
-    m_dlg( new KSpreadConditionalWidget( this ) ),
+    m_dlg( new ConditionalWidget( this ) ),
     m_marker( marker )
 {
   QStringList list( m_view->doc()->styleManager()->styleNames() );
@@ -296,17 +297,17 @@ KSpreadConditionalDlg::KSpreadConditionalDlg( KSpreadView * parent, const char *
   init();
 }
 
-void KSpreadConditionalDlg::init()
+void ConditionalDialog::init()
 {
-  QValueList<KSpreadConditional> conditionList;
-  QValueList<KSpreadConditional> otherList;
+  QValueList<Conditional> conditionList;
+  QValueList<Conditional> otherList;
   bool found;
   int numCondition;
 
-  QValueList<KSpreadConditional>::iterator it1;
-  QValueList<KSpreadConditional>::iterator it2;
+  QValueList<Conditional>::iterator it1;
+  QValueList<Conditional>::iterator it2;
 
-  KSpreadCell * obj = m_view->activeSheet()->cellAt( m_marker.left(),
+  Cell * obj = m_view->activeSheet()->cellAt( m_marker.left(),
                                                      m_marker.top() );
 
   conditionList = obj->conditionList();
@@ -317,7 +318,7 @@ void KSpreadConditionalDlg::init()
   {
     for ( int y = m_marker.top(); y <= m_marker.bottom(); y++ )
     {
-      KSpreadCell * obj2 = m_view->activeSheet()->cellAt( x, y );
+      Cell * obj2 = m_view->activeSheet()->cellAt( x, y );
       otherList = obj2->conditionList();
 
       it1 = conditionList.begin();
@@ -411,7 +412,7 @@ void KSpreadConditionalDlg::init()
   }
 }
 
-void KSpreadConditionalDlg::init( KSpreadConditional const & tmp, int numCondition )
+void ConditionalDialog::init( Conditional const & tmp, int numCondition )
 {
   kdDebug() << "Adding " << numCondition << endl;
   QComboBox * cb  = 0;
@@ -450,30 +451,30 @@ void KSpreadConditionalDlg::init( KSpreadConditional const & tmp, int numConditi
 
   switch( tmp.cond )
   {
-   case None :
+    case Conditional::None :
     break;
 
-   case Equal :
+    case Conditional::Equal :
     cb->setCurrentItem( 1 );
     break;
 
-   case Superior :
+    case Conditional::Superior :
     cb->setCurrentItem( 2 );
     break;
 
-   case Inferior :
+    case Conditional::Inferior :
     cb->setCurrentItem( 3 );
     break;
 
-   case SuperiorEqual :
+    case Conditional::SuperiorEqual :
     cb->setCurrentItem( 4 );
     break;
 
-   case InferiorEqual :
+    case Conditional::InferiorEqual :
     cb->setCurrentItem( 5 );
     break;
 
-   case Between :
+    case Conditional::Between :
     cb->setCurrentItem(6);
 
     if ( tmp.strVal2 )
@@ -485,7 +486,7 @@ void KSpreadConditionalDlg::init( KSpreadConditional const & tmp, int numConditi
     }
     break;
 
-   case Different :
+    case Conditional::Different :
     cb->setCurrentItem(7);
     if ( tmp.strVal2 )
       kl2->setText( *tmp.strVal2 );
@@ -495,12 +496,12 @@ void KSpreadConditionalDlg::init( KSpreadConditional const & tmp, int numConditi
       kl2->setText( value );
     }
     break;
-   case DifferentTo :
+    case Conditional::DifferentTo :
     cb->setCurrentItem(8);
     break;
   }
 
-  if ( tmp.cond != None )
+  if ( tmp.cond != Conditional::None )
   {
     kl1->setEnabled( true );
 
@@ -514,37 +515,37 @@ void KSpreadConditionalDlg::init( KSpreadConditional const & tmp, int numConditi
   }
 }
 
-Conditional KSpreadConditionalDlg::typeOfCondition( QComboBox const * const cb ) const
+Conditional::Type ConditionalDialog::typeOfCondition( QComboBox const * const cb ) const
 {
-  Conditional result = None;
+  Conditional::Type result = Conditional::None;
   switch( cb->currentItem() )
   {
    case 0 :
-    result = None;
+     result = Conditional::None;
     break;
    case 1 :
-    result = Equal;
+     result = Conditional::Equal;
     break;
    case 2 :
-    result = Superior;
+     result = Conditional::Superior;
     break;
    case 3 :
-    result = Inferior;
+     result = Conditional::Inferior;
     break;
    case 4 :
-    result = SuperiorEqual;
+     result = Conditional::SuperiorEqual;
     break;
    case 5 :
-    result = InferiorEqual;
+     result = Conditional::InferiorEqual;
     break;
    case 6 :
-    result = Between;
+     result = Conditional::Between;
     break;
    case 7 :
-    result = Different;
+     result = Conditional::Different;
     break;
    case 8 :
-    result = DifferentTo;
+     result = Conditional::DifferentTo;
     break;
    default:
     kdDebug(36001) << "Error in list" << endl;
@@ -554,7 +555,7 @@ Conditional KSpreadConditionalDlg::typeOfCondition( QComboBox const * const cb )
   return result;
 }
 
-bool KSpreadConditionalDlg::checkInputData( KLineEdit const * const edit1,
+bool ConditionalDialog::checkInputData( KLineEdit const * const edit1,
                                             KLineEdit const * const edit2 )
 {
   bool b1 = false;
@@ -578,7 +579,7 @@ bool KSpreadConditionalDlg::checkInputData( KLineEdit const * const edit1,
   return true;
 }
 
-bool KSpreadConditionalDlg::checkInputData()
+bool ConditionalDialog::checkInputData()
 {
   if ( m_dlg->m_firstValue_1->isEnabled() && !checkInputData( m_dlg->m_firstValue_1, m_dlg->m_secondValue_1 ) )
     return false;
@@ -590,15 +591,15 @@ bool KSpreadConditionalDlg::checkInputData()
   return true;
 }
 
-bool KSpreadConditionalDlg::getCondition( KSpreadConditional & newCondition, const QComboBox * cb,
+bool ConditionalDialog::getCondition( Conditional & newCondition, const QComboBox * cb,
                                           const KLineEdit * edit1, const KLineEdit * edit2,
-                                          const QComboBox * sb, KSpreadStyle * style )
+                                          const QComboBox * sb, Style * style )
 {
   if ( !cb->isEnabled() )
     return false;
 
   newCondition.cond = typeOfCondition( cb );
-  if ( newCondition.cond == None )
+  if ( newCondition.cond == Conditional::None )
     return false;
 
   bool ok = false;
@@ -636,7 +637,7 @@ bool KSpreadConditionalDlg::getCondition( KSpreadConditional & newCondition, con
   return true;
 }
 
-void KSpreadConditionalDlg::slotOk()
+void ConditionalDialog::slotOk()
 {
   kdDebug() << "slotOk" << endl;
 
@@ -646,11 +647,11 @@ void KSpreadConditionalDlg::slotOk()
   kdDebug() << "Input data is valid" << endl;
 
   m_view->doc()->emitBeginOperation( false );
-  KSpreadStyleManager * manager = m_view->doc()->styleManager();
+  StyleManager * manager = m_view->doc()->styleManager();
 
-  QValueList<KSpreadConditional> newList;
+  QValueList<Conditional> newList;
 
-  KSpreadConditional newCondition;
+  Conditional newCondition;
 
   if ( getCondition( newCondition, m_dlg->m_condition_1, m_dlg->m_firstValue_1,
                      m_dlg->m_secondValue_1, m_dlg->m_style_1, manager->style( m_dlg->m_style_1->currentText() ) ) )
@@ -666,7 +667,7 @@ void KSpreadConditionalDlg::slotOk()
 
   kdDebug() << "Setting conditional list" << endl;
   m_view->activeSheet()->setConditional( m_view->selectionInfo(), newList );
-  m_view->slotUpdateView( m_view->activeSheet(), m_view->selectionInfo()->selection() );
+  m_view->slotUpdateView( m_view->activeSheet(), *m_view->selectionInfo() );
 
   accept();
 }
