@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2003,2004 Ariya Hidayat <ariya@kde.org>
+   Copyright (C) 2005 Tomas Mecir <mecirt@gmail.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -13,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include "formula.h"
@@ -43,7 +44,7 @@
 /*
   To understand how this formula engine works, please refer to the documentation
   in file DESIGN.html.
-      
+
   Useful references:
    - "Principles of Compiler Design", A.V.Aho, J.D.Ullman, Addison Wesley, 1978
    - "Writing Interactive Compilers and Interpreters", P.J. Brown,
@@ -59,7 +60,6 @@
 
  /*
 TODO - features:
- - array/list for function arguments
  - handle Intersection
  - cell reference is made relative (absolute now)
  - shared formula (different owner, same data)
@@ -94,12 +94,13 @@ class Formula::Private
 {
 public:
   Formula *formula;
-  KSpreadCell *cell;
+  Cell *cell;
+  Sheet *sheet;
   bool dirty;
   bool valid;
   QString expression;
   QValueVector<Opcode> codes;
-  QValueVector<KSpreadValue> constants;
+  QValueVector<Value> constants;
 };
 
 class TokenStack : public QValueVector<Token>
@@ -126,7 +127,7 @@ const Token Token::null;
 
 // helper function: return operator of given token text
 // e.g. "*" yields Operator::Asterisk, and so on
-static Token::Op matchOperator( const QString& text )
+Token::Op KSpread::matchOperator( const QString& text )
 {
   Token::Op result = Token::InvalidOp;
 
@@ -158,9 +159,6 @@ static Token::Op matchOperator( const QString& text )
     if( text == "<>" ) result = Token::NotEqual;
     if( text == "<=" ) result = Token::LessEqual;
     if( text == ">=" ) result = Token::GreaterEqual;
-
-    // This is for old KScript backward compatibility
-    // Remove in KOffice 2.0
     if( text == "==" ) result = Token::Equal;
   }
 
@@ -196,13 +194,13 @@ static int opPrecedence( Token::Op op )
 }
 
 // helper function
-static KSpreadValue tokenAsValue( const Token& token )
+static Value tokenAsValue( const Token& token )
 {
-  KSpreadValue value;
-  if( token.isBoolean() ) value = KSpreadValue( token.asBoolean() );
-  else if( token.isInteger() ) value = KSpreadValue( token.asInteger() );
-  else if( token.isFloat() ) value = KSpreadValue( token.asFloat() );
-  else if( token.isString() ) value = KSpreadValue( token.asString() );
+  Value value;
+  if( token.isBoolean() ) value = Value( token.asBoolean() );
+  else if( token.isInteger() ) value = Value( token.asInteger() );
+  else if( token.isFloat() ) value = Value( token.asFloat() );
+  else if( token.isString() ) value = Value( token.asString() );
   return value;
 }
 
@@ -357,7 +355,7 @@ void TokenStack::ensureSpace()
  **********************/
 
 // helper function: return true for valid identifier character
-static bool isIdentifier( QChar ch )
+bool KSpread::isIdentifier( QChar ch )
 {
   return ( ch.unicode() == '_' ) || (ch.unicode() == '$' ) || ( ch.isLetter() );
 }
@@ -371,10 +369,11 @@ static bool isIdentifier( QChar ch )
 
 // Constructor
 
-Formula::Formula( KSpreadCell *cell )
+Formula::Formula (Sheet *sheet, Cell *cell)
 {
   d = new Private;
   d->cell = cell;
+  d->sheet = sheet;
   clear();
 }
 
@@ -382,6 +381,7 @@ Formula::Formula()
 {
   d = new Private;
   d->cell = 0;
+  d->sheet = 0;
   clear();
 }
 
@@ -392,10 +392,15 @@ Formula::~Formula()
   delete d;
 }
 
-KSpreadCell* Formula::cell()
+Cell* Formula::cell() const
 {
   return d->cell;
-}  
+}
+
+Sheet* Formula::sheet() const
+{
+  return d->sheet;
+}
 
 // Sets a new expression for this formula.
 // note that both the real lex and parse processes will happen later on
@@ -423,6 +428,8 @@ bool Formula::isValid() const
   if( d->dirty )
   {
     KLocale* locale = d->cell ? d->cell->locale() : 0;
+    if ((!locale) && d->sheet)
+      locale = d->sheet->doc()->locale();
     Tokens tokens = scan( d->expression, locale );
     if( tokens.valid() )
       compile( tokens );
@@ -451,10 +458,12 @@ void Formula::clear()
 Tokens Formula::tokens() const
 {
   KLocale* locale = d->cell ? d->cell->locale() : 0;
+  if ((!locale) && d->sheet)
+    locale = d->sheet->doc()->locale();
   return scan( d->expression, locale );
 }
 
-Tokens Formula::scan( const QString& expr, KLocale* locale )
+Tokens Formula::scan( const QString& expr, KLocale* locale ) const
 {
   // to hold the result
   Tokens tokens;
@@ -473,14 +482,14 @@ Tokens Formula::scan( const QString& expr, KLocale* locale )
   QString ex = expr;
   QString tokenText;
   int tokenStart = 0;
-  
+
   // first character must be equal sign (=)
   if( ex[0] != '=' )
     return tokens;
 
   // but the scanner should not see this equal sign
-  ex.remove( 0, 1 );  
-    
+  ex.remove( 0, 1 );
+
   // force a terminator
   ex.append( QChar() );
 
@@ -581,30 +590,34 @@ Tokens Formula::scan( const QString& expr, KLocale* locale )
        // we're done with identifier
        else
        {
-         // check for "true" or "false"
-         if( ( tokenText.lower() == "true" ) || ( tokenText.lower() == "false" ) )
+         // check for cell reference,  e.g A1, VV123, ...
+         QRegExp exp("(\\$?)([a-zA-Z]+)(\\$?)([0-9]+)$");
+         int n = exp.search( tokenText );
+         if( n >= 0 )
+           state = InCell;
+         else
          {
-           tokens.append( Token( Token::Boolean, tokenText, tokenStart ) );
+           bool gotNamed = false;
+           // check for named areas ...
+           if (d->sheet) {
+             const QValueList<Reference> areas = d->sheet->doc()->listArea();
+             QValueList<Reference>::const_iterator it;
+             for (it = areas.begin(); it != areas.end(); ++it) {
+               if ((*it).ref_name.lower() == tokenText.lower()) {
+                 // we got a named area
+                 tokens.append (Token (Token::Range, tokenText, tokenStart));
+                 gotNamed = true;
+                 break;
+                }
+              }
+           }
+           if (!gotNamed)
+             tokens.append (Token (Token::Identifier, tokenText,
+               tokenStart));
            tokenStart = i;
            tokenText = "";
            state = Start;
          }
-
-         else
-         {
-           // check for cell reference,  e.g A1, VV123, ...
-           QRegExp exp("(\\$?)([a-zA-Z]+)(\\$?)([0-9]+)$");
-           int n = exp.search( tokenText );
-           if( n >= 0 )
-             state = InCell;
-           else
-           {
-             tokens.append( Token( Token::Identifier, tokenText, tokenStart ) );
-             tokenStart = i;
-             tokenText = "";
-             state = Start;
-           }
-          }
        }
        break;
 
@@ -630,6 +643,7 @@ Tokens Formula::scan( const QString& expr, KLocale* locale )
          {
 
            // we're done with named area
+           // (Tomas) huh? this doesn't seem to check for named areas ...
            tokens.append( Token( Token::Range, tokenText, tokenStart ) );
            tokenText = "";
            state = Start;
@@ -677,14 +691,37 @@ Tokens Formula::scan( const QString& expr, KLocale* locale )
 
        else
        {
-         // must be followed by '!'
+         // must be followed by '!', otherwise we have a string in ''
          i++;
          if( ex[i] == '!' )
          {
            tokenText.append( ex[i++] );
            state = InCell;
          }
-         else state = Bad;
+         else {
+	   // this is the same as the check in InIdentifier ... TODO merge
+           bool gotNamed = false;
+           // check for named areas ...
+           if (d->sheet) {
+	     QString txt = tokenText.mid(1, tokenText.length() - 2).lower();
+             const QValueList<Reference> areas = d->sheet->doc()->listArea();
+             QValueList<Reference>::const_iterator it;
+             for (it = areas.begin(); it != areas.end(); ++it) {
+               if ((*it).ref_name.lower() == txt) {
+                 // we got a named area
+                 tokens.append (Token (Token::Range, tokenText, tokenStart));
+                 gotNamed = true;
+                 break;
+                }
+              }
+           }
+           if (!gotNamed)
+             tokens.append (Token (Token::Identifier, tokenText,
+               tokenStart));
+           tokenStart = i;
+           tokenText = "";
+           state = Start;
+	 }
        }
        break;
 
@@ -773,7 +810,7 @@ Tokens Formula::scan( const QString& expr, KLocale* locale )
 
     case InString:
 
-       // consume until "'
+       // consume until "
        if( ch != '"' ) tokenText.append( ex[i++] );
 
        else
@@ -790,8 +827,8 @@ Tokens Formula::scan( const QString& expr, KLocale* locale )
        break;
     };
   };
-  
-  if( state == Bad ) 
+
+  if( state == Bad )
     tokens.setValid( false );
 
   return tokens;
@@ -838,7 +875,7 @@ void Formula::compile( const Tokens& tokens ) const
     ( tokenType == Token::Identifier ) )
     {
       syntaxStack.push( token );
-      d->constants.append( KSpreadValue( token.text() ) );
+      d->constants.append( Value( token.text() ) );
       if (tokenType == Token::Cell)
         d->codes.append( Opcode( Opcode::Cell, d->constants.count()-1 ) );
       else if (tokenType == Token::Range)
@@ -870,7 +907,7 @@ void Formula::compile( const Tokens& tokens ) const
     if( syntaxStack.itemCount() >= 1 )
     if( !syntaxStack.top().isOperator() )
     {
-      d->constants.append( KSpreadValue( 0.01 ) );
+      d->constants.append( Value( 0.01 ) );
       d->codes.append( Opcode( Opcode::Load, d->constants.count()-1 ) );
       d->codes.append( Opcode( Opcode::Mul ) );
     }
@@ -1112,22 +1149,29 @@ void Formula::compile( const Tokens& tokens ) const
 
 // Evaluates the formula, returns the result.
 
-KSpreadValue Formula::eval() const
-{
-  QValueStack<KSpreadValue> stack;
-  unsigned index;
-  KSpreadValue val1, val2;
-  QString c;
-  QValueVector<KSpreadValue> args;
+struct stackEntry {
+  void reset () { row1 = col1 = row2 = col2 = -1; };
+  Value val;
+  int row1, col1, row2, col2;
+};
 
-  KSpreadSheet *sheet = 0;
+Value Formula::eval() const
+{
+  QValueStack<stackEntry> stack;
+  stackEntry entry;
+  unsigned index;
+  Value val1, val2;
+  QString c;
+  QValueVector<Value> args;
+
+  Sheet *sheet = 0;
   ValueParser* parser = 0;
   ValueConverter* converter = 0;
   ValueCalc* calc = 0;
-  
-  if( d->cell )
+
+  if (d->sheet)
   {
-    sheet = d->cell->sheet();
+    sheet = d->sheet;
     converter = sheet->doc()->converter();
     calc = sheet->doc()->calc();
   }
@@ -1137,10 +1181,15 @@ KSpreadValue Formula::eval() const
     converter = new ValueConverter( parser );
     calc = new ValueCalc( converter );
   }
-  
 
   Function* function;
-  
+  FuncExtra fe;
+  fe.mycol = fe.myrow = 0;
+  if (d->cell) {
+    fe.mycol = d->cell->column();
+    fe.myrow = d->cell->row();
+  }
+
   if( d->dirty )
   {
     Tokens tokens = scan( d->expression );
@@ -1148,12 +1197,13 @@ KSpreadValue Formula::eval() const
     if( tokens.valid() )
       compile( tokens );
   }
-  
+
   if( !d->valid )
-    return KSpreadValue::errorVALUE();
+    return Value::errorVALUE();
 
   for( unsigned pc = 0; pc < d->codes.count(); pc++ )
   {
+    Value ret;   // for the function caller
     Opcode& opcode = d->codes[pc];
     index = opcode.index;
     switch( opcode.type )
@@ -1164,171 +1214,224 @@ KSpreadValue Formula::eval() const
 
       // load a constant, push to stack
       case Opcode::Load:
-        val1 = d->constants[index];
-        stack.push( val1 );
+        entry.reset();
+        entry.val = d->constants[index];
+        stack.push (entry);
         break;
 
       // unary operation
       case Opcode::Neg:
-        val1 = converter->asFloat (stack.pop());
-        if( val1.isError() ) return KSpreadValue::errorVALUE();
-        stack.push( -val1.asFloat() );
+        entry.reset();
+        entry.val = stack.pop().val;
+        if (!entry.val.isError()) // do nothing if we got an error
+          entry.val = calc->mul (entry.val, -1);
+        stack.push (entry);
         break;
 
       // binary operation: take two values from stack, do the operation,
       // push the result to stack
       case Opcode::Add:
-        val2 = stack.pop();
-        val1 = stack.pop();
+        entry.reset();
+        val2 = stack.pop().val;
+        val1 = stack.pop().val;
         val2 = calc->add( val1, val2 );
-        if( val2.isError() ) return val2;
-        stack.push( val2 );
+        entry.reset();
+        entry.val = val2;
+        stack.push (entry);
         break;
 
       case Opcode::Sub:
-        val2 = stack.pop();
-        val1 = stack.pop();
+        val2 = stack.pop().val;
+        val1 = stack.pop().val;
         val2 = calc->sub( val1, val2 );
-        if( val2.isError() ) return val2;
-        stack.push( val2 );
+        entry.reset();
+        entry.val = val2;
+        stack.push (entry);
         break;
 
       case Opcode::Mul:
-        val2 = stack.pop();
-        val1 = stack.pop();
+        val2 = stack.pop().val;
+        val1 = stack.pop().val;
         val2 = calc->mul( val1, val2 );
-        if( val2.isError() ) return val2;
-        stack.push( val2 );
+        entry.reset();
+        entry.val = val2;
+        stack.push (entry);
         break;
 
       case Opcode::Div:
-        val2 = stack.pop();
-        val1 = stack.pop();
-        if( val1.isZero() ) return KSpreadValue::errorDIV0();
+        val2 = stack.pop().val;
+        val1 = stack.pop().val;
         val2 = calc->div( val1, val2 );
-        if( val2.isError() ) return val2;
-        stack.push( val2 );
+        entry.reset();
+        entry.val = val2;
+        stack.push (entry);
         break;
 
       case Opcode::Pow:
-        val2 = stack.pop();
-        val1 = stack.pop();
+        val2 = stack.pop().val;
+        val1 = stack.pop().val;
         val2 = calc->pow( val1, val2 );
-        if( val2.isError() ) return val2;
-        stack.push( val2 );
+        entry.reset();
+        entry.val = val2;
+        stack.push (entry);
         break;
 
       // string concatenation
       case Opcode::Concat:
-        val1 = converter->asString( stack.pop());
-        if( val1.isError() ) return KSpreadValue::errorVALUE();
-        val2 = converter->asString( stack.pop());
-        if( val2.isError() ) return KSpreadValue::errorVALUE();
-        val1.setValue( val2.asString().append( val1.asString() ) );
-        stack.push( val1 );
+        val1 = converter->asString (stack.pop().val);
+	val2 = converter->asString (stack.pop().val);
+        if (val1.isError() || val2.isError())
+	  val1 = Value::errorVALUE();
+	else
+          val1.setValue( val2.asString().append( val1.asString() ) );
+        entry.reset();
+        entry.val = val1;
+        stack.push (entry);
         break;
 
       // logical not
       case Opcode::Not:
-        val1 = converter->asBoolean( stack.pop());
-        if( val1.isError() ) return KSpreadValue::errorVALUE();
-        val1.setValue( !val1.asBoolean() );
-        stack.push( val1 );
+        val1 = converter->asBoolean (stack.pop().val);
+        if( val1.isError() )
+	  val1 = Value::errorVALUE();
+	else
+	  val1.setValue( !val1.asBoolean() );
+        entry.reset();
+        entry.val = val1;
+        stack.push (entry);
         break;
 
       // comparison
       case Opcode::Equal:
-        val1 = stack.pop();
-        if( val1.isError() ) return KSpreadValue::errorVALUE();
-        val2 = stack.pop();
-        if( val2.isError() ) return KSpreadValue::errorVALUE();
+        val1 = stack.pop().val;
+        val2 = stack.pop().val;
         if( !val1.allowComparison( val2 ) )
-          return KSpreadValue::errorVALUE();
-        if( val2.compare( val1 ) == 0 )
-          stack.push( KSpreadValue( true ) );
+          val1 = Value::errorNA();
+	else if( val2.compare( val1 ) == 0 )
+          val1 = Value (true);
         else
-          stack.push( KSpreadValue( false ) );
+          val1 = Value (false);
+        entry.reset();
+        entry.val = val1;
+        stack.push (entry);
         break;
 
       // less than
       case Opcode::Less:
-        val1 = stack.pop();
-        if( val1.isError() ) return KSpreadValue::errorVALUE();
-        val2 = stack.pop();
-        if( val2.isError() ) return KSpreadValue::errorVALUE();
+        val1 = stack.pop().val;
+        val2 = stack.pop().val;
         if( !val1.allowComparison( val2 ) )
-          return KSpreadValue::errorVALUE();
-        if( val2.compare( val1 ) < 0 )
-          stack.push( KSpreadValue( true ) );
+          val1 = Value::errorNA();
+	else if( val2.compare( val1 ) < 0 )
+          val1 = Value (true);
         else
-          stack.push( KSpreadValue( false ) );
+          val1 = Value (false);
+        entry.reset();
+        entry.val = val1;
+        stack.push (entry);
         break;
 
       // greater than
       case Opcode::Greater:
-        val1 = stack.pop();
-        if( val1.isError() ) return KSpreadValue::errorVALUE();
-        val2 = stack.pop();
-        if( val2.isError() ) return KSpreadValue::errorVALUE();
+        val1 = stack.pop().val;
+        val2 = stack.pop().val;
         if( !val1.allowComparison( val2 ) )
-          return KSpreadValue::errorVALUE();
-        if( val2.compare( val1 ) > 0 )
-          stack.push( KSpreadValue( true ) );
+          val1 = Value::errorNA();
+	else if( val2.compare( val1 ) > 0 )
+          val1 = Value (true);
         else
-          stack.push( KSpreadValue( false ) );
+          val1 = Value (false);
+        entry.reset();
+        entry.val = val1;
+        stack.push (entry);
         break;
 
 
       case Opcode::Cell:
         c = d->constants[index].asString();
-        val1 = KSpreadValue::empty();
+        val1 = Value::empty();
+        entry.reset();
         if (sheet)
         {
-          KSpreadPoint cell (c, sheet->workbook(), sheet);
+          Point cell (c, sheet->workbook(), sheet);
           if (cell.isValid())
-            val1 = cell.sheet->value (cell.column(), cell.row());
+          {
+            val1 = cell.sheet()->value (cell.column(), cell.row());
+            // store the reference, so we can use it within functions
+            entry.col1 = entry.col2 = cell.column();
+            entry.row1 = entry.row2 = cell.row();
+          }
         }
-        stack.push( val1 );
+        entry.val = val1;
+        stack.push (entry);
         break;
-      
+
       case Opcode::Range:
         c = d->constants[index].asString();
-        val1 = KSpreadValue::empty();
+        val1 = Value::empty();
+        entry.reset();
         if (sheet)
         {
-          KSpreadRange range (c, sheet->workbook(), sheet);
+          Range range (c, sheet->workbook(), sheet);
           if (range.isValid())
-            val1 = range.sheet->valueRange (range.startCol(), range.startRow(),
+          {
+            val1 = range.sheet()->valueRange (range.startCol(), range.startRow(),
                 range.endCol(), range.endRow());
+            // store the reference, so we can use it within functions
+            entry.col1 = range.startCol();
+            entry.row1 = range.startRow();
+            entry.col2 = range.endCol();
+            entry.row2 = range.endRow();
+          }
         }
-        stack.push( val1 );
+        entry.val = val1;
+        stack.push (entry);
         break;
-      
+
       case Opcode::Ref:
         val1 = d->constants[index];
-        stack.push( val1 );
+        entry.reset();
+        entry.val = val1;
+        stack.push (entry);
         break;
 
       // calling function
       case Opcode::Function:
-        return KSpreadValue::errorVALUE();
-        
         if( stack.count() < index )
           // (Tomas) umm, how could that be ? I mean, the index value
           //  is computed from the stack *confused*
-          return KSpreadValue::errorVALUE(); // not enough arguments
+          return Value::errorVALUE(); // not enough arguments
+
         args.clear();
+        fe.ranges.clear ();
+        fe.ranges.reserve (index);
+        fe.sheet = sheet;
         for( ; index; index-- )
-          args.insert( args.begin(), stack.pop() );
+        {
+          stackEntry e = stack.pop();
+          args.insert (args.begin(), e.val);
+          // TODO: create and fill a FunctionExtra object, if needed
+          // problem: we don't know if we need it, as we don't have the
+          // fuction name yet ...
+          fe.ranges[index - 1].col1 = e.col1;
+          fe.ranges[index - 1].row1 = e.row1;
+          fe.ranges[index - 1].col2 = e.col2;
+          fe.ranges[index - 1].row2 = e.row2;
+        }
+
         // function name as string value
-        val1 = converter->asString (stack.pop());
+        val1 = converter->asString (stack.pop().val);
         if( val1.isError() )
-          return KSpreadValue::errorVALUE();
+          return Value::errorVALUE();
         function = FunctionRepository::self()->function ( val1.asString() );
         if( !function )
-          return KSpreadValue::errorVALUE(); // no such function
-        stack.push( function->exec( this, args ) );
-        
+          return Value::errorVALUE(); // no such function
+
+        ret = function->exec (args, calc, &fe);
+        entry.reset();
+        entry.val = ret;
+        stack.push (entry);
+
         break;
 
       default:
@@ -1336,11 +1439,17 @@ KSpreadValue Formula::eval() const
     }
   }
 
+  if (!d->sheet) {
+    delete parser;
+    delete converter;
+    delete calc;
+  }
+
   // more than one value in stack ? unsuccesful execution...
   if( stack.count() != 1 )
-    return KSpreadValue::errorVALUE();
+    return Value::errorVALUE();
 
-  return stack.pop();
+  return stack.pop().val;
 
 }
 
@@ -1357,8 +1466,8 @@ QString Formula::dump() const
   }
 
   result = QString("Expression: [%1]\n").arg( d->expression );
-#if 0  
-  KSpreadValue value = eval();
+#if 0
+  Value value = eval();
   result.append( QString("Result: %1\n").arg(
       converter->asString(value).asString() ) );
 #endif
@@ -1367,7 +1476,7 @@ QString Formula::dump() const
   for( unsigned c = 0; c < d->constants.count(); c++ )
   {
     QString vtext;
-    KSpreadValue val = d->constants[c];
+    Value val = d->constants[c];
     if( val.isString() ) vtext = QString("[%1]").arg( val.asString() );
     else if( val.isNumber() ) vtext = QString("%1").arg( val.asFloat() );
     else if( val.isBoolean() ) vtext = QString("%1").arg( val.asBoolean() ? "True":"False");
@@ -1405,7 +1514,7 @@ QString Formula::dump() const
   return result;
 }
 
-QTextStream& operator<<( QTextStream& ts, Formula formula ) 
+QTextStream& operator<<( QTextStream& ts, Formula formula )
 {
   ts << formula.dump();
   return ts;
