@@ -1,7 +1,16 @@
 /* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-   Copyright (C) 1999 - 2003 The KSpread Team
-                             www.koffice.org/kspread
+
+   Copyright 2005 Raphael Langerhorst <raphael.langerhorst@kdemail.net>
+   Copyright 2002-2004 Ariya Hidayat <ariya@kde.org>
+   Copyright 2002-2003 Norbert Andres <nandres@web.de>
+   Copyright 2002 John Dailey <dailey@vt.edu>
+   Copyright 2001-2002 Philipp Mueller <philipp.mueller@gmx.de>
+   Copyright 2000-2002 Laurent Montel <montel@kde.org>
+   Copyright 2000-2001 Werner Trobin <trobin@kde.org>
+   Copyright 1999-2001 David Faure <faure@kde.org>
+   Copyright 1998-2000 Torben Weis <weis@kde.org>
+   Copyright 1998-1999 Stephan Kulow <coolo@kde.org>
+   Copyright 1998 Reginald Stadlbauer <reggie@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -15,18 +24,26 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
-#include "kspread_sheet.h"
-#include "kspread_undo.h"
-#include "kspread_doc.h"
-
 #include <math.h>
+
+#include <qregexp.h>
+
 #include <kconfig.h>
 #include <kdebug.h>
-#include <qregexp.h>
+
+#include "kspread_doc.h"
+#include "kspread_locale.h"
+#include "kspread_sheet.h"
+#include "kspread_undo.h"
+#include "kspread_value.h"
+#include "valueconverter.h"
+#include "kspread_autofill.h"
+
+using namespace KSpread;
 
 QStringList *AutoFillSequenceItem::month = 0L;
 QStringList *AutoFillSequenceItem::shortMonth = 0L;
@@ -40,12 +57,12 @@ QStringList *AutoFillSequenceItem::other = 0L;
  **********************************************************************************/
 
 AutoFillDeltaSequence::AutoFillDeltaSequence( AutoFillSequence *_first, AutoFillSequence *_next )
-  : m_ok(TRUE),
+  : m_ok(true),
     m_sequence(0L)
 {
   if ( _first->count() != _next->count() )
   {
-    m_ok = FALSE;
+    m_ok = false;
     return;
   }
 
@@ -60,7 +77,7 @@ AutoFillDeltaSequence::AutoFillDeltaSequence( AutoFillSequence *_first, AutoFill
     double d;
     if ( !item->getDelta( item2, d ) )
       {
-        m_ok = FALSE;
+        m_ok = false;
         return;
       }
     m_sequence->at( i++ ) = d;
@@ -77,19 +94,19 @@ AutoFillDeltaSequence::~AutoFillDeltaSequence()
 bool AutoFillDeltaSequence::equals( AutoFillDeltaSequence *_delta )
 {
   if ( m_sequence == 0L )
-    return FALSE;
+    return false;
   if ( _delta->getSequence() == 0L )
-    return FALSE;
+    return false;
   if ( m_sequence->size() != _delta->getSequence()->size() )
-    return FALSE;
+    return false;
 
   for ( unsigned int i = 0; i < m_sequence->size(); i++ )
   {
     if ( m_sequence->at( i ) != _delta->getSequence()->at( i ) )
-      return FALSE;
+      return false;
   }
 
-  return TRUE;
+  return true;
 }
 
 double AutoFillDeltaSequence::getItemDelta( int _pos )
@@ -184,7 +201,7 @@ AutoFillSequenceItem::AutoFillSequenceItem( const QString &_str )
     if( other==0L)
       {
 	//	other=new QStringList();
-	KConfig *config = KSpreadFactory::global()->config();
+	KConfig *config = Factory::global()->config();
 	config->setGroup( "Parameters" );
 	other=new QStringList(config->readListEntry("Other list"));
       }
@@ -249,24 +266,24 @@ AutoFillSequenceItem::AutoFillSequenceItem( const QString &_str )
 bool AutoFillSequenceItem::getDelta( AutoFillSequenceItem *seq, double &_delta )
 {
     if ( seq->getType() != m_Type )
-        return FALSE;
+        return false;
 
     switch( m_Type )
     {
     case INTEGER:
         _delta = (double)( seq->getIValue() - m_IValue );
-        return TRUE;
+        return true;
     case FLOAT:
         _delta = seq->getDValue() - m_DValue;
-        return TRUE;
+        return true;
     case FORMULA:
     case STRING:
         if ( m_String == seq->getString() )
         {
             _delta = 0.0;
-            return TRUE;
+            return true;
         }
-        return FALSE;
+        return false;
     case MONTH:
         {
             int i = month->findIndex( m_String );
@@ -277,7 +294,7 @@ bool AutoFillSequenceItem::getDelta( AutoFillSequenceItem *seq, double &_delta )
                 _delta = -1.0;
             else
                 _delta = ( double )( k - i );
-            return TRUE;
+            return true;
         }
 
     case SHORTMONTH:
@@ -290,7 +307,7 @@ bool AutoFillSequenceItem::getDelta( AutoFillSequenceItem *seq, double &_delta )
                 _delta = -1.0;
             else
                 _delta = ( double )( k - i );
-            return TRUE;
+            return true;
         }
 
     case DAY:
@@ -304,7 +321,7 @@ bool AutoFillSequenceItem::getDelta( AutoFillSequenceItem *seq, double &_delta )
             else
                 _delta = ( double )( k - i );
             kdDebug() << m_String << " i: " << i << " j: " << j << " k: " << k << " delta: " << _delta << endl;
-            return TRUE;
+            return true;
         }
 
     case SHORTDAY:
@@ -317,7 +334,7 @@ bool AutoFillSequenceItem::getDelta( AutoFillSequenceItem *seq, double &_delta )
                 _delta = -1.0;
             else
                 _delta = ( double )( k - i );
-            return TRUE;
+            return true;
         }
     case OTHER:
       {
@@ -332,10 +349,10 @@ bool AutoFillSequenceItem::getDelta( AutoFillSequenceItem *seq, double &_delta )
 	  _delta = -1.0;
 	  else*/
 	  _delta = ( double )( k - i );
-	return TRUE;
+	return true;
       }
      default:
-      return FALSE;
+      return false;
     }
 }
 
@@ -488,9 +505,9 @@ QString AutoFillSequenceItem::getPredecessor( int _no, double _delta )
  *
  **********************************************************************************/
 
-AutoFillSequence::AutoFillSequence( KSpreadCell *_cell )
+AutoFillSequence::AutoFillSequence( Cell *_cell )
 {
-    sequence.setAutoDelete( TRUE );
+    sequence.setAutoDelete( true );
 
     if ( _cell->isFormula() )
     {
@@ -514,15 +531,15 @@ bool AutoFillSequence::matches( AutoFillSequence* _seq, AutoFillDeltaSequence *_
 {
     AutoFillDeltaSequence delta( this, _seq );
     if ( !delta.isOk() )
-        return FALSE;
+        return false;
 
     if ( delta.equals( _delta ) )
-         return TRUE;
+         return true;
 
-    return FALSE;
+    return false;
 }
 
-void AutoFillSequence::fillCell( KSpreadCell *src, KSpreadCell *dest, AutoFillDeltaSequence *delta, int _block, bool down )
+void AutoFillSequence::fillCell( Cell *src, Cell *dest, AutoFillDeltaSequence *delta, int _block, bool down )
 {
     QString erg = "";
 
@@ -530,7 +547,7 @@ void AutoFillSequence::fillCell( KSpreadCell *src, KSpreadCell *dest, AutoFillDe
     if ( sequence.first() != 0L && sequence.first()->getType() == AutoFillSequenceItem::FORMULA )
     {
         QString f = dest->decodeFormula( sequence.first()->getString() );
-        dest->setCellText( f, true );
+        dest->setCellText( f );
         dest->copyFormat( src );
         return;
     }
@@ -548,29 +565,31 @@ void AutoFillSequence::fillCell( KSpreadCell *src, KSpreadCell *dest, AutoFillDe
         erg += item->getPredecessor( _block, delta->getItemDelta( i++ ) );
     }
 
-    dest->setCellText( erg, true );
+    dest->setCellText( erg );
     dest->copyFormat( src );
 }
 
 /**********************************************************************************
  *
- * KSpreadSheet
+ * Sheet
  *
  **********************************************************************************/
 
-void KSpreadSheet::autofill( QRect &src, QRect &dest )
+void Sheet::autofill( QRect &src, QRect &dest )
 {
     if (src == dest)
     {
         return;
     }
 
+    setRegionPaintDirty( dest );
+
     doc()->emitBeginOperation();
 
-    if ( !m_pDoc->undoBuffer()->isLocked() )
+    if ( !doc()->undoLocked() )
     {
-      KSpreadUndoAutofill *undo = new KSpreadUndoAutofill( m_pDoc, this, dest );
-      m_pDoc->undoBuffer()->appendUndo( undo );
+      UndoAutofill *undo = new UndoAutofill( doc(), this, dest );
+      doc()->addCommand( undo );
     }
 
     // Fill from left to right
@@ -579,14 +598,14 @@ void KSpreadSheet::autofill( QRect &src, QRect &dest )
         for ( int y = src.top(); y <= src.bottom(); y++ )
         {
             int x;
-            QPtrList<KSpreadCell> destList;
+            QPtrList<Cell> destList;
             for ( x = src.right() + 1; x <= dest.right(); x++ )
                 destList.append( nonDefaultCell( x, y ) );
-            QPtrList<KSpreadCell> srcList;
+            QPtrList<Cell> srcList;
             for ( x = src.left(); x <= src.right(); x++ )
                 srcList.append( cellAt( x, y ) );
             QPtrList<AutoFillSequence> seqList;
-            seqList.setAutoDelete( TRUE );
+            seqList.setAutoDelete( true );
             for ( x = src.left(); x <= src.right(); x++ )
                 seqList.append( new AutoFillSequence( cellAt( x, y ) ) );
             fillSequence( srcList, destList, seqList );
@@ -599,16 +618,16 @@ void KSpreadSheet::autofill( QRect &src, QRect &dest )
         for ( int x = src.left(); x <= dest.right(); x++ )
         {
             int y;
-            QPtrList<KSpreadCell> destList;
+            QPtrList<Cell> destList;
             for ( y = src.bottom() + 1; y <= dest.bottom(); y++ )
                 destList.append( nonDefaultCell( x, y ) );
-            QPtrList<KSpreadCell> srcList;
+            QPtrList<Cell> srcList;
             for ( y = src.top(); y <= src.bottom(); y++ )
             {
                 srcList.append( cellAt( x, y ) );
             }
             QPtrList<AutoFillSequence> seqList;
-            seqList.setAutoDelete( TRUE );
+            seqList.setAutoDelete( true );
             for ( y = src.top(); y <= src.bottom(); y++ )
                 seqList.append( new AutoFillSequence( cellAt( x, y ) ) );
             fillSequence( srcList, destList, seqList );
@@ -624,19 +643,19 @@ void KSpreadSheet::autofill( QRect &src, QRect &dest )
         for ( int y = dest.top(); y <= dest.bottom(); y++ )
         {
             int x;
-            QPtrList<KSpreadCell> destList;
+            QPtrList<Cell> destList;
 
             for ( x = dest.left(); x < src.left(); x++ )
             {
                 destList.append( nonDefaultCell( x, y ) );
             }
-            QPtrList<KSpreadCell> srcList;
+            QPtrList<Cell> srcList;
             for ( x = src.left(); x <= src.right(); x++ )
             {
                 srcList.append( cellAt( x, y ) );
             }
             QPtrList<AutoFillSequence> seqList;
-            seqList.setAutoDelete( TRUE );
+            seqList.setAutoDelete( true );
             for ( x = src.left(); x <= src.right(); x++ )
                 seqList.append( new AutoFillSequence( cellAt( x, y ) ) );
             fillSequence( srcList, destList, seqList, false );
@@ -653,16 +672,16 @@ void KSpreadSheet::autofill( QRect &src, QRect &dest )
         for ( int x = startVal; x <= endVal; x++ )
         {
             int y;
-            QPtrList<KSpreadCell> destList;
+            QPtrList<Cell> destList;
             for ( y = dest.top(); y < src.top(); y++ )
                 destList.append( nonDefaultCell( x, y ) );
-            QPtrList<KSpreadCell> srcList;
-            for ( y = dest.top(); y <= dest.bottom(); ++y )
+            QPtrList<Cell> srcList;
+            for ( y = src.top(); y <= src.bottom(); ++y )
             {
                 srcList.append( cellAt( x, y ) );
             }
             QPtrList<AutoFillSequence> seqList;
-            seqList.setAutoDelete( TRUE );
+            seqList.setAutoDelete( true );
             for ( y = src.top(); y <= src.bottom(); y++ )
                 seqList.append( new AutoFillSequence( cellAt( x, y ) ) );
             fillSequence( srcList, destList, seqList, false );
@@ -674,12 +693,13 @@ void KSpreadSheet::autofill( QRect &src, QRect &dest )
 }
 
 
-void KSpreadSheet::fillSequence( QPtrList<KSpreadCell>& _srcList,
-				 QPtrList<KSpreadCell>& _destList,
+void Sheet::fillSequence( QPtrList<Cell>& _srcList,
+				 QPtrList<Cell>& _destList,
                                  QPtrList<AutoFillSequence>& _seqList,
                                  bool down)
 {
-
+    doc()->emitBeginOperation(true);
+	
     /* try finding an interval to use to fill the sequence */
     if (!FillSequenceWithInterval(_srcList, _destList, _seqList, down))
     {
@@ -687,22 +707,30 @@ void KSpreadSheet::fillSequence( QPtrList<KSpreadCell>& _srcList,
       FillSequenceWithCopy(_srcList, _destList, down);
     }
 
+    doc()->emitEndOperation();
+
 }
 
-double getDiff(KSpreadCell * cell1, KSpreadCell * cell2, AutoFillSequenceItem::Type type)
+QVariant getDiff( const Value& value1, const Value& value2  , AutoFillSequenceItem::Type type  )
 {
+  if ( type == AutoFillSequenceItem::FLOAT )
+	  return QVariant( value2.asFloat() - value1.asFloat() );
+  if ( type == AutoFillSequenceItem::TIME || type == AutoFillSequenceItem::DATE )
+	  return QVariant( (int)( value2.asInteger() - value1.asInteger() ) );
+
+  return QVariant( (int)0 );
   // note: date and time difference can be calculated as
   // the difference of the serial number
-  if( (type == AutoFillSequenceItem::FLOAT) ||
+ /* if( (type == AutoFillSequenceItem::FLOAT) ||
       (type == AutoFillSequenceItem::DATE) ||
       (type == AutoFillSequenceItem::TIME) )
-    return ( cell2->value().asFloat() - cell1->value().asFloat() );
+    return ( value2.asFloat() - value1.asFloat() );
   else
-    return 0.0;
+    return 0.0;*/
 }
 
-bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
-                                            QPtrList<KSpreadCell>& _destList,
+bool Sheet::FillSequenceWithInterval(QPtrList<Cell>& _srcList,
+                                            QPtrList<Cell>& _destList,
                                             QPtrList<AutoFillSequence>& _seqList,
                                             bool down)
 {
@@ -710,22 +738,24 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
     return false;
 
   QPtrList<AutoFillDeltaSequence> deltaList;
-  deltaList.setAutoDelete( TRUE );
+  deltaList.setAutoDelete( true );
   bool ok = false;
 
   if ( _srcList.first()->value().isNumber() || _srcList.first()->isDate() || _srcList.first()->isTime() )
   {
     AutoFillSequenceItem::Type type;
 
-    QMemArray<double> * tmp  = new QMemArray<double> ( _seqList.count() );
-    QMemArray<double> * diff = new QMemArray<double> ( _seqList.count() );
+    QValueVector< QVariant > tmp( _seqList.count() );  /*= new QValueList< QVariant > ( _seqList.count() )*/;
+    QValueVector< QVariant > diff( _seqList.count() ); /*= new QValueList< QVariant > ( _seqList.count() )*/;
     int p = -1;
     int count = 0;
     int tmpcount = 0;
 
-    KSpreadCell * cell = _srcList.first();
-    KSpreadCell * cell2 = _srcList.next();
+    Cell * cell = _srcList.first();
+    Cell * cell2 = _srcList.next();
 
+    bool singleCellOnly = (cell2 == 0);
+    
     if ( cell->isDate() )
       type = AutoFillSequenceItem::DATE;
     else if ( cell->isTime() )
@@ -733,36 +763,61 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
     else if ( cell->value().isNumber() )
       type = AutoFillSequenceItem::FLOAT;
     else
-      return false; // Cannot happen du to if condition
+      return false; // Cannot happen due to if condition
 
-    while ( cell && cell2 )
+    while ( cell && (cell2 || singleCellOnly) )
     {
+      
+      Value cellValue = cell->value();
+      Value cell2Value;
 
-      // check if both cells contain the same type
-      if ( ( !cell2->value().isNumber() )
+      //If we only have a single cell, the interval will depend upon the data type.
+      //- For numeric values, set the interval to 0 as we don't know what might be useful as a sequence
+      //- For time values, set the interval to one hour, as this will probably be the most useful setting
+      //- For date values, set the interval to one day, as this will probably be the most useful setting
+      //
+      //Note that the above options were chosen for consistency with Excel.  Gnumeric (1.59) sets
+      //the interval to 0 for all types, OpenOffice.org (2.00) uses increments of 1.00, 1 hour and 1 day
+      //respectively
+      if (singleCellOnly)
+      {
+      	if (type == AutoFillSequenceItem::FLOAT)
+		cell2Value = cellValue;
+	else if ( type == AutoFillSequenceItem::TIME)
+		cell2Value = Value( cellValue.asTime().addSecs( 60*60 ) );
+	else if ( type == AutoFillSequenceItem::DATE)
+		cell2Value = Value ( cellValue.asDate().addDays( 1 ) );
+      }
+      else
+      {
+	cell2Value = cell2->value();      
+	      
+      	// check if both cells contain the same type
+      	if ( ( !cellValue.isNumber() )
            || ( cell2->isDate() && type != AutoFillSequenceItem::DATE )
            || ( cell2->isTime() && type != AutoFillSequenceItem::TIME ) )
-      {
-        count = 0;
-        ok = false;
-        break;
+      	{
+        	count = 0;
+        	ok = false;
+       		break;
+      	}
       }
 
-      double delta = getDiff(cell, cell2, type);
-
+      QVariant delta = getDiff(cellValue , cell2Value , type ); 
+      
       if (count < 1)
       {
         p = count;
-        diff->at( count++ ) = delta;
+        diff[ count++ ] = delta;
       }
       else
       {
         // same value again?
-        if (diff->at( p ) == delta)
+        if (diff[ p ] == delta)
         {
           // store it somewhere else for the case we need it later
           ++p;
-          tmp->at( tmpcount++ ) = delta;
+          tmp[ tmpcount++ ] = delta;
         }
         else
         {
@@ -771,7 +826,7 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
           {
             for ( int i = 0; i < tmpcount; ++i )
             {
-              diff->at( count++ ) = tmp->at( i );
+              diff[ count++ ] = tmp.at( i );
             }
 
             tmpcount = 0;
@@ -779,7 +834,7 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
 
           // insert the value
           p = 0;
-          diff->at( count++ ) = delta;
+          diff[ count++ ] = delta;
         }
       }
 
@@ -791,35 +846,30 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
     // we have found something:
     if (count > 0 && (tmpcount > 0 || count == 1))
     {
-      double initDouble=0.0;
+      QVariant cellValue( (int) 0 );
 
-      KSpreadCell * dest;
-      KSpreadCell * src;
+      Cell * dest;
+      Cell * src;
 
       int i = tmpcount;
       if (down)
       {
         dest = _destList.first();
         src  = _srcList.last();
-
-        if( (type == AutoFillSequenceItem::FLOAT) ||
-            (type == AutoFillSequenceItem::DATE) ||
-            (type == AutoFillSequenceItem::TIME) )
-          initDouble = src->value().asFloat();
       }
       else
       {
         dest = _destList.last();
         src  = _srcList.first();
 
-        if( (type == AutoFillSequenceItem::FLOAT) ||
-            (type == AutoFillSequenceItem::DATE) ||
-            (type == AutoFillSequenceItem::TIME) )
-          initDouble = src->value().asFloat();
-
         i   *= -1;
       }
 
+      if ( type == AutoFillSequenceItem::FLOAT )
+	      cellValue = src->value().asFloat();
+      else
+	      cellValue = (int)src->value().asInteger();
+
       QString res;
       // copy all the data
       while (dest)
@@ -834,22 +884,36 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
           while ( i < 0)
             i += count;
         }
+	
+	QVariant currentDiff = diff.at( i );
+	
+	if (cellValue.type() == QVariant::Double)
+        	if (down)
+            		cellValue = cellValue.asDouble() + currentDiff.asDouble();  	
+          	else
+            		cellValue = cellValue.asDouble() -  currentDiff.asDouble();
+	else
+		if (down)
+			cellValue = cellValue.asInt() + currentDiff.asInt();
+		else
+			cellValue = cellValue.asInt() - currentDiff.asInt();
 
-        if( (type == AutoFillSequenceItem::FLOAT) ||
-            (type == AutoFillSequenceItem::DATE) ||
-            (type == AutoFillSequenceItem::TIME) )
-        {
-          if (down)
-            initDouble += diff->at( i );
-          else
-            initDouble -= diff->at( i );
-
-          res.sprintf("%f", initDouble );
-        }
-
-        dest->setCellText( res, true );
-        dest->copyFormat( src );
-        dest->setFormatType( src->formatType() );
+	if ( type == AutoFillSequenceItem::TIME)
+	{
+		Value timeValue = doc()->converter()->asTime( Value(cellValue.asInt()) );
+		Value stringValue = doc()->converter()->asString( timeValue );
+		dest->setCellText( stringValue.asString() );
+	}
+	else if ( type == AutoFillSequenceItem::DATE)
+	{
+		Value dateValue = doc()->converter()->asDate( Value(cellValue.asInt()) );
+		Value stringValue = doc()->converter()->asString( dateValue );
+		dest->setCellText( stringValue.asString() );
+	}
+	else
+        	dest->setCellText( cellValue.asString() );
+        
+	dest->copyFormat( src );
 
         if (down)
         {
@@ -875,13 +939,13 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
       ok = false;
     }
 
-    delete tmp;
-    delete diff;
+   // delete tmp;
+   // delete diff;
 
     return ok;
   }
 
-  // What is the interval (block)? If your table looks like this:
+  // What is the interval (block)? If your sheet looks like this:
   // 1 3 5 7 9
   // then the interval has the length 1 and the delta list is [2].
   // 2 200 3 300 4 400
@@ -928,7 +992,7 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
 	{
 	  if ( !_seqList.at( (tst-1) * step + s )->
 	       matches( _seqList.at( tst * step + s ), deltaList.at( s ) ) )
-	    ok = FALSE;
+	    ok = false;
 	}
       }
       // Did we find a valid interval ?
@@ -939,7 +1003,7 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
 	int block = _seqList.count() / step;
 
 	// Start iterating with the first cell
-	KSpreadCell * cell;
+	Cell * cell;
         if (down)
           cell = _destList.first();
         else
@@ -1001,13 +1065,13 @@ bool KSpreadSheet::FillSequenceWithInterval(QPtrList<KSpreadCell>& _srcList,
   return ok;
 }
 
-void KSpreadSheet::FillSequenceWithCopy(QPtrList<KSpreadCell>& _srcList,
-                                        QPtrList<KSpreadCell>& _destList,
+void Sheet::FillSequenceWithCopy(QPtrList<Cell>& _srcList,
+                                        QPtrList<Cell>& _destList,
                                         bool down)
 {
   // We did not find any valid interval. So just copy over the marked
   // area.
-  KSpreadCell * cell;
+  Cell * cell;
 
   if (down)
     cell = _destList.first();
@@ -1041,21 +1105,41 @@ void KSpreadSheet::FillSequenceWithCopy(QPtrList<KSpreadCell>& _srcList,
     {
       if ( _srcList.at( s )->isFormula() )
       {
- 	QString d = _srcList.at( s )->encodeFormula();
-	cell->setCellText( cell->decodeFormula( d ), true );
+        QString d = _srcList.at( s )->encodeFormula();
+        cell->setCellText( cell->decodeFormula( d ) );
       }
       else if(_srcList.at( s )->value().isNumber() && _srcList.count()==1)
       {
-	double val;
-        if ( _srcList.at( s )->formatType() == KSpreadFormat::Percentage )
-            factor = 0.01;
+        double val;
+        int format_type = _srcList.at( s )->formatType();
+        if ( format_type == Percentage_format )
+        {
+          factor = 0.01;  // one percent
+        }
+        else if ( _srcList.at( s )->isTime() )
+        {
+          // FIXME this is a workaround to avoid those nasty one minute off
+          //       "dragging down" time is inaccurate overa large lists!
+          //       This is the best approximation I could find (raphael)
+          if (down)
+          {
+//             factor = 1.000002/24.  + 0.000000001;
+            factor = 0.041666751;
+          }
+          else
+          {  //when dragging "up" the error must of course be the other way round
+            factor = 0.0416665;
+          }
+        }
+
         if (!down)
           val = (_srcList.at( s )->value().asFloat() - (incr * factor));
         else
           val = (_srcList.at( s )->value().asFloat() + (incr * factor));
-	QString tmp;
-	tmp = tmp.setNum(val);
-	cell->setCellText( tmp, true );
+        
+        QString tmp;
+        tmp = tmp.setNum(val);
+        cell->setCellText( tmp );
         ++incr;
       }
       else if((AutoFillSequenceItem::month != 0L)
@@ -1085,18 +1169,26 @@ void KSpreadSheet::FillSequenceWithCopy(QPtrList<KSpreadCell>& _srcList,
       {
 	QRegExp number("(\\d+)");
 	int pos =number.search(_srcList.at( s )->text());
-	if( pos )
+	if( pos!=-1 )
 	{
 	  QString tmp=number.cap(1);
-	  int num=tmp.toInt();
+	  int num=tmp.toInt()+incr;
 	  cell->setCellText(_srcList.at( s )->text().replace(number,QString::number(num)));
+          ++incr;
 	}
-	else
-	  cell->setCellText( _srcList.at( s )->text(), true );
+	else if ( !_srcList.at( s )->link().isEmpty() )
+        {
+	  cell->setCellText( _srcList.at( s )->text() );
+	  cell->setLink( _srcList.at( s )->link() );
+        }
+        else
+        {
+	  cell->setCellText( _srcList.at( s )->text() );
+        }
       }
     }
     else
-      cell->setCellText( "", true );
+      cell->setCellText( "" );
 
     cell->copyFormat( _srcList.at( s ) );
 
