@@ -27,17 +27,18 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef KSPREAD_CELL
 #define KSPREAD_CELL
 
-class KSpreadCell;
-class KSpreadSheet;
-class KSpreadCanvas;
-class KSpreadView;
+#include <qpainter.h>
+#include <qptrlist.h>
+#include <qdatetime.h>
+
+#include "kspread_condition.h"
 
 class KLocale;
 class QDomElement;
@@ -46,42 +47,44 @@ class KoXmlWriter;
 class KoGenStyles;
 class KoGenStyle;
 class KSParseNode;
-class KSpreadGenValidationStyles;
+class KoRect;
+class KoPoint;
+class KoOasisStyles;
+class KoOasisLoadingContext;
 
-
-#include <qpainter.h>
-#include <qptrlist.h>
-#include <qdatetime.h>
-
-#include "kspread_format.h"
-#include "kspread_global.h"
-#include "kspread_condition.h"
-#include "kspread_value.h"
-
-#include <koOasisStyles.h>
-#include <koRect.h>
-struct KSpreadValidity
+namespace KSpread
 {
-    KSpreadValidity()
-	{
-	    valMin = 0.0;
-	    valMax = 0.0;
-	    m_cond = None;
-	    m_action = Stop;
-	    m_allow = Allow_All;
-	    displayMessage = true;
-	    allowEmptyCell = false;
-            displayValidationInformation = false;
-	};
+class Canvas;
+class Format;
+class GenValidationStyles;
+class Sheet;
+class Value;
+class View;
+class ConditionalDialog;
+
+struct Validity
+{
+  Validity()
+  {
+      valMin = 0.0;
+      valMax = 0.0;
+      m_cond = Conditional::None;
+      m_action = Action::Stop;
+      m_restriction = Restriction::None;
+      displayMessage = true;
+      allowEmptyCell = false;
+      displayValidationInformation = false;
+  };
+
     QString message;
     QString title;
     QString titleInfo;
     QString messageInfo;
     double valMin;
     double valMax;
-    Conditional m_cond;
-    Action m_action;
-    Allow m_allow;
+    Conditional::Type m_cond;
+    Action::Type m_action;
+    Restriction::Type m_restriction;
     QTime  timeMin;
     QTime  timeMax;
     QDate  dateMin;
@@ -92,37 +95,35 @@ struct KSpreadValidity
     QStringList listValidity;
 };
 
-class CellPrivate;
-
+class Formula;
 
 /**
- * For every cell in the spread sheet there is a KSpreadCell object.
+ * For every cell in the spread sheet there is a Cell object.
  *
- * KSpreadCell contains format information and algorithm and it
+ * Cell contains format information and algorithm and it
  * contains the calculation algorithm.
  *
  * However, all empty cells are represented by one instace, called the
- * default cell. @ref #isDefault tells wether a cell is the default one
- * or not.
+ * default cell. @ref #isDefault can be used to determine whether or not a Cell object represents
+ * the default one.
  */
-class KSPREAD_EXPORT KSpreadCell : public KSpreadFormat
+class KSPREAD_EXPORT Cell
 {
-  friend class SelectPrivate;
-  friend class KSpreadConditions;
+  friend class Conditions;
 public:
 
-  KSpreadCell (KSpreadSheet *_sheet, int _column, int _row);
-  KSpreadCell (KSpreadSheet *_sheet, KSpreadStyle * _style, int _column, int _row);
+  Cell (Sheet *_sheet, int _column, int _row);
+  Cell (Sheet *_sheet, Style * _style, int _column, int _row);
 
     /**
      * @see #sheetDies
      */
-    ~KSpreadCell();
+    ~Cell();
 
     /**
      * Returns the worksheet which owns this cell.
      */
-    KSpreadSheet* sheet() const;
+    Sheet* sheet() const;
 
     /**
      * Returns true if this is a default cell (with row and column equal to zero).
@@ -172,7 +173,7 @@ public:
      * Given the sheet and cell position, this static function returns the full name
      * of the cell, i.e. with the name of the sheet.
      */
-    static QString fullName( const KSpreadSheet *s, int col, int row );
+    static QString fullName( const Sheet *s, int col, int row );
 
     /**
      * Given the column number, this static function returns the corresponding
@@ -183,7 +184,7 @@ public:
     /**
      * Returns the locale setting of this cell.
      */
-    KLocale* locale();
+    KLocale* locale() const;
     /**
      * Returns true if this cell holds a formula.
      */
@@ -197,21 +198,30 @@ public:
 
     QString strOutText() const;
 
+    Formula *formula () const;
+
+    /** Returns the format object of this cell */
+    Format* format() const;
+
     /**
      * Returns the value that this cell holds. It could be from the user
      * (i.e. when s/he enters a value) or a result of formula.
      */
-    const KSpreadValue value() const;
+    const Value value() const;
 
     /**
      * Sets the value for this cell.
      */
-    void setValue( const KSpreadValue& value );
+    void setValue( const Value& value );
+    /** Like setValue, but also sets formatting and input text. Can therefore
+    be used as a replacement for setCellText, if we don't need to parse. */
+    void setCellValue (const Value &v, FormatType fmtType = No_format,
+  const QString &txt = QString::null);
 
-    KSpreadCell* previousCell() const;
-    KSpreadCell* nextCell() const;
-    void setPreviousCell( KSpreadCell* c );
-    void setNextCell( KSpreadCell* c );
+    Cell* previousCell() const;
+    Cell* nextCell() const;
+    void setPreviousCell( Cell* c );
+    void setNextCell( Cell* c );
 
     /**
      * Moves around the cell. It cares about obscured and obscuring cells and
@@ -220,41 +230,52 @@ public:
     void move( int column, int row );
 
     /**
-     * The @ref KSpreadSheet calls this method if the sheet becomes deleted.
-     * At the time this method is called other cells may already be deleted or
-     * in some inconsistent state.
-     *
-     * The purpose of this method is to clear up some variables so that the destructor
-     * runs without any crash. Especially all variables referencing other cells
-     * must be voided. This method may not call any other method since the whole sheet
-     * and all remaining cells are in an inconsistent state now.
+     * This method notifies the cell that the parent sheet is being deleted.
      */
+    // Note:  This used to remove any links from this cell to other cells.  However, this caused a problem
+    // in other parts of the code which relied upon walking from one cell to the next using
+    // nextCell().
     void sheetDies();
 
     /**
      * Save this cell.
-     * @param _x_offset ...
-     * @param _y_offset ...
-     * @param force if set to true, all the properties of the format are stored (used for "Copy")
+     * @param doc document to save cell in
+     * @param _x_offset x offset
+     * @param _y_offset y offset
+     * @param force if set to true, all the properties of the format are stored (used for "Copy"),
+     *              otherwise only the non-default properties will be stored.
+     *              Set this to false if you want smaller files.
+     * @param copy if set to true, all cell formats will be copied instead of referencing the format (style name),
+     *             thus resulting in larger output (files).
+     *             Set this to false if you want smaller files.
      * @param era set this to true if you want to encode relative references as absolutely (they will be switched
      *            back to relative references during decoding) - is used for cutting to clipboard
-     * Usually this is false, to only store the properties explicitly set.
+     *            Usually this is false, to only store the properties explicitely set.
      */
     QDomElement save( QDomDocument& doc, int _x_offset = 0, int _y_offset = 0, bool force = false, bool copy = false, bool era = false );
 
-    virtual bool saveOasis( KoXmlWriter& xmlwriter , KoGenStyles &mainStyles, int row, int column, int maxCol, int &repeated, KSpreadGenValidationStyles &valStyle );
+    bool saveOasis( KoXmlWriter& xmlwriter, KoGenStyles& mainStyles,
+                    int row, int column, int &repeated,
+                    GenValidationStyles &valStyle );
 
     void saveOasisValue (KoXmlWriter &xmlWriter);
 
-    QString saveOasisCellStyle( KoGenStyle &currentCellStyle,KoGenStyles &mainStyles, bool force = false, bool copy = false );
+    /**
+     * @return the OASIS style's name
+     */
+    QString saveOasisCellStyle( KoGenStyle &currentCellStyle,KoGenStyles &mainStyles );
 
-    bool load( const QDomElement& cell, int _xshift, int _yshift, PasteMode pm = Normal,
-	       Operation op = OverWrite, bool paste = false );
+    bool load( const QDomElement& cell, int _xshift, int _yshift, Paste::Mode pm = Paste::Normal,
+               Paste::Operation op = Paste::OverWrite, bool paste = false );
 
-    bool loadOasis( const QDomElement & element, const KoOasisStyles &oasisStyles );
-    void loadOasisValidation( const QString& validationName );
+    /**
+     * Loads a cell from an OASIS XML element.
+     * @param element An OASIS XML element
+     * @param oasisContext The loading context assoiated with the XML element
+     */
+    bool loadOasis( const QDomElement & element, KoOasisLoadingContext &oasisContext , Style* style);
 
-    QTime toTime(const QDomElement &element) ;
+    QTime toTime(const QDomElement &element);
     QDate toDate(const QDomElement &element);
 
     /**
@@ -262,91 +283,110 @@ public:
      *
      * @see #copyAll
      */
-    void copyFormat( int _column, int _row );
+    void copyFormat( const int column, const int row );
     /**
      * A convenience function.
      *
      * @see #copyAll
      */
-    void copyFormat( KSpreadCell *_cell );
-    void copyContent( KSpreadCell *_cell );
+    void copyFormat( const Cell* cell );
+    void copyContent( const Cell* cell );
     /**
-     * Copies the format and the content. It does not copy the @ref #m_row and @ref #m_column attributes.
+     * Copies the format and the content. It does not copy the #m_row and #m_column attributes.
      * Besides that all persistent attributes are copied. @ref #setCellText is called to set the real
      * content.
      *
      * @see #copyFormat
      */
-    void copyAll( KSpreadCell *cell);
+    void copyAll( Cell *cell);
+
+    enum BorderSides
+    {
+      Border_None   =0x00,
+      Border_Left   =0x01,
+      Border_Right  =0x02,
+      Border_Top    =0x04,
+      Border_Bottom =0x08,
+      Border_SizeGrip   =0x10   //the size grip is the little square on the bottom right-hand corner of a highlighted range of cells
+              //which the user can click and drag to resize the range and change which cells are included.
+              //this is not used with normal borders
+    };
 
     /**
      * Paints the cell.
      *
      * @param rect the portion of the canvas that is actually in view
      * @param painter the painter object to paint on
-     * @param the view of this data.  This may be NULL, but no selection
+     * @param view the view of this data.  This may be NULL, but no selection
      *        will be included with the painting.
-     * @param coordinates coordinates on the painter where the top left corner
+     * @param coordinate coordinates on the painter where the top left corner
      *                    of the cell should be painted plus width and height
      * @param cellRef the column/row coordinates of the cell.
-     * @param paintBorderRight whether to draw the right border too.
-     * @param paintBorderBottom whether to draw the bottom border too.
+     * @param paintBorder a combination of flags from the Cell::BorderSides enum which specifies which cell borders to paint
+     * @param rightPen pen to use to draw the right border if @p paintBorder includes the Border_Right flag
+     * @param bottomPen pen to use to draw the bottom border if @p paintBorderBottom includes the Border_Bottom flag
+     * @param leftPen pen to use to draw the left border if @p paintBorderLeft includes the Border_Left flag
+     * @param topPen pen to use to draw the top border if @p paintBorderTop includes the Border_Top flag
+     * @param mergedCellsPainted list of merged cells being painted
      * @param drawCursor whether to draw the cursor and selection or not
      */
     void paintCell( const KoRect & rect, QPainter & painter,
-                    KSpreadView * view, const KoPoint & coordinate,
+                    View * view, const KoPoint & coordinate,
                     const QPoint & cellRef,
-                    bool paintBorderRight,
-                    bool paintBorderBottom,
-                    bool paintBorderLeft,
-                    bool paintBorderTop,
-                    QPen & rightPen,
+		    int paintBorder,
+		    QPen & rightPen,
                     QPen & bottomPen,
                     QPen & leftPen,
                     QPen & topPen,
+		    QValueList<QPoint> &mergedCellsPainted,
                     bool drawCursor = true );
 
+
     /**
      * @param _col the column this cell is assumed to be in.
      *             This parameter defaults to the return value of @ref #column.
+     * @param _canvas the canvas this cell is assumed to be in.
      *
      * @return the width of this cell as int
      */
-    int width( int _col = -1, const KSpreadCanvas *_canvas = 0L ) const;
+    int width( int _col = -1, const Canvas *_canvas = 0L ) const;
 
     /**
      * @param _row the row this cell is assumed to be in.
+     * @param _canvas the canvas this cell is assumed to be in.
      *
      * @return the height of this cell as int
      */
-    int height( int _row = -1, const KSpreadCanvas *_canvas = 0L ) const;
+    int height( int _row = -1, const Canvas *_canvas = 0L ) const;
 
     /**
+     * @param _canvas the canvas this cell is assumed to be in.
      * @param _col the column this cell is assumed to be in.
      *             This parameter defaults to the return value of @ref #column.
      *
      * @return the width of this cell as double
      */
-    double dblWidth( int _col = -1, const KSpreadCanvas *_canvas = 0L ) const;
+    double dblWidth( int _col = -1, const Canvas *_canvas = 0L ) const;
 
     /**
      * @param _row the row this cell is assumed to be in.
+     * @param _canvas the canvas this cell is assumed to be in.
      *
      * @return the height of this cell as double
      */
-    double dblHeight( int _row = -1, const KSpreadCanvas *_canvas = 0L ) const;
+    double dblHeight( int _row = -1, const Canvas *_canvas = 0L ) const;
 
     /**
-     * @return a QRect for this cell (i.e., a 1x1 rect)
+     * @return a QRect for this cell (i.e., a 1x1 rect).  @see zoomedCellRect
      */
     QRect cellRect();
 
     /**
-     * @return TRUE if the cell should be printed in a print out.
+     * @return true if the cell should be printed in a print out.
      *         That si the case if it has any content, border, backgroundcolor,
      *         or background brush.
      *
-     * @see KSpreadSheet::print
+     * @see Sheet::print
      */
     bool needsPrinting() const;
 
@@ -400,46 +440,95 @@ public:
     //
     ////////////////////////////////
 
+    /**
+     * @return effective pen for the left border
+     * If this cell is merged by another cell, the other cell's
+     * left border pen. If this cell's conditional formatting contains
+     * a left border pen and the condition is matched, the conditional
+     * formatting's pen. Otherwise, its own left border pen.
+     */
     const QPen & effLeftBorderPen( int col, int row ) const;
+    /**
+     * @return effective pen for the top border
+     * @see effLeftBorderPen
+     */
     const QPen & effTopBorderPen( int col, int row ) const;
+    /**
+     * @return effective pen for the right border
+     * @see effLeftBorderPen
+     */
     const QPen & effRightBorderPen( int col, int row ) const;
+    /**
+     * @return effective pen for the bottom border
+     * @see effLeftBorderPen
+     */
     const QPen & effBottomBorderPen( int col, int row ) const;
+    /**
+     * @return effective pen for the go up diagonal border
+     * If this cell's conditional formatting contains a go up diagonal pen and
+     * the condition is matched, the conditional formatting's pen. Otherwise,
+     * its own go up diagonal pen.
+     */
     const QPen & effGoUpDiagonalPen( int col, int row ) const;
+    /**
+     * @return effective pen for the go up diagonal border
+     * @see effGoUpDiagonalPen
+     */
     const QPen & effFallDiagonalPen( int col, int row ) const;
     const QColor & effTextColor( int col, int row ) const;
 
+    /**
+     * @return "worth" of the effective bottom border pen
+     * @see Style::calculateValue
+     * @see effLeftBorderPen
+     */
     uint effBottomBorderValue( int col, int row ) const;
+    /**
+     * @return "worth" of the effective right border pen
+     * @see Style::calculateValue
+     * @see effLeftBorderPen
+     */
     uint effRightBorderValue( int col, int row ) const;
+    /**
+     * @return "worth" of the effective left border pen
+     * @see Style::calculateValue
+     * @see effLeftBorderPen
+     */
     uint effLeftBorderValue( int col, int row ) const;
+    /**
+     * @return "worth" of the effective top border pen
+     * @see Style::calculateValue
+     * @see effLeftBorderPen
+     */
     uint effTopBorderValue( int col, int row ) const;
 
     /**
-     * @reimp
+     * @see Format::leftBorderPen
      */
     const QPen& leftBorderPen( int col, int row ) const;
 
     /**
-     * @reimp
+     * @see Format::topBorderPen
      */
     const QPen& topBorderPen( int col, int row ) const;
 
     /**
-     * @reimp
+     * @see Format::rightBorderPen
      */
     const QPen& rightBorderPen( int col, int row ) const;
 
     /**
-     * @reimp
+     * @see Format::bottomBorderPen
      */
     const QPen& bottomBorderPen( int col, int row ) const;
 
     /**
-     * @reimp
+     * @see Format::bgColor
      */
     const QColor& bgColor( int col, int row ) const;
 
     /**
-     * @reimp
+     * @see Format::backGroundBrush
      */
     const QBrush& backGroundBrush( int col, int row ) const;
 
@@ -450,30 +539,25 @@ public:
     ////////////////////////////////
 
     /**
-     * @reimp
+     * @see Format::setLeftBorderPen
      */
     void setLeftBorderPen( const QPen& p );
 
     /**
-     * @reimp
+     * @see Format::setTopBorderPen
      */
     void setTopBorderPen( const QPen& p );
 
     /**
-     * @reimp
+     * @see Format::setRightBorderPen
      */
     void setRightBorderPen( const QPen& p );
 
     /**
-     * @reimp
+     * @see Format::setBottomBorderPen
      */
     void setBottomBorderPen( const QPen& p );
 
-    /**
-     * @reimp
-     */
-    virtual void setCurrency( int type, QString const & symbol );
-
     //////////////////////
     //
     // Other stuff
@@ -482,13 +566,15 @@ public:
 
     /**
      * Return the format of this cell.
-     * Convenience method for KSpreadFormat::getFormatType
+     * Convenience method for Format::getFormatType
      * Note that this is "how the user would like the data to be displayed if possible".
      * If he selects a date format, and the cell contains a string, we won't apply that format.
      */
     FormatType formatType() const;
 
+    /** returns true, if cell format is of date type or content is a date */
     bool isDate() const;
+    /** returns true, if cell format is of time type or content is a time */
     bool isTime() const;
 
     void setNumber( double number );
@@ -496,22 +582,26 @@ public:
     /** return the cell's value as a double */
     double getDouble ();
 
+    /** converts content to double format */
     void convertToDouble ();
+    /** converts content to percentageformat */
     void convertToPercent ();
+    /** converts content to money format */
     void convertToMoney ();
+    /** converts content to time format */
     void convertToTime ();
+    /** converts content to date format */
     void convertToDate ();
 
-    /**
-     * return size of the text
-     */
+    /** return width of the text */
     double textWidth() const;
+    /** return height of the text */
     double textHeight() const;
 
 
     /**
     * Refreshing chart
-    * bool refresh is default true
+    * @param refresh is default true
     * when it's false it's just for test
     * it's used when you paste cell
     */
@@ -519,27 +609,19 @@ public:
 
     QString testAnchor( int _x, int _y ) const;
 
-    /**
-     * Called if the user clicks on a cell. If the cell is for example a button, then
-     * @ref #m_strAction is executed.
-     */
-    void clicked( KSpreadCanvas *_canvas );
-
     /**
      * Starts calculating.
      * @param delay true if you want to check for delay condition in doc()
      *         false if you really have to calculate the value right now
      *         e.g. if you sort with formula as key
      *
-     * @return TRUE on success and FALSE on error.
+     * @return true on success and false on error.
      */
     bool calc(bool delay = true);
 
-    /**
-     * Set the calcDirtyFlag
-     */
+    /** Set the calcDirtyFlag */
     void setCalcDirtyFlag();
-
+    /** Checks the calcDirtyFlag */
     bool calcDirtyFlag();
 
     /**
@@ -551,44 +633,45 @@ public:
      * @param isDepending true if the cell is now depending on this one, false if it is not any longer
      *                    depending on it.
      */
-    void NotifyDepending( int col, int row, KSpreadSheet* sheet, bool isDepending );
+    void NotifyDepending( int col, int row, Sheet* sheet, bool isDepending );
 
     /**
      * Causes the format to be recalculated when the cell is drawn next time.
      * This flag is for example set if the width of the column changes or if
      * some cell specific format value like font or text change.
      */
-    virtual void setLayoutDirtyFlag( bool format = false );
+    void setLayoutDirtyFlag( bool format = false );
     bool layoutDirtyFlag() const;
 
     void clearDisplayDirtyFlag();
     void setDisplayDirtyFlag();
 
     /**
-     * Tells this cell that the @ref KSpreadCell 'cell' obscures this one.
+     * Tells this cell that the @ref Cell 'cell' obscures this one.
      * If this cell has to be redrawn, then the obscuring cell is redrawn instead.
      *
      * @param cell the obscuring cell
      * @param isForcing whether this is a forced obscuring (merged cells) or
      *                  just a temporary obscure (text overlap).
      */
-    void obscure( KSpreadCell *cell, bool isForcing = false);
+    void obscure( Cell *cell, bool isForcing = false);
     /**
      * Tells this cell that it is no longer obscured.
      *
      * @param cell the cell that is no longer obscuring this one.
      */
-    void unobscure(KSpreadCell* cell);
+    void unobscure(Cell* cell);
     /**
-     * @return TRUE if this cell is obscured by another.
+     * @return true if this cell is obscured by another.
      */
     bool isObscured() const;
     /**
-     * If obscuring is forced then the marker may never reside on this cell.
+     * If this cell is part of a merged cell, then the marker may
+     * never reside on this cell.
      *
-     * @return TRUE if an obscuring cell is forced to obscure this one.
+     * @return true if another cell has this one merged into itself.
      */
-    bool isObscuringForced() const;
+    bool isPartOfMerged() const;
 
     /**
      * Return the cell that is obscuring this one (merged cells only).
@@ -596,19 +679,20 @@ public:
      *
      * @return the cell that decides the format for the cell in question.
      */
-    KSpreadCell *KSpreadCell::ultimateObscuringCell() const;
+    Cell *ultimateObscuringCell() const;
 
     /**
      * @return the obscuring cell list (might be empty)
      */
-    QValueList<KSpreadCell*> obscuringCells() const;
+    QValueList<Cell*> obscuringCells() const;
 
     void clearObscuringCells();
 
 
     /**
-     * Force the cell to occupy other cells space.
-     * If '_x' and '_y' are 0 then the forcing is disabled.
+     * Merge a number of cells, i.e. force the cell to occupy other
+     * cells space.  If '_x' and '_y' are 0 then the merging is
+     * disabled.
      *
      * @param _col is the column this cell is assumed to be in.
      * @param _row is the row this cell is assumed to be in.
@@ -616,12 +700,12 @@ public:
      * @param _y tells to occupy _y additional cells in the vertical
      *
      */
-    void forceExtraCells( int _col, int _row, int _x, int _y );
+    void mergeCells( int _col, int _row, int _x, int _y );
 
     /**
-     * @return TRUE if the cell is forced to obscure other cells.
+     * @return true if the cell is forced to obscure other cells.
      */
-    bool isForceExtraCells() const;
+    bool doesMergeCells() const;
 
     /**
      * @return the number of obscured cells in the horizontal direction as a
@@ -639,6 +723,7 @@ public:
      * @return the amount of obscured cells in the horizontal direction
      */
     int extraXCells() const;
+
     /**
      * @return the amount of obscured cells in the vertical direction
      */
@@ -653,9 +738,11 @@ public:
      * @param _era encode relative references absolutely (this is used for copying
      *             a cell to make the paste operation create a formula that points
      *             to the original cells, not the cells at the same relative position)
+     * @param _col row the formula is in
+     * @param _row column the formula is in
      */
-    QString encodeFormula( bool _era = false, int _col = -1, int _row = -1 );
-    QString decodeFormula( const QString &_text, int _col = -1, int _row = -1 );
+    QString encodeFormula( bool _era = false, int _col = -1, int _row = -1 ) const;
+    QString decodeFormula( const QString &_text, int _col = -1, int _row = -1 ) const;
 
     /**
      * Merges the @p new_text with @p old_text during a paste operation.
@@ -666,10 +753,10 @@ public:
      *
      * @return the merged text.
      */
-    QString pasteOperation( const QString &new_text, const QString &old_text, Operation op );
+    QString pasteOperation( const QString &new_text, const QString &old_text, Paste::Operation op );
 
     /**
-     * @return TRUE if the cell contains a formula that could not
+     * @return true if the cell contains a formula that could not
      *         be evaluated. These cells usually appear with "####" on the screen.
      */
     bool hasError() const;
@@ -687,8 +774,8 @@ public:
 
     /**
      * Parses the formula.
-     * Fills @ref #dependList and @ref #formula.
-     * @return FALSE on error.
+     * Fills #dependList and #formula.
+     * @return false on error.
      */
     bool makeFormula();
 
@@ -698,14 +785,14 @@ public:
     /**
      * Gets a copy of the list of current conditions
      */
-    QValueList<KSpreadConditional> conditionList() const;
+    QValueList<Conditional> conditionList() const;
 
     /**
      * Replace the old set of conditions with a new one
      */
-    void setConditionList(const QValueList<KSpreadConditional> &newList);
+    void setConditionList(const QValueList<Conditional> &newList);
 
-    KSpreadValidity * getValidity( int newStruct = -1 );
+    Validity * getValidity( int newStruct = -1 );
 
     void removeValidity();
 
@@ -717,7 +804,7 @@ public:
 
     /**
      * Calculates the text parameters stored in cell
-     * Applies font to use to @param painter
+     * Applies font to use to @p painter
      */
     void calculateTextParameters( QPainter &painter, int _col, int _row );
 
@@ -727,17 +814,19 @@ public:
     int defineAlignX();
 
 
+
+
     /**
      * Used for comparing cells (when sorting)
      */
-    bool operator > ( const KSpreadCell & ) const;
-    bool operator < ( const KSpreadCell & ) const;
+    bool operator > ( const Cell & ) const;
+    bool operator < ( const Cell & ) const;
 
     void freeAllObscuredCells();
 
     /* descriptions of the flags are just below */
     enum CellFlags{
-    /* this uses the same flags variable as KSpreadFormat.  The least significant
+    /* this uses the same flags variable as Format.  The least significant
        16 bits are reserved for the base class, and the most significant 16
        have been left for this subclass to use. */
       Flag_LayoutDirty           = 0x00010000,
@@ -745,7 +834,7 @@ public:
       Flag_Progress              = 0x00040000,
       Flag_UpdatingDeps          = 0x00080000,
       Flag_DisplayDirty          = 0x00100000,
-      Flag_ForceExtra            = 0x00200000,
+      Flag_Merged                = 0x00200000,
       Flag_CellTooShortX         = 0x00400000,
       Flag_CellTooShortY         = 0x00800000,
       Flag_ParseError            = 0x01000000,
@@ -753,6 +842,7 @@ public:
       Flag_DependancyError       = 0x04000000,
       Flag_PaintingCell          = 0x08000000, // On during painting
       Flag_TextFormatDirty       = 0x10000000
+     // Flag_Highlight     = 0x20000000
     };
 
     void clearFlag( CellFlags flag );
@@ -774,7 +864,7 @@ public:
    * CalcDirty
    * Shows whether recalculation is necessary.
    * If this cell must be recalculated for some reason, for example the user
-   * entered a new formula, then this flag is set. If @ref #bFormula is FALSE
+   * entered a new formula, then this flag is set. If @ref #bFormula is false
    * nothing will happen at all.
    *
    * Progress
@@ -791,17 +881,17 @@ public:
    * DisplayDirty - TODO - is this unused now??
    * If this flag is set, then it is known that this cell has to be updated
    * on the display. This means that somewhere in the calling stack there is a
-   * function which will call @ref KSpreadSheet::updateCell once it retains
+   * function which will call @ref Sheet::updateCell once it retains
    * the control. If a function changes the contents/layout of this cell and this
    * flag is not set, then the function must set it at once. After the changes
    * are done the function must call <tt>m_pSheet->updateCell(...).
-   * The flag is cleared by the function m_pSheet->updateCell.
+   * The flag is cleared by the function format()->sheet()->updateCell.
    *
-   * ForceExtra
-   * Tells whether the cell is forced to exceed its size.
-   * Cells may occupy other cells space on demand. But you may force
-   * a cell to do so by setting this flag. Forcing the cell to have
-   * no extra size will disable this flag!
+   * Merged
+   * Tells whether the cell is merged with other cells.  Cells may
+   * occupy other cells space on demand. You may force a cell to
+   * do so by setting this flag. Merging the cell with 0 in both
+   * directions, will disable this flag!
    *
    * CellTooShortX
    * When it's True displays ** and/or the red triangle and when the
@@ -816,25 +906,12 @@ public:
 
 protected:
     /**
-     * @reimp
-     */
-    void formatChanged();
-    /**
-     * @reimp
-     */
-    KSpreadFormat* fallbackFormat( int col, int row );
-    /**
-     * @reimp
-     */
-    const KSpreadFormat* fallbackFormat( int col, int row ) const;
-
-    /**
-     * Applies the font to use to @param painter
+     * Applies the font to use to @p painter
      */
     void applyZoomedFont( QPainter &painter, int _col, int _row );
 
     /**
-     * Called from @ref #makeFormat to determine the space
+     * Called from makeFormat() to determine the space
      * needed for the text.
      */
     void textSize( QPainter &_paint );
@@ -868,19 +945,42 @@ protected:
      */
     void checkNumberFormat();
 
+   /**
+     * Load the text paragraphs from an OASIS XML cell description.
+     * @param parent The DOM element representing the cell.
+     */
+    void loadOasisCellText( const QDomElement& parent );
+    void loadOasisObjects( const QDomElement& e, KoOasisLoadingContext& oasisContext );
+    void loadOasisValidation( const QString& validationName );
+
     void loadOasisValidationCondition( QString &valExpression );
     void saveOasisAnnotation( KoXmlWriter &xmlwriter );
     void loadOasisConditional( QDomElement * style );
 
+
+
 private:
 
-    CellPrivate* d;
+    class Extra;
+    class Private;
+    Private *d;
     // static const char* s_dataTypeToString[];
 
     /* helper functions to the paintCell(...) function */
+ /*   void paintCellHighlight(QPainter& painter,
+          const KoRect& cellRect,
+          const QPoint& cellRef,
+          const int highlightBorder,
+          const QPen& rightPen,
+          const QPen& bottomPen,
+          const QPen& leftPen,
+          const QPen& topPen
+         );*/
+
+
     void paintCellBorders( QPainter& painter, const KoRect &rect,
-                           const KoRect &cellRect, 
-			   const QPoint &cellRef,
+                           const KoRect &cellRect,
+         const QPoint &cellRef,
                            bool paintBorderRight, bool paintBorderBottom,
                            bool paintBorderLeft, bool paintBorderTop,
                            QPen & rightPen, QPen & bottomPen,
@@ -906,17 +1006,19 @@ private:
                           const QPoint &cellRef, bool selected,
                           QColor &backgroundColor );
     void paintObscuredCells( const KoRect& rect, QPainter& painter,
-                             KSpreadView* view, const KoRect &cellRect,
+                             View* view, const KoRect &cellRect,
                              const QPoint &cellRef,
                              bool paintBorderRight, bool paintBorderBottom,
                              bool paintBorderLeft, bool paintBorderTop,
                              QPen & rightPen, QPen & bottomPen,
-                             QPen & leftPen, QPen & topPen );
+                             QPen & leftPen, QPen & topPen,
+			     QValueList<QPoint> &mergedCellsPainted );
     void paintCellDiagonalLines( QPainter& painter, const KoRect &cellRect,
                                  const QPoint &cellRef );
 
 
 
+
   /** handle the fact that a cell has been updated - calls cellUpdated()
   in the parent Sheet object */
   void valueChanged ();
@@ -927,7 +1029,7 @@ private:
 
 
   /* helper functions to the load/save routines */
-  bool loadCellData(const QDomElement &text, Operation op);
+  bool loadCellData(const QDomElement &text, Paste::Operation op);
   bool saveCellResult( QDomDocument& doc, QDomElement& result,
                        QString str );
   void update();
@@ -941,10 +1043,18 @@ private:
   void offsetAlign( int _col, int _row );
 
     void checkForNamedAreas( QString & formula ) const;
-    void convertFormula( QString & text, const QString & f ) const;
+    /**
+     * replacements:
+     * 1. "==" -> "="
+     * 2. "Sheet1!A1"  -> "[Sheet1.A1]"
+     *    "A1" -> "[.A1]"
+     * 3. ","  -> "."
+     */
     QString convertFormulaToOasisFormat( const QString & formula ) const;
     void loadOasisValidationValue( const QStringList &listVal );
 
 };
 
+} // namespace KSpread
+
 #endif  // KSPREAD_CELL
