@@ -14,28 +14,32 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <stdlib.h>
 
 #include <kdebug.h>
 
-#include "kspread_cluster.h"
 #include "kspread_cell.h"
+#include "kspread_format.h"
+
+#include "kspread_cluster.h"
+
+using namespace KSpread;
 
 /****************************************************
  *
- * KSpreadCluster
+ * Cluster
  *
  ****************************************************/
 
 /* Generate a matrix LEVEL1 with the size LEVEL1*LEVEL1 */
-KSpreadCluster::KSpreadCluster()
-    : m_first( 0 ), m_autoDelete( FALSE ), m_biggestX(0), m_biggestY(0)
+Cluster::Cluster()
+    : m_first( 0 ), m_autoDelete( false ), m_biggestX(0), m_biggestY(0)
 {
-    m_cluster = (KSpreadCell***)malloc( KSPREAD_CLUSTER_LEVEL1 * KSPREAD_CLUSTER_LEVEL1 * sizeof( KSpreadCell** ) );
+    m_cluster = (Cell***)malloc( KSPREAD_CLUSTER_LEVEL1 * KSPREAD_CLUSTER_LEVEL1 * sizeof( Cell** ) );
 
     for( int x = 0; x < KSPREAD_CLUSTER_LEVEL1; ++x )
 	for( int y = 0; y < KSPREAD_CLUSTER_LEVEL1; ++y )
@@ -43,13 +47,13 @@ KSpreadCluster::KSpreadCluster()
 }
 
 /* Delete the matrix LEVEL1 and all existing LEVEL2 matrizes */
-KSpreadCluster::~KSpreadCluster()
+Cluster::~Cluster()
 {
 // Can't we use clear(), to remove double code - Philipp?
     for( int x = 0; x < KSPREAD_CLUSTER_LEVEL1; ++x )
 	for( int y = 0; y < KSPREAD_CLUSTER_LEVEL1; ++y )
         {
-	    KSpreadCell** cl = m_cluster[ y * KSPREAD_CLUSTER_LEVEL1 + x ];
+	    Cell** cl = m_cluster[ y * KSPREAD_CLUSTER_LEVEL1 + x ];
 	    if ( cl )
 	    {
 		free( cl );
@@ -59,10 +63,10 @@ KSpreadCluster::~KSpreadCluster()
 
     if ( m_autoDelete )
     {
-	KSpreadCell* cell = m_first;
+	Cell* cell = m_first;
 	while( cell )
         {
-	    KSpreadCell* n = cell->nextCell();
+	    Cell* n = cell->nextCell();
 	    delete cell;
 	    cell = n;
 	}
@@ -71,12 +75,12 @@ KSpreadCluster::~KSpreadCluster()
     free( m_cluster );
 }
 
-void KSpreadCluster::clear()
+void Cluster::clear()
 {
     for( int x = 0; x < KSPREAD_CLUSTER_LEVEL1; ++x )
 	for( int y = 0; y < KSPREAD_CLUSTER_LEVEL1; ++y )
         {
-	    KSpreadCell** cl = m_cluster[ y * KSPREAD_CLUSTER_LEVEL1 + x ];
+	    Cell** cl = m_cluster[ y * KSPREAD_CLUSTER_LEVEL1 + x ];
 	    if ( cl )
 	    {
 		free( cl );
@@ -86,10 +90,10 @@ void KSpreadCluster::clear()
 
     if ( m_autoDelete )
     {
-	KSpreadCell* cell = m_first;
+	Cell* cell = m_first;
 	while( cell )
         {
-	    KSpreadCell* n = cell->nextCell();
+	    Cell* n = cell->nextCell();
 	    delete cell;
 	    cell = n;
 	}
@@ -99,11 +103,11 @@ void KSpreadCluster::clear()
     m_biggestX = m_biggestY = 0;
 }
 
-KSpreadCell* KSpreadCluster::lookup( int x, int y ) const
+Cell* Cluster::lookup( int x, int y ) const
 {
     if ( x >= KSPREAD_CLUSTER_MAX || x < 0 || y >= KSPREAD_CLUSTER_MAX || y < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::lookup: invalid column or row value (col: "
+	kdDebug(36001) << "Cluster::lookup: invalid column or row value (col: "
 		       << x << "  | row: " << y << ")" << endl;
 	return 0;
     }
@@ -112,7 +116,7 @@ KSpreadCell* KSpreadCluster::lookup( int x, int y ) const
     int dx = x % KSPREAD_CLUSTER_LEVEL2;
     int dy = y % KSPREAD_CLUSTER_LEVEL2;
 
-    KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
+    Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
     if ( !cl )
 	return 0;
 
@@ -120,11 +124,11 @@ KSpreadCell* KSpreadCluster::lookup( int x, int y ) const
 }
 
 /* Paste a cell in LEVEL2 (it's more paste than insert) */
-void KSpreadCluster::insert( KSpreadCell* cell, int x, int y )
+void Cluster::insert( Cell* cell, int x, int y )
 {
     if ( x >= KSPREAD_CLUSTER_MAX || x < 0 || y >= KSPREAD_CLUSTER_MAX || y < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::insert: invalid column or row value (col: "
+	kdDebug(36001) << "Cluster::insert: invalid column or row value (col: "
 		       << x << "  | row: " << y << ")" << endl;
 	return;
     }
@@ -134,10 +138,10 @@ void KSpreadCluster::insert( KSpreadCell* cell, int x, int y )
     int dx = x % KSPREAD_CLUSTER_LEVEL2;
     int dy = y % KSPREAD_CLUSTER_LEVEL2;
 
-    KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
+    Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
     if ( !cl )
     {
-	cl = (KSpreadCell**)malloc(  KSPREAD_CLUSTER_LEVEL2 * KSPREAD_CLUSTER_LEVEL2 * sizeof( KSpreadCell*  ) );
+	cl = (Cell**)malloc(  KSPREAD_CLUSTER_LEVEL2 * KSPREAD_CLUSTER_LEVEL2 * sizeof( Cell*  ) );
 	m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ] = cl;
 
 	for( int a = 0; a < KSPREAD_CLUSTER_LEVEL2; ++a )
@@ -156,17 +160,17 @@ void KSpreadCluster::insert( KSpreadCell* cell, int x, int y )
 	m_first->setPreviousCell( cell );
     }
     m_first = cell;
-    
+
     if (x > m_biggestX) m_biggestX = x;
     if (y > m_biggestY) m_biggestY = y;
 }
 
 /* Removes the cell of a matrix, the matrix itself keeps unchanged */
-void KSpreadCluster::remove( int x, int y )
+void Cluster::remove( int x, int y )
 {
     if ( x >= KSPREAD_CLUSTER_MAX || x < 0 || y >= KSPREAD_CLUSTER_MAX || y < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::remove: invalid column or row value (col: "
+	kdDebug(36001) << "Cluster::remove: invalid column or row value (col: "
 		       << x << "  | row: " << y << ")" << endl;
 	return;
     }
@@ -176,11 +180,11 @@ void KSpreadCluster::remove( int x, int y )
     int dx = x % KSPREAD_CLUSTER_LEVEL2;
     int dy = y % KSPREAD_CLUSTER_LEVEL2;
 
-    KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
+    Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
     if ( !cl )
 	return;
 
-    KSpreadCell* c = cl[ dy * KSPREAD_CLUSTER_LEVEL2 + dx ];
+    Cell* c = cl[ dy * KSPREAD_CLUSTER_LEVEL2 + dx ];
     if ( !c )
 	return;
 
@@ -190,9 +194,9 @@ void KSpreadCluster::remove( int x, int y )
     {
 	if ( m_first == c )
 	    m_first = c->nextCell();
-        if(c->isForceExtraCells())
+        if(c->doesMergeCells())
         {
-	    c->forceExtraCells(c->column(),c->row(),0,0);
+	    c->mergeCells(c->column(),c->row(),0,0);
         }
 	delete c;
     }
@@ -209,55 +213,55 @@ void KSpreadCluster::remove( int x, int y )
     }
 }
 
-bool KSpreadCluster::shiftRow( const QPoint& marker )
+bool Cluster::shiftRow( const QPoint& marker )
 {
     bool dummy;
     return shiftRow( marker, dummy );
 }
 
-bool KSpreadCluster::shiftColumn( const QPoint& marker )
+bool Cluster::shiftColumn( const QPoint& marker )
 {
     bool dummy;
     return shiftColumn( marker, dummy );
 }
 
-void KSpreadCluster::unshiftColumn( const QPoint& marker )
+void Cluster::unshiftColumn( const QPoint& marker )
 {
     bool dummy;
     unshiftColumn( marker, dummy );
 }
 
-void KSpreadCluster::unshiftRow( const QPoint& marker )
+void Cluster::unshiftRow( const QPoint& marker )
 {
     bool dummy;
     unshiftRow( marker, dummy );
 }
 
-void KSpreadCluster::setAutoDelete( bool b )
+void Cluster::setAutoDelete( bool b )
 {
     m_autoDelete = b;
 }
 
-bool KSpreadCluster::autoDelete() const
+bool Cluster::autoDelete() const
 {
     return m_autoDelete;
 }
 
-KSpreadCell* KSpreadCluster::firstCell() const
+Cell* Cluster::firstCell() const
 {
     return m_first;
 }
 
-bool KSpreadCluster::shiftRow( const QPoint& marker, bool& work )
+bool Cluster::shiftRow( const QPoint& marker, bool& work )
 {
-    work = FALSE;
+    work = false;
 
     if ( marker.x() >= KSPREAD_CLUSTER_MAX || marker.x() < 0 ||
 	 marker.y() >= KSPREAD_CLUSTER_MAX || marker.y() < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::shiftRow: invalid column or row value (col: "
+	kdDebug(36001) << "Cluster::shiftRow: invalid column or row value (col: "
 		       << marker.x() << "  | row: " << marker.y() << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     int cx = marker.x() / KSPREAD_CLUSTER_LEVEL2;
@@ -267,20 +271,20 @@ bool KSpreadCluster::shiftRow( const QPoint& marker, bool& work )
 
     // Is there a cell at the bottom most position ?
     // In this case the shift is impossible.
-    KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + KSPREAD_CLUSTER_LEVEL1 - 1 ];
+    Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + KSPREAD_CLUSTER_LEVEL1 - 1 ];
     if ( cl && cl[ dy * KSPREAD_CLUSTER_LEVEL2 + KSPREAD_CLUSTER_LEVEL2 - 1 ] )
-	return FALSE;
+	return false;
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     // Move cells in this row one down.
     for( int i = KSPREAD_CLUSTER_LEVEL1 - 1; i >= cx ; --i )
     {
-	KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + i ];
+	Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + i ];
 	if ( cl )
         {
-	    work = TRUE;
+	    work = true;
 	    int left = 0;
 	    if ( i == cx )
 		left = dx;
@@ -289,7 +293,7 @@ bool KSpreadCluster::shiftRow( const QPoint& marker, bool& work )
 		right = KSPREAD_CLUSTER_LEVEL2 - 2;
 	    for( int k = right; k >= left; --k )
 	    {
-		KSpreadCell* c = cl[ dy * KSPREAD_CLUSTER_LEVEL2 + k ];
+		Cell* c = cl[ dy * KSPREAD_CLUSTER_LEVEL2 + k ];
 		if ( c )
 	        {
 		    remove( c->column(), c->row() );
@@ -302,19 +306,19 @@ bool KSpreadCluster::shiftRow( const QPoint& marker, bool& work )
 
     setAutoDelete( a );
 
-    return TRUE;
+    return true;
 }
 
-bool KSpreadCluster::shiftColumn( const QPoint& marker, bool& work )
+bool Cluster::shiftColumn( const QPoint& marker, bool& work )
 {
-    work = FALSE;
+    work = false;
 
     if ( marker.x() >= KSPREAD_CLUSTER_MAX || marker.x() < 0 ||
 	 marker.y() >= KSPREAD_CLUSTER_MAX || marker.y() < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::shiftColumn: invalid column or row value (col: "
+	kdDebug(36001) << "Cluster::shiftColumn: invalid column or row value (col: "
 		       << marker.x() << "  | row: " << marker.y() << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     int cx = marker.x() / KSPREAD_CLUSTER_LEVEL2;
@@ -324,20 +328,20 @@ bool KSpreadCluster::shiftColumn( const QPoint& marker, bool& work )
 
     // Is there a cell at the right most position ?
     // In this case the shift is impossible.
-    KSpreadCell** cl = m_cluster[ KSPREAD_CLUSTER_LEVEL1 * ( KSPREAD_CLUSTER_LEVEL1 - 1 ) + cx ];
+    Cell** cl = m_cluster[ KSPREAD_CLUSTER_LEVEL1 * ( KSPREAD_CLUSTER_LEVEL1 - 1 ) + cx ];
     if ( cl && cl[ KSPREAD_CLUSTER_LEVEL2 * ( KSPREAD_CLUSTER_LEVEL2 - 1 ) + dx ] )
-	return FALSE;
+	return false;
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     // Move cells in this column one right.
     for( int i = KSPREAD_CLUSTER_LEVEL1 - 1; i >= cy ; --i )
     {
-	KSpreadCell** cl = m_cluster[ i * KSPREAD_CLUSTER_LEVEL1 + cx ];
+	Cell** cl = m_cluster[ i * KSPREAD_CLUSTER_LEVEL1 + cx ];
 	if ( cl )
         {
-	    work = TRUE;
+	    work = true;
 
 	    int top = 0;
 	    if ( i == cy )
@@ -347,7 +351,7 @@ bool KSpreadCluster::shiftColumn( const QPoint& marker, bool& work )
 		bottom = KSPREAD_CLUSTER_LEVEL2 - 2;
 	    for( int k = bottom; k >= top; --k )
 	    {
-		KSpreadCell* c = cl[ k * KSPREAD_CLUSTER_LEVEL2 + dx ];
+		Cell* c = cl[ k * KSPREAD_CLUSTER_LEVEL2 + dx ];
 		if ( c )
 	        {
 		    remove( c->column(), c->row() );
@@ -360,77 +364,77 @@ bool KSpreadCluster::shiftColumn( const QPoint& marker, bool& work )
 
     setAutoDelete( a );
 
-    return TRUE;
+    return true;
 }
 
-bool KSpreadCluster::insertColumn( int col )
+bool Cluster::insertColumn( int col )
 {
     if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::insertColumn: invalid column value (col: "
+	kdDebug(36001) << "Cluster::insertColumn: invalid column value (col: "
 		       << col << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     // Is there a cell at the right most position ?
     // In this case the shift is impossible.
     for( int t1 = 0; t1 < KSPREAD_CLUSTER_LEVEL1; ++t1 )
     {
-	KSpreadCell** cl = m_cluster[ t1 * KSPREAD_CLUSTER_LEVEL1 + KSPREAD_CLUSTER_LEVEL1 - 1 ];
+	Cell** cl = m_cluster[ t1 * KSPREAD_CLUSTER_LEVEL1 + KSPREAD_CLUSTER_LEVEL1 - 1 ];
 	if ( cl )
 	    for( int t2 = 0; t2 < KSPREAD_CLUSTER_LEVEL2; ++t2 )
 		if ( cl[ t2 * KSPREAD_CLUSTER_LEVEL2 + KSPREAD_CLUSTER_LEVEL2 - 1 ] )
-		    return FALSE;
+		    return false;
     }
 
     for( int t1 = 0; t1 < KSPREAD_CLUSTER_LEVEL1; ++t1 )
     {
-	bool work = TRUE;
+	bool work = true;
 	for( int t2 = 0; work && t2 < KSPREAD_CLUSTER_LEVEL2; ++t2 )
 	    shiftRow( QPoint( col, t1 * KSPREAD_CLUSTER_LEVEL2 + t2 ), work );
     }
 
-    return TRUE;
+    return true;
 }
 
-bool KSpreadCluster::insertRow( int row )
+bool Cluster::insertRow( int row )
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::insertRow: invalid row value (row: "
+	kdDebug(36001) << "Cluster::insertRow: invalid row value (row: "
 		       << row << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     // Is there a cell at the bottom most position ?
     // In this case the shift is impossible.
     for( int t1 = 0; t1 < KSPREAD_CLUSTER_LEVEL1; ++t1 )
     {
-	KSpreadCell** cl = m_cluster[ KSPREAD_CLUSTER_LEVEL1 * ( KSPREAD_CLUSTER_LEVEL1 - 1 ) + t1 ];
+	Cell** cl = m_cluster[ KSPREAD_CLUSTER_LEVEL1 * ( KSPREAD_CLUSTER_LEVEL1 - 1 ) + t1 ];
 	if ( cl )
 	    for( int t2 = 0; t2 < KSPREAD_CLUSTER_LEVEL2; ++t2 )
 		if ( cl[ KSPREAD_CLUSTER_LEVEL2 * ( KSPREAD_CLUSTER_LEVEL2 - 1 ) + t2 ] )
-		    return FALSE;
+		    return false;
     }
 
     for( int t1 = 0; t1 < KSPREAD_CLUSTER_LEVEL1; ++t1 )
     {
-	bool work = TRUE;
+	bool work = true;
 	for( int t2 = 0; work && t2 < KSPREAD_CLUSTER_LEVEL2; ++t2 )
 	    shiftColumn( QPoint( t1 * KSPREAD_CLUSTER_LEVEL2 + t2, row ), work );
     }
 
-    return TRUE;
+    return true;
 }
 
-void KSpreadCluster::unshiftColumn( const QPoint& marker, bool& work )
+void Cluster::unshiftColumn( const QPoint& marker, bool& work )
 {
-    work = FALSE;
+    work = false;
 
     if ( marker.x() >= KSPREAD_CLUSTER_MAX || marker.x() < 0 ||
 	 marker.y() >= KSPREAD_CLUSTER_MAX || marker.y() < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::unshiftColumn: invalid column or row value (col: "
+	kdDebug(36001) << "Cluster::unshiftColumn: invalid column or row value (col: "
 		       << marker.x() << "  | row: " << marker.y() << ")" << endl;
 	return;
     }
@@ -441,15 +445,15 @@ void KSpreadCluster::unshiftColumn( const QPoint& marker, bool& work )
     int dy = marker.y() % KSPREAD_CLUSTER_LEVEL2;
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     // Move cells in this column one column to the left.
     for( int i = cy; i < KSPREAD_CLUSTER_LEVEL1; ++i )
     {
-	KSpreadCell** cl = m_cluster[ i * KSPREAD_CLUSTER_LEVEL1 + cx ];
+	Cell** cl = m_cluster[ i * KSPREAD_CLUSTER_LEVEL1 + cx ];
 	if ( cl )
         {
-	    work = TRUE;
+	    work = true;
 
 	    int top = 0;
 	    if ( i == cy )
@@ -457,7 +461,7 @@ void KSpreadCluster::unshiftColumn( const QPoint& marker, bool& work )
 	    int bottom = KSPREAD_CLUSTER_LEVEL2 - 1;
 	    for( int k = top; k <= bottom; ++k )
 	    {
-		KSpreadCell* c = cl[ k * KSPREAD_CLUSTER_LEVEL2 + dx ];
+		Cell* c = cl[ k * KSPREAD_CLUSTER_LEVEL2 + dx ];
 		if ( c )
 	        {
 		    remove( c->column(), c->row() );
@@ -471,14 +475,14 @@ void KSpreadCluster::unshiftColumn( const QPoint& marker, bool& work )
     setAutoDelete( a );
 }
 
-void KSpreadCluster::unshiftRow( const QPoint& marker, bool& work )
+void Cluster::unshiftRow( const QPoint& marker, bool& work )
 {
-    work = FALSE;
+    work = false;
 
     if ( marker.x() >= KSPREAD_CLUSTER_MAX || marker.x() < 0 ||
 	 marker.y() >= KSPREAD_CLUSTER_MAX || marker.y() < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::unshiftRow: invalid column or row value (col: "
+	kdDebug(36001) << "Cluster::unshiftRow: invalid column or row value (col: "
 		       << marker.x() << "  | row: " << marker.y() << ")" << endl;
 	return;
     }
@@ -489,15 +493,15 @@ void KSpreadCluster::unshiftRow( const QPoint& marker, bool& work )
     int dy = marker.y() % KSPREAD_CLUSTER_LEVEL2;
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     // Move cells in this row one row up.
     for( int i = cx; i < KSPREAD_CLUSTER_LEVEL1; ++i )
     {
-	KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + i ];
+	Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + i ];
 	if ( cl )
         {
-	    work = TRUE;
+	    work = true;
 
 	    int left = 0;
 	    if ( i == cx )
@@ -505,7 +509,7 @@ void KSpreadCluster::unshiftRow( const QPoint& marker, bool& work )
 	    int right = KSPREAD_CLUSTER_LEVEL2 - 1;
 	    for( int k = left; k <= right; ++k )
 	    {
-		KSpreadCell* c = cl[ dy * KSPREAD_CLUSTER_LEVEL2 + k ];
+		Cell* c = cl[ dy * KSPREAD_CLUSTER_LEVEL2 + k ];
 		if ( c )
 	        {
 		    remove( c->column(), c->row() );
@@ -519,11 +523,11 @@ void KSpreadCluster::unshiftRow( const QPoint& marker, bool& work )
     setAutoDelete( a );
 }
 
-void KSpreadCluster::removeColumn( int col )
+void Cluster::removeColumn( int col )
 {
     if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::removeColumn: invalid column value (col: "
+	kdDebug(36001) << "Cluster::removeColumn: invalid column value (col: "
 		       << col << ")" << endl;
 	return;
     }
@@ -533,7 +537,7 @@ void KSpreadCluster::removeColumn( int col )
 
     for( int y1 = 0; y1 < KSPREAD_CLUSTER_LEVEL1; ++y1 )
     {
-	KSpreadCell** cl = m_cluster[ y1 * KSPREAD_CLUSTER_LEVEL1 + cx ];
+	Cell** cl = m_cluster[ y1 * KSPREAD_CLUSTER_LEVEL1 + cx ];
 	if ( cl )
 	    for( int y2 = 0; y2 < KSPREAD_CLUSTER_LEVEL2; ++y2 )
 		if ( cl[ y2 * KSPREAD_CLUSTER_LEVEL2 + dx ] )
@@ -542,17 +546,17 @@ void KSpreadCluster::removeColumn( int col )
 
     for( int t1 = 0; t1 < KSPREAD_CLUSTER_LEVEL1; ++t1 )
     {
-	bool work = TRUE;
+	bool work = true;
 	for( int t2 = 0; work && t2 < KSPREAD_CLUSTER_LEVEL2; ++t2 )
 	    unshiftRow( QPoint( col, t1 * KSPREAD_CLUSTER_LEVEL2 + t2 ), work );
     }
 }
 
-void KSpreadCluster::removeRow( int row )
+void Cluster::removeRow( int row )
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadCluster::removeRow: invalid row value (row: "
+	kdDebug(36001) << "Cluster::removeRow: invalid row value (row: "
 		       << row << ")" << endl;
 	return;
     }
@@ -562,7 +566,7 @@ void KSpreadCluster::removeRow( int row )
 
     for( int x1 = 0; x1 < KSPREAD_CLUSTER_LEVEL1; ++x1 )
     {
-	KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + x1 ];
+	Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + x1 ];
 	if ( cl )
 	    for( int x2 = 0; x2 < KSPREAD_CLUSTER_LEVEL2; ++x2 )
 		if ( cl[ dy * KSPREAD_CLUSTER_LEVEL2 + x2 ] )
@@ -571,17 +575,17 @@ void KSpreadCluster::removeRow( int row )
 
     for( int t1 = 0; t1 < KSPREAD_CLUSTER_LEVEL1; ++t1 )
     {
-	bool work = TRUE;
+	bool work = true;
 	for( int t2 = 0; work && t2 < KSPREAD_CLUSTER_LEVEL2; ++t2 )
 	    unshiftColumn( QPoint( t1 * KSPREAD_CLUSTER_LEVEL2 + t2, row ), work );
     }
 }
 
-void KSpreadCluster::clearColumn( int col )
+void Cluster::clearColumn( int col )
 {
   if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
   {
-    kdDebug(36001) << "KSpreadCluster::clearColumn: invalid column value (col: "
+    kdDebug(36001) << "Cluster::clearColumn: invalid column value (col: "
     << col << ")" << endl;
     return;
   }
@@ -591,7 +595,7 @@ void KSpreadCluster::clearColumn( int col )
 
   for( int cy = 0; cy < KSPREAD_CLUSTER_LEVEL1; ++cy )
   {
-    KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
+    Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL1 + cx ];
     if ( cl )
       for( int dy = 0; dy < KSPREAD_CLUSTER_LEVEL2; ++dy )
         if ( cl[ dy * KSPREAD_CLUSTER_LEVEL2 + dx ] )
@@ -602,11 +606,11 @@ void KSpreadCluster::clearColumn( int col )
   }
 }
 
-void KSpreadCluster::clearRow( int row )
+void Cluster::clearRow( int row )
 {
   if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
   {
-    kdDebug(36001) << "KSpreadCluster::clearRow: invalid row value (row: "
+    kdDebug(36001) << "Cluster::clearRow: invalid row value (row: "
         << row << ")" << endl;
     return;
   }
@@ -616,7 +620,7 @@ void KSpreadCluster::clearRow( int row )
 
   for( int cx = 0; cx < KSPREAD_CLUSTER_LEVEL1; ++cx )
   {
-    KSpreadCell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL2 + cx ];
+    Cell** cl = m_cluster[ cy * KSPREAD_CLUSTER_LEVEL2 + cx ];
     if ( cl )
       for( int dx = 0; dx < KSPREAD_CLUSTER_LEVEL2; ++dx )
         if ( cl[ dy * KSPREAD_CLUSTER_LEVEL2 + dx ] )
@@ -627,11 +631,11 @@ void KSpreadCluster::clearRow( int row )
   }
 }
 
-KSpreadValue KSpreadCluster::valueRange (int col1, int row1,
+Value Cluster::valueRange (int col1, int row1,
     int col2, int row2) const
 {
-  KSpreadValue empty;
-  
+  Value empty;
+
   //swap first/second values if needed
   if (col1 > col2)
   {
@@ -645,77 +649,45 @@ KSpreadValue KSpreadCluster::valueRange (int col1, int row1,
       (col2 > KSPREAD_CLUSTER_MAX))
     return empty;
 
-  //return nothing if we are out of range occupied by cells
+  // if we are out of range occupied by cells, we return an empty
+  // array of the requested size
   if ((row1 > m_biggestY) || (col1 > m_biggestX))
-    return empty;
-
-  //split the requested range into cluster-sized sections
-  int x1, y1, x2, y2;
-  x1 = col1 / KSPREAD_CLUSTER_LEVEL2;
-  y1 = row1 / KSPREAD_CLUSTER_LEVEL2;
-  x2 = col2 / KSPREAD_CLUSTER_LEVEL2;
-  y2 = row2 / KSPREAD_CLUSTER_LEVEL2;
-  
-  if ((x1 == x2) && (y1 == y2))  //start/end in the same chunk
   {
-    return makeArray (col1, row1, col2, row2);
+    int cols = col2 - col1 + 1;
+    int rows = row2 - row1 + 1;
+    Value array (cols, rows);
+    return array;
   }
-  
-  KSpreadValue result (x2 - x1 + 1, y2 - y1 + 1);
-  
-  //fill up each section with cell values - if a cell contains a range,
-  //we only take one its element - or else we'd have problems with
-  //matrix functions and expansion of returned matrices to nearby cells
-  for (int y = y1; y <= y2; ++y)
-    for (int x = x1; x <= x2; ++x)
-      //check whether the current element contains anything
-      if (m_cluster[y * KSPREAD_CLUSTER_LEVEL1 + x])
-      {
-        //compute start/end indexes
-        int xx1, xx2, yy1, yy2;
-        xx1 = x * KSPREAD_CLUSTER_LEVEL2;
-        xx2 = (x + 1) * KSPREAD_CLUSTER_LEVEL2 - 1;
-        yy1 = y * KSPREAD_CLUSTER_LEVEL2;
-        yy2 = (y + 1) * KSPREAD_CLUSTER_LEVEL2 - 1;
-        if (xx1 < col1) xx1 = col1;
-        if (yy1 < row1) yy1 = row1;
-        if (xx2 > col2) xx2 = col2;
-        if (yy2 > row2) yy2 = row2;
-        
-        //fill the array element
-        result.setElement (x, y, makeArray (xx1, yy1, xx2, yy2));
-      }
-  //now return the result
-  return result;
+
+  return makeArray (col1, row1, col2, row2);
 }
 
-KSpreadValue KSpreadCluster::makeArray (int col1, int row1,
+Value Cluster::makeArray (int col1, int row1,
     int col2, int row2) const
 {
-  //this generates a simple, one-dimensional array
+  // this generates an array of values
+  // TODO: make this thing faster by skipping empty regions
   int cols = col2 - col1 + 1;
   int rows = row2 - row1 + 1;
-  KSpreadValue array (cols, rows);
+  Value array (cols, rows);
   for (int row = row1; row <= row2; ++row)
     for (int col = col1; col <= col2; ++col)
     {
-      KSpreadCell *cell = lookup (col, row);
+      Cell *cell = lookup (col, row);
       if (cell)
       {
-        KSpreadValue val = cell->value();
-        while (val.isArray ())
-          val = val.element (0, 0);
-        array.setElement (col-col1, row-row1, val); 
+        Value val = cell->value();
+        array.setElement (col-col1, row-row1, val);
       }
     }
-  
+
   //return the result
   return array;
 }
 
-KSpreadCell* KSpreadCluster::getFirstCellColumn(int col) const
+Cell* Cluster::getFirstCellColumn(int col) const
 {
-  KSpreadCell* cell = lookup(col, 1);
+  Cell* cell = lookup(col, 1);
 
   if (cell == NULL)
   {
@@ -724,9 +696,9 @@ KSpreadCell* KSpreadCluster::getFirstCellColumn(int col) const
   return cell;
 }
 
-KSpreadCell* KSpreadCluster::getLastCellColumn(int col) const
+Cell* Cluster::getLastCellColumn(int col) const
 {
-  KSpreadCell* cell = lookup(col, KS_rowMax);
+  Cell* cell = lookup(col, KS_rowMax);
 
   if (cell == NULL)
   {
@@ -735,9 +707,9 @@ KSpreadCell* KSpreadCluster::getLastCellColumn(int col) const
   return cell;
 }
 
-KSpreadCell* KSpreadCluster::getFirstCellRow(int row) const
+Cell* Cluster::getFirstCellRow(int row) const
 {
-  KSpreadCell* cell = lookup(1, row);
+  Cell* cell = lookup(1, row);
 
   if (cell == NULL)
   {
@@ -746,9 +718,9 @@ KSpreadCell* KSpreadCluster::getFirstCellRow(int row) const
   return cell;
 }
 
-KSpreadCell* KSpreadCluster::getLastCellRow(int row) const
+Cell* Cluster::getLastCellRow(int row) const
 {
-  KSpreadCell* cell = lookup(KS_colMax, row);
+  Cell* cell = lookup(KS_colMax, row);
 
   if (cell == NULL)
   {
@@ -757,7 +729,7 @@ KSpreadCell* KSpreadCluster::getLastCellRow(int row) const
   return cell;
 }
 
-KSpreadCell* KSpreadCluster::getNextCellUp(int col, int row) const
+Cell* Cluster::getNextCellUp(int col, int row) const
 {
   int cx = col / KSPREAD_CLUSTER_LEVEL2;
   int cy = (row - 1) / KSPREAD_CLUSTER_LEVEL2;
@@ -786,7 +758,7 @@ KSpreadCell* KSpreadCluster::getNextCellUp(int col, int row) const
   return NULL;
 }
 
-KSpreadCell* KSpreadCluster::getNextCellDown(int col, int row) const
+Cell* Cluster::getNextCellDown(int col, int row) const
 {
   int cx = col / KSPREAD_CLUSTER_LEVEL2;
   int cy = (row + 1) / KSPREAD_CLUSTER_LEVEL2;
@@ -815,7 +787,7 @@ KSpreadCell* KSpreadCluster::getNextCellDown(int col, int row) const
   return NULL;
 }
 
-KSpreadCell* KSpreadCluster::getNextCellLeft(int col, int row) const
+Cell* Cluster::getNextCellLeft(int col, int row) const
 {
   int cx = (col - 1) / KSPREAD_CLUSTER_LEVEL2;
   int cy = row / KSPREAD_CLUSTER_LEVEL2;
@@ -844,7 +816,7 @@ KSpreadCell* KSpreadCluster::getNextCellLeft(int col, int row) const
   return NULL;
 }
 
-KSpreadCell* KSpreadCluster::getNextCellRight(int col, int row) const
+Cell* Cluster::getNextCellRight(int col, int row) const
 {
   int cx = (col + 1) / KSPREAD_CLUSTER_LEVEL2;
   int cy = row / KSPREAD_CLUSTER_LEVEL2;
@@ -875,12 +847,12 @@ KSpreadCell* KSpreadCluster::getNextCellRight(int col, int row) const
 
 /****************************************************
  *
- * KSpreadColumnCluster
+ * ColumnCluster
  *
  ****************************************************/
 
-KSpreadColumnCluster::KSpreadColumnCluster()
-    : m_first( 0 ), m_autoDelete( FALSE )
+ColumnCluster::ColumnCluster()
+    : m_first( 0 ), m_autoDelete( false )
 {
     m_cluster = (ColumnFormat***)malloc( KSPREAD_CLUSTER_LEVEL1 * sizeof( ColumnFormat** ) );
 
@@ -888,7 +860,7 @@ KSpreadColumnCluster::KSpreadColumnCluster()
 	m_cluster[ x ] = 0;
 }
 
-KSpreadColumnCluster::~KSpreadColumnCluster()
+ColumnCluster::~ColumnCluster()
 {
     for( int x = 0; x < KSPREAD_CLUSTER_LEVEL1; ++x )
     {
@@ -915,11 +887,11 @@ KSpreadColumnCluster::~KSpreadColumnCluster()
     free( m_cluster );
 }
 
-ColumnFormat* KSpreadColumnCluster::lookup( int col )
+ColumnFormat* ColumnCluster::lookup( int col )
 {
     if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
     {
-	kdDebug(36001) << "KSpreadColumnCluster::lookup: invalid column value (col: "
+	kdDebug(36001) << "ColumnCluster::lookup: invalid column value (col: "
 		       << col << ")" << endl;
 	return 0;
     }
@@ -934,11 +906,11 @@ ColumnFormat* KSpreadColumnCluster::lookup( int col )
     return cl[ dx ];
 }
 
-const ColumnFormat* KSpreadColumnCluster::lookup( int col ) const
+const ColumnFormat* ColumnCluster::lookup( int col ) const
 {
     if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
     {
-	kdDebug(36001) << "KSpreadColumnCluster::lookup: invalid column value (col: "
+	kdDebug(36001) << "ColumnCluster::lookup: invalid column value (col: "
 		       << col << ")" << endl;
 	return 0;
     }
@@ -953,7 +925,7 @@ const ColumnFormat* KSpreadColumnCluster::lookup( int col ) const
     return cl[ dx ];
 }
 
-void KSpreadColumnCluster::clear()
+void ColumnCluster::clear()
 {
     for( int x = 0; x < KSPREAD_CLUSTER_LEVEL1; ++x )
     {
@@ -979,11 +951,11 @@ void KSpreadColumnCluster::clear()
     m_first = 0;
 }
 
-void KSpreadColumnCluster::insertElement( ColumnFormat* lay, int col )
+void ColumnCluster::insertElement( ColumnFormat* lay, int col )
 {
     if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
     {
-	kdDebug(36001) << "KSpreadColumnCluster::insertElement: invalid column value (col: "
+	kdDebug(36001) << "ColumnCluster::insertElement: invalid column value (col: "
 		       << col << ")" << endl;
 	return;
     }
@@ -1014,11 +986,11 @@ void KSpreadColumnCluster::insertElement( ColumnFormat* lay, int col )
     m_first = lay;
 }
 
-void KSpreadColumnCluster::removeElement( int col )
+void ColumnCluster::removeElement( int col )
 {
     if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
     {
-	kdDebug(36001) << "KSpreadColumnCluster::removeElement: invalid column value (col: "
+	kdDebug(36001) << "ColumnCluster::removeElement: invalid column value (col: "
 		       << col << ")" << endl;
 	return;
     }
@@ -1055,13 +1027,13 @@ void KSpreadColumnCluster::removeElement( int col )
     }
 }
 
-bool KSpreadColumnCluster::insertColumn( int col )
+bool ColumnCluster::insertColumn( int col )
 {
     if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
     {
-	kdDebug(36001) << "KSpreadColumnCluster::insertColumn: invalid column value (col: "
+	kdDebug(36001) << "ColumnCluster::insertColumn: invalid column value (col: "
 		       << col << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     int cx = col / KSPREAD_CLUSTER_LEVEL2;
@@ -1071,10 +1043,10 @@ bool KSpreadColumnCluster::insertColumn( int col )
     // In this case the shift is impossible.
     ColumnFormat** cl = m_cluster[ KSPREAD_CLUSTER_LEVEL1 - 1 ];
     if ( cl && cl[ KSPREAD_CLUSTER_LEVEL2 - 1 ] )
-	return FALSE;
+	return false;
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     for( int i = KSPREAD_CLUSTER_LEVEL1 - 1; i >= cx ; --i )
     {
@@ -1102,16 +1074,16 @@ bool KSpreadColumnCluster::insertColumn( int col )
 
     setAutoDelete( a );
 
-    return TRUE;
+    return true;
 }
 
-bool KSpreadColumnCluster::removeColumn( int column )
+bool ColumnCluster::removeColumn( int column )
 {
     if ( column >= KSPREAD_CLUSTER_MAX || column < 0 )
     {
-	kdDebug(36001) << "KSpreadColumnCluster::removeColumn: invalid column value (col: "
+	kdDebug(36001) << "ColumnCluster::removeColumn: invalid column value (col: "
 		       << column << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     int cx = column / KSPREAD_CLUSTER_LEVEL2;
@@ -1120,7 +1092,7 @@ bool KSpreadColumnCluster::removeColumn( int column )
     removeElement( column );
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     for( int i = cx; i < KSPREAD_CLUSTER_LEVEL1; ++i )
     {
@@ -1146,27 +1118,59 @@ bool KSpreadColumnCluster::removeColumn( int column )
 
     setAutoDelete( a );
 
-    return TRUE;
+    return true;
 }
 
-void KSpreadColumnCluster::setAutoDelete( bool a )
+void ColumnCluster::setAutoDelete( bool a )
 {
     m_autoDelete = a;
 }
 
-bool KSpreadColumnCluster::autoDelete() const
+bool ColumnCluster::autoDelete() const
 {
     return m_autoDelete;
 }
 
+ColumnFormat* ColumnCluster::next( int col ) const
+{
+  if ( col >= KSPREAD_CLUSTER_MAX || col < 0 )
+  {
+    kdDebug(36001) << "ColumnCluster::next: invalid column value (col: "
+        << col << ")" << endl;
+    return 0;
+  }
+
+  int cx = (col + 1) / KSPREAD_CLUSTER_LEVEL2;
+  int dx = (col + 1) % KSPREAD_CLUSTER_LEVEL2;
+
+  while ( cx < KSPREAD_CLUSTER_LEVEL1 )
+  {
+    if ( m_cluster[ cx ] )
+    {
+      while ( dx < KSPREAD_CLUSTER_LEVEL2 )
+      {
+
+        if ( m_cluster[ cx ][  dx ] )
+        {
+          return m_cluster[ cx ][ dx ];
+        }
+        ++dx;
+      }
+    }
+    ++cx;
+    dx = 0;
+  }
+  return 0;
+}
+
 /****************************************************
  *
- * KSpreadRowCluster
+ * RowCluster
  *
  ****************************************************/
 
-KSpreadRowCluster::KSpreadRowCluster()
-    : m_first( 0 ), m_autoDelete( FALSE )
+RowCluster::RowCluster()
+    : m_first( 0 ), m_autoDelete( false )
 {
     m_cluster = (RowFormat***)malloc( KSPREAD_CLUSTER_LEVEL1 * sizeof( RowFormat** ) );
 
@@ -1174,7 +1178,7 @@ KSpreadRowCluster::KSpreadRowCluster()
 	    m_cluster[ x ] = 0;
 }
 
-KSpreadRowCluster::~KSpreadRowCluster()
+RowCluster::~RowCluster()
 {
     for( int x = 0; x < KSPREAD_CLUSTER_LEVEL1; ++x )
     {
@@ -1200,11 +1204,11 @@ KSpreadRowCluster::~KSpreadRowCluster()
     free( m_cluster );
 }
 
-const RowFormat* KSpreadRowCluster::lookup( int row ) const
+const RowFormat* RowCluster::lookup( int row ) const
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadRowCluster::lookup: invalid row value (row: "
+	kdDebug(36001) << "RowCluster::lookup: invalid row value (row: "
 		       << row << ")" << endl;
 	return 0;
     }
@@ -1219,11 +1223,11 @@ const RowFormat* KSpreadRowCluster::lookup( int row ) const
     return cl[ dx ];
 }
 
-RowFormat* KSpreadRowCluster::lookup( int row )
+RowFormat* RowCluster::lookup( int row )
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadRowCluster::lookup: invalid row value (row: "
+	kdDebug(36001) << "RowCluster::lookup: invalid row value (row: "
 		       << row << ")" << endl;
 	return 0;
     }
@@ -1238,7 +1242,7 @@ RowFormat* KSpreadRowCluster::lookup( int row )
     return cl[ dx ];
 }
 
-void KSpreadRowCluster::clear()
+void RowCluster::clear()
 {
     for( int x = 0; x < KSPREAD_CLUSTER_LEVEL1; ++x )
     {
@@ -1264,11 +1268,11 @@ void KSpreadRowCluster::clear()
     m_first = 0;
 }
 
-void KSpreadRowCluster::insertElement( RowFormat* lay, int row )
+void RowCluster::insertElement( RowFormat* lay, int row )
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadRowCluster::insertElement: invalid row value (row: "
+	kdDebug(36001) << "RowCluster::insertElement: invalid row value (row: "
 		       << row << ")" << endl;
 	return;
     }
@@ -1299,11 +1303,11 @@ void KSpreadRowCluster::insertElement( RowFormat* lay, int row )
     m_first = lay;
 }
 
-void KSpreadRowCluster::removeElement( int row )
+void RowCluster::removeElement( int row )
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadRowCluster::removeElement: invalid row value (row: "
+	kdDebug(36001) << "RowCluster::removeElement: invalid row value (row: "
 		       << row << ")" << endl;
 	return;
     }
@@ -1340,13 +1344,13 @@ void KSpreadRowCluster::removeElement( int row )
     }
 }
 
-bool KSpreadRowCluster::insertRow( int row )
+bool RowCluster::insertRow( int row )
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadRowCluster::insertRow: invalid row value (row: "
+	kdDebug(36001) << "RowCluster::insertRow: invalid row value (row: "
 		       << row << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     int cx = row / KSPREAD_CLUSTER_LEVEL2;
@@ -1356,10 +1360,10 @@ bool KSpreadRowCluster::insertRow( int row )
     // In this case the shift is impossible.
     RowFormat** cl = m_cluster[ KSPREAD_CLUSTER_LEVEL1 - 1 ];
     if ( cl && cl[ KSPREAD_CLUSTER_LEVEL2 - 1 ] )
-	return FALSE;
+	return false;
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     for( int i = KSPREAD_CLUSTER_LEVEL1 - 1; i >= cx ; --i )
     {
@@ -1387,16 +1391,16 @@ bool KSpreadRowCluster::insertRow( int row )
 
     setAutoDelete( a );
 
-    return TRUE;
+    return true;
 }
 
-bool KSpreadRowCluster::removeRow( int row )
+bool RowCluster::removeRow( int row )
 {
     if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
     {
-	kdDebug(36001) << "KSpreadRowCluster::removeRow: invalid row value (row: "
+	kdDebug(36001) << "RowCluster::removeRow: invalid row value (row: "
 		       << row << ")" << endl;
-	return FALSE;
+	return false;
     }
 
     int cx = row / KSPREAD_CLUSTER_LEVEL2;
@@ -1405,7 +1409,7 @@ bool KSpreadRowCluster::removeRow( int row )
     removeElement( row );
 
     bool a = autoDelete();
-    setAutoDelete( FALSE );
+    setAutoDelete( false );
 
     for( int i = cx; i < KSPREAD_CLUSTER_LEVEL1; ++i )
     {
@@ -1431,15 +1435,47 @@ bool KSpreadRowCluster::removeRow( int row )
 
     setAutoDelete( a );
 
-    return TRUE;
+    return true;
 }
 
-void KSpreadRowCluster::setAutoDelete( bool a )
+void RowCluster::setAutoDelete( bool a )
 {
     m_autoDelete = a;
 }
 
-bool KSpreadRowCluster::autoDelete() const
+bool RowCluster::autoDelete() const
 {
     return m_autoDelete;
 }
+
+RowFormat* RowCluster::next( int row ) const
+{
+  if ( row >= KSPREAD_CLUSTER_MAX || row < 0 )
+  {
+    kdDebug(36001) << "RowCluster::removeRow: invalid row value (row: "
+        << row << ")" << endl;
+    return false;
+  }
+
+  int cx = (row + 1) / KSPREAD_CLUSTER_LEVEL2;
+  int dx = (row + 1) % KSPREAD_CLUSTER_LEVEL2;
+
+  while ( cx < KSPREAD_CLUSTER_LEVEL1 )
+  {
+    if ( m_cluster[ cx ] )
+    {
+      while ( dx < KSPREAD_CLUSTER_LEVEL2 )
+      {
+
+        if ( m_cluster[ cx ][  dx ] )
+        {
+          return m_cluster[ cx ][ dx ];
+        }
+        ++dx;
+      }
+    }
+    ++cx;
+    dx = 0;
+  }
+  return 0;
+}
