@@ -13,11 +13,28 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
-/* Philipp
+#ifndef kspread_cluster_h
+#define kspread_cluster_h
+
+#include "kspread_value.h"
+
+#define KSPREAD_CLUSTER_LEVEL1 128
+#define KSPREAD_CLUSTER_LEVEL2 256
+#define KSPREAD_CLUSTER_MAX (128*256)
+
+class QPoint;
+
+namespace KSpread
+{
+class Cell;
+class ColumnFormat;
+class RowFormat;
+
+/** Philipp
 This class defines a pointer map to all cells, which makes access to them more performant
 and additionally limits memory consumption.
 
@@ -52,30 +69,16 @@ are equal in both matrizes, but normally it will be the regular case, that you h
 a lot of rows than columns. Maybe something like LEVEL1=128/256 and LEVEL2=256/128 (x/y), still keeping
 2^15 values/cells in each direction (benefit: you won't loose memory in empty columns).
 */
-
-#ifndef kspread_cluster_h
-#define kspread_cluster_h
-
-class KSpreadCell;
-class ColumnFormat;
-class RowFormat;
-
-class QPoint;
-
-#define KSPREAD_CLUSTER_LEVEL1 128
-#define KSPREAD_CLUSTER_LEVEL2 256
-#define KSPREAD_CLUSTER_MAX (128*256)
-
-class KSpreadCluster
+class Cluster
 {
 public:
-    KSpreadCluster();
-    ~KSpreadCluster();
+    Cluster();
+    ~Cluster();
 
-    KSpreadCell* lookup( int x, int y ) const;
+    Cell* lookup( int x, int y ) const;
 
     /**
-     * Removes all cells from the table and frees memory that
+     * Removes all cells from the sheet and frees memory that
      * was used for the clusters.
      */
     void clear();
@@ -84,7 +87,7 @@ public:
      * Inserts a cell at the requested position. If there is already
      * a cell, then @ref #remove is called on it.
      */
-    void insert( KSpreadCell* cell, int x, int y );
+    void insert( Cell* cell, int x, int y );
     /**
      * Removes the cell at the given position, if there is any.
      */
@@ -93,14 +96,14 @@ public:
     void setAutoDelete( bool );
     bool autoDelete() const;
 
-    KSpreadCell* firstCell() const;
+    Cell* firstCell() const;
 
     bool shiftRow( const QPoint& marker );
     /**
      * Moves all cells in the column marker.x() beginning with
      * the one at marker.y() one position downwards.
      *
-     * @return FALSE if a cell would drop out of the table because of that.
+     * @return false if a cell would drop out of the sheet because of that.
      *         In this case the shift is not performed.
      */
     bool shiftColumn( const QPoint& marker );
@@ -115,7 +118,7 @@ public:
     /**
      * Moves all columns beginning with @p col one position
      * to the right. If that does not work because a cell would
-     * drop out of the table, then FALSE is returned.
+     * drop out of the sheet, then false is returned.
      *
      * @see #removeColumn
      */
@@ -135,10 +138,15 @@ public:
     /**
      * Removes all elements from the column.
      *
-     * @param presereDoM preserve the DependingOnMe lists of the cells if non-empty
      */
-    void clearColumn( int col, bool preserveDoM = false );
-    void clearRow( int row, bool preserveDoM = false );
+    void clearColumn( int col );
+    void clearRow( int row );
+
+  /** Retrieve a range of values stored in a Value.
+  The range is two-leveled with similar structure and reasoning as the
+  storage of cells themselves.
+  */
+  Value valueRange (int col1, int row1, int col2, int row2) const;
 
   /**
    * Retrieve the first used cell in a given column.  Can be used in conjunction
@@ -149,7 +157,7 @@ public:
    * @return Returns a pointer to the cell, or NULL if there are no used cells
    *         in this column
    */
-  KSpreadCell* getFirstCellColumn(int col) const;
+  Cell* getFirstCellColumn(int col) const;
 
   /**
    * Retrieve the last used cell in a given column.  Can be used in conjunction
@@ -160,7 +168,7 @@ public:
    * @return Returns a pointer to the cell, or NULL if there are no used cells
    *         in this column
    */
-  KSpreadCell* getLastCellColumn(int col) const;
+  Cell* getLastCellColumn(int col) const;
 
   /**
    * Retrieve the first used cell in a given row.  Can be used in conjunction
@@ -171,7 +179,7 @@ public:
    * @return Returns a pointer to the cell, or NULL if there are no used cells
    *         in this row
    */
-  KSpreadCell* getFirstCellRow(int row) const;
+  Cell* getFirstCellRow(int row) const;
 
   /**
    * Retrieve the last used cell in a given row.  Can be used in conjunction
@@ -182,7 +190,7 @@ public:
    * @return Returns a pointer to the cell, or NULL if there are no used cells
    *         in this row
    */
-  KSpreadCell* getLastCellRow(int row) const;
+  Cell* getLastCellRow(int row) const;
 
   /**
    * Retrieves the next used cell above the given col/row pair.  The given
@@ -193,7 +201,7 @@ public:
    *
    * @return Returns the next used cell above this one, or NULL if there are none
    */
-  KSpreadCell* getNextCellUp(int col, int row) const;
+  Cell* getNextCellUp(int col, int row) const;
 
   /**
    * Retrieves the next used cell below the given col/row pair.  The given
@@ -204,7 +212,7 @@ public:
    *
    * @return Returns the next used cell below this one, or NULL if there are none
    */
-  KSpreadCell* getNextCellDown(int col, int row) const;
+  Cell* getNextCellDown(int col, int row) const;
 
   /**
    * Retrieves the next used cell to the right of the given col/row pair.
@@ -216,7 +224,7 @@ public:
    * @return Returns the next used cell to the right of this one, or NULL if
    * there are none
    */
-  KSpreadCell* getNextCellRight(int col, int row) const;
+  Cell* getNextCellRight(int col, int row) const;
 
   /**
    * Retrieves the next used cell to the left of the given col/row pair.
@@ -228,11 +236,11 @@ public:
    * @return Returns the next used cell to the left of this one, or NULL if
    * there are none
    */
-  KSpreadCell* getNextCellLeft(int col, int row) const;
+  Cell* getNextCellLeft(int col, int row) const;
 
 private:
     /**
-     * @param work is set to TRUE if the method found some clusters
+     * @param work is set to true if the method found some clusters
      *        which belong to the shifted row.
      */
     bool shiftRow( const QPoint& marker, bool& work );
@@ -241,16 +249,20 @@ private:
     void unshiftColumn( const QPoint& marker, bool& work );
     void unshiftRow( const QPoint& marker, bool& work );
 
-    KSpreadCell*** m_cluster;
-    KSpreadCell* m_first;
+    /** helper method used by valueRange */
+    Value makeArray (int col1, int row1, int col2, int row2) const;
+
+    Cell*** m_cluster;
+    Cell* m_first;
     bool m_autoDelete;
+    int m_biggestX, m_biggestY;
 };
 
-class KSpreadColumnCluster
+class ColumnCluster
 {
 public:
-    KSpreadColumnCluster();
-    ~KSpreadColumnCluster();
+    ColumnCluster();
+    ~ColumnCluster();
 
     const ColumnFormat* lookup( int col ) const;
     ColumnFormat* lookup( int col );
@@ -266,7 +278,8 @@ public:
     void setAutoDelete( bool );
     bool autoDelete() const;
 
-    ColumnFormat* first()const { return m_first; }
+    ColumnFormat* first() const { return m_first; }
+    ColumnFormat* next( int col ) const;
 
 private:
     ColumnFormat*** m_cluster;
@@ -274,11 +287,11 @@ private:
     bool m_autoDelete;
 };
 
-class KSpreadRowCluster
+class RowCluster
 {
 public:
-    KSpreadRowCluster();
-    ~KSpreadRowCluster();
+    RowCluster();
+    ~RowCluster();
 
     const RowFormat* lookup( int col ) const;
     RowFormat* lookup( int col );
@@ -294,7 +307,8 @@ public:
     void setAutoDelete( bool );
     bool autoDelete() const;
 
-    RowFormat* first()const { return m_first; }
+    RowFormat* first() const { return m_first; }
+    RowFormat* next( int row ) const;
 
 private:
     RowFormat*** m_cluster;
@@ -302,4 +316,6 @@ private:
     bool m_autoDelete;
 };
 
+} // namespace KSpread;
+
 #endif
