@@ -15,38 +15,43 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef __kspread_format_h__
 #define __kspread_format_h__
 
-class KSpreadSheet;
-class KSpreadCanvas;
+#include <qbrush.h>
+#include <qcolor.h>
+#include <qfont.h>
+#include <qpen.h>
+
+#include <koffice_export.h>
+
+#include "kspread_global.h"
 
 class QDomElement;
 class QDomDocument;
 class DCOPObject;
-class KoOasisStyles;
-class KoGenStyles;
-
-#include <qbrush.h>
-#include <qpen.h>
-#include <qcolor.h>
-#include <qfont.h>
-
-#include "kspread_global.h"
-#include <koffice_export.h>
 class KLocale;
-class KSpreadCurrency;
-class KSpreadStyle;
-class KoStyleStack;
 class KoGenStyle;
+class KoGenStyles;
+class KoOasisLoadingContext;
+class KoOasisStyles;
+class KoStyleStack;
+
+namespace KSpread
+{
+class Canvas;
+class Cell;
+class Currency;
+class Sheet;
+class Style;
 
 /**
  */
-class KSPREAD_EXPORT KSpreadFormat
+class KSPREAD_EXPORT Format
 {
 public:
     enum Align { Left = 1, Center = 2, Right = 3, Undefined = 4 };
@@ -54,6 +59,7 @@ public:
     enum FloatFormat { AlwaysSigned = 1, AlwaysUnsigned = 2, OnlyNegSigned = 3 };
     enum FloatColor { NegRed = 1, AllBlack = 2, NegBrackets = 3, NegRedBrackets = 4 };
 
+    // TODO Stefan: merge with Style::FlagsSet
     enum Properties{ PAlign  = 0x01,
 		     PAlignY = 0x02,
          //PFactor was here
@@ -91,12 +97,23 @@ public:
     };
 
     /**
-     * don't pass in 0 for _style: only if you copy another format on this directly after...
+     * Constructor.
+     * @param sheet The sheet this format belongs to.
+     * @todo The format should not belong to a sheet. The cell belongs to a sheet. Move
+     * the sheet pointer to Cell.
+     * @param style The style this format should use.
+     * You should pass @c 0 in here, only if you copy another Format on
+     * this directly after.
      */
-    KSpreadFormat( KSpreadSheet * _sheet, KSpreadStyle * _style );
-    virtual ~KSpreadFormat();
+    Format( Sheet* sheet, Style* style );
+    /**
+     * Destructor.
+     * The associated style's reference counter is decremented and if it has no further
+     * references, it is deleted.
+     */
+    virtual ~Format();
 
-    void copy( const KSpreadFormat & _l );
+    void copy( const Format& other );
 
     void defaultStyleFormat();
 
@@ -106,17 +123,20 @@ public:
     //
     ////////////////////////////////
 
-    bool load( const QDomElement & f, PasteMode pm, bool paste = false );
-    bool loadFormat( const QDomElement & f, PasteMode pm = Normal, bool paste = false );
+    bool load( const QDomElement & f, Paste::Mode pm, bool paste = false );
+    bool loadFormat( const QDomElement & f, Paste::Mode pm = Paste::Normal, bool paste = false );
     QDomElement save( QDomDocument& doc,int _col, int _row,bool force = false, bool copy = false ) const;
     QDomElement saveFormat( QDomDocument& doc, bool force = false, bool copy = false ) const;
     QDomElement saveFormat( QDomDocument& doc, int _col, int _row, bool force = false, bool copy = false ) const;
-    virtual bool loadOasisStyleProperties(KoStyleStack & styleStack, const KoOasisStyles& oasisStyles );
-    virtual QString saveOasisCellStyle( KoGenStyle &currentCellStyle, KoGenStyles &mainStyle ,int _col, int _row , bool force = false, bool copy = false);
-    virtual QString saveOasisCellStyle( KoGenStyle &currentCellStyle, KoGenStyles &mainStyle );
-    void saveOasisFontCellStyle( KoGenStyle &currentCellStyle, const QFont &_font );
 
+    void loadOasisStyle(/* const QDomElement& element,*/ KoOasisLoadingContext& context );
+    bool loadOasisStyleProperties(KoStyleStack & styleStack, const KoOasisStyles& oasisStyles );
     bool loadFontOasisStyle( KoStyleStack & font );
+    /**
+     * @return the OASIS style's name, if it was created. QString::null otherwise.
+     */
+    QString saveOasisCellStyle( KoGenStyle &currentCellStyle, KoGenStyles &mainStyle );
+
     ////////////////////////////////
     //
     // Properties
@@ -161,18 +181,19 @@ public:
     static double globalColWidth();
 
 
-    virtual void setKSpreadStyle( KSpreadStyle * style );
+    void setStyle( Style * style );
+    void setCell( Cell* cell ) { m_pCell = cell; }
 
     /**
      * sets the format of the content, e.g. #.##0.00, dd/mmm/yyyy,...
      */
-    virtual void setFormatString( QString const & format );
+    void setFormatString( QString const & format );
 
-    virtual void setAlign( Align _align );
-    virtual void setAlignY( AlignY _alignY );
-    virtual void setPrefix( const QString& _prefix );
-    virtual void setPostfix( const QString& _postfix );
-    virtual void setPrecision( int _p );
+    void setAlign( Align _align );
+    void setAlignY( AlignY _alignY );
+    void setPrefix( const QString& _prefix );
+    void setPostfix( const QString& _postfix );
+    void setPrecision( int _p );
 
     virtual void setLeftBorderPen( const QPen& _p );
     void setLeftBorderStyle( Qt::PenStyle s );
@@ -194,21 +215,21 @@ public:
     void setBottomBorderColor( const QColor & _c );
     void setBottomBorderWidth( int _w );
 
-    virtual void setFallDiagonalPen( const QPen& _p );
+    void setFallDiagonalPen( const QPen& _p );
     void setFallDiagonalStyle( Qt::PenStyle s );
     void setFallDiagonalColor( const QColor & _c );
     void setFallDiagonalWidth( int _w );
 
-    virtual void setGoUpDiagonalPen( const QPen& _p );
+    void setGoUpDiagonalPen( const QPen& _p );
     void setGoUpDiagonalStyle( Qt::PenStyle s );
     void setGoUpDiagonalColor( const QColor & _c );
     void setGoUpDiagonalWidth( int _w );
 
-    virtual void setBackGroundBrush( const QBrush& _p);
+    void setBackGroundBrush( const QBrush& _p);
     void setBackGroundBrushStyle( Qt::BrushStyle s);
     void setBackGroundBrushColor( const QColor & _c);
 
-    virtual void setTextFont( const QFont& _f );
+    void setTextFont( const QFont& _f );
     void setTextFontSize( int _s );
     void setTextFontFamily( const QString& _f );
     void setTextFontBold( bool _b );
@@ -216,33 +237,33 @@ public:
     void setTextFontUnderline( bool _i );
     void setTextFontStrike( bool _i );
 
-    virtual void setTextPen( const QPen& _p );
+    void setTextPen( const QPen& _p );
     void setTextColor( const QColor & _c );
 
-    virtual void setBgColor( const QColor & _c );
+    void setBgColor( const QColor & _c );
 
-    virtual void setFloatFormat( FloatFormat _f );
-    virtual void setFloatColor( FloatColor _c );
+    void setFloatFormat( FloatFormat _f );
+    void setFloatColor( FloatColor _c );
 
-    virtual void setMultiRow( bool _b );
+    void setMultiRow( bool _b );
 
-    virtual void setVerticalText( bool _b );
+    void setVerticalText( bool _b );
 
-    virtual void setFormatType(FormatType _format);
+    void setFormatType(FormatType _format);
 
-    virtual void setAngle(int _angle);
+    void setAngle(int _angle);
 
-    virtual void setComment( const QString& c );
+    void setComment( const QString& c );
 
-    virtual void setIndent( double _indent );
+    void setIndent( double _indent );
 
-    virtual void setDontPrintText ( bool _b );
-    virtual void setNotProtected ( bool _b );
-    virtual void setHideAll( bool _b );
-    virtual void setHideFormula( bool _b );
+    void setDontPrintText( bool _b );
+    void setNotProtected( bool _b );
+    void setHideAll( bool _b );
+    void setHideFormula( bool _b );
 
-    virtual void setCurrency( Currency const & c );
-    virtual void setCurrency( int type, QString const & symbol );
+    void setCurrency( Currency const & c );
+    void setCurrency( int type, QString const & symbol );
 
     ////////////////////////////////
     //
@@ -272,47 +293,47 @@ public:
     Qt::PenStyle bottomBorderStyle( int col, int row ) const;
     const QColor& bottomBorderColor( int col, int row ) const;
 
-    virtual const QPen& fallDiagonalPen( int col, int row ) const;
+    const QPen& fallDiagonalPen( int col, int row ) const;
     int fallDiagonalWidth( int col, int row ) const;
     Qt::PenStyle fallDiagonalStyle( int col, int row ) const;
     const QColor& fallDiagonalColor( int col, int row ) const;
 
-    virtual const QPen& goUpDiagonalPen( int col, int row ) const;
+    const QPen& goUpDiagonalPen( int col, int row ) const;
     int goUpDiagonalWidth( int col, int row ) const;
     Qt::PenStyle goUpDiagonalStyle( int col, int row ) const;
     const QColor& goUpDiagonalColor( int col, int row ) const;
 
-    virtual const QBrush& backGroundBrush( int col, int row ) const;
+    const QBrush& backGroundBrush( int col, int row ) const;
     Qt::BrushStyle backGroundBrushStyle( int col, int row ) const;
     const QColor& backGroundBrushColor(int col, int row ) const;
 
-    virtual uint bottomBorderValue( int col, int row ) const;
-    virtual uint rightBorderValue( int col, int row ) const;
-    virtual uint leftBorderValue( int col, int row ) const;
-    virtual uint topBorderValue( int col, int row ) const;
+    uint bottomBorderValue( int col, int row ) const;
+    uint rightBorderValue( int col, int row ) const;
+    uint leftBorderValue( int col, int row ) const;
+    uint topBorderValue( int col, int row ) const;
 
     /**
      * @return the precision of the floating point representation.
      */
-    virtual int precision( int col, int row ) const;
+    int precision( int col, int row ) const;
     /**
      * @return the prefix of a numeric value ( for example "$" )
      */
-    virtual QString prefix( int col, int row ) const;
+    QString prefix( int col, int row ) const;
     /**
      * @return the postfix of a numeric value ( for example "DM" )
      */
-    virtual QString postfix( int col, int row ) const;
+    QString postfix( int col, int row ) const;
     /**
      * @return the way of formatting a floating point value
      */
-    virtual FloatFormat floatFormat( int col, int row ) const;
+    FloatFormat floatFormat( int col, int row ) const;
     /**
      * @return the color format of a floating point value
      */
-    virtual FloatColor floatColor( int col, int row ) const;
+    FloatColor floatColor( int col, int row ) const;
 
-    virtual const QPen& textPen( int col, int row ) const;
+    const QPen& textPen( int col, int row ) const;
     /**
      * @return the text color.
      */
@@ -324,9 +345,9 @@ public:
      *
      * @return the background color.
      */
-    virtual const QColor& bgColor( int col, int row ) const;
+    const QColor& bgColor( int col, int row ) const;
 
-    virtual const QFont textFont( int col, int row ) const;
+    const QFont textFont( int col, int row ) const;
     int textFontSize( int col, int row ) const;
     QString const & textFontFamily( int col, int row ) const;
     bool textFontBold( int col, int row ) const;
@@ -334,68 +355,69 @@ public:
     bool textFontUnderline( int col, int row ) const;
     bool textFontStrike( int col, int row ) const;
 
-    virtual Align align( int col, int row ) const;
-    virtual AlignY alignY( int col, int row ) const;
+    Align align( int col, int row ) const;
+    AlignY alignY( int col, int row ) const;
 
-    virtual bool multiRow( int col, int row ) const;
+    bool multiRow( int col, int row ) const;
 
-    virtual bool verticalText( int col, int row ) const;
+    bool verticalText( int col, int row ) const;
 
-    virtual FormatType getFormatType(int col, int row )const ;
+    FormatType getFormatType(int col, int row )const ;
 
-    virtual int getAngle(int col, int row) const;
+    int getAngle(int col, int row) const;
 
-    virtual QString comment(int col, int row) const;
-    virtual QString * commentP( int col, int row ) const;
+    const QString* comment() const { return m_strComment; };
+    QString comment(int col, int row) const;
+    QString * commentP( int col, int row ) const;
 
-    virtual double getIndent(int col, int row) const;
+    double getIndent(int col, int row) const;
 
-    virtual bool getDontprintText( int col, int row) const;
-    virtual bool notProtected( int col, int row) const;
-    virtual bool isHideAll( int col, int row) const;
-    virtual bool isHideFormula( int col, int row) const;
-    virtual bool isProtected( int col, int row ) const;
+    bool getDontprintText( int col, int row) const;
+    bool notProtected( int col, int row) const;
+    bool isHideAll( int col, int row) const;
+    bool isHideFormula( int col, int row) const;
+    bool isProtected( int col, int row ) const;
 
-    KSpreadStyle * kspreadStyle() const { return m_pStyle; }
-    KSpreadSheet* sheet() { return m_pSheet; }
-    const KSpreadSheet* sheet() const { return m_pSheet; }
+    Style* style() const { return m_pStyle; }
+    Sheet* sheet() { return m_pSheet; }
+    const Sheet* sheet() const { return m_pSheet; }
 
-    virtual bool hasProperty( Properties p, bool withoutParent = false ) const;
+    bool hasProperty( Properties p, bool withoutParent = false ) const;
 
     /**
      * returns false if no currency information is set or
      * doesn't apply
      */
-    virtual bool currencyInfo( Currency & currency) const;
+    bool currencyInfo( Currency & currency) const;
 
     QString getCurrencySymbol() const;
     QFont font() const;
 
 protected:
-    virtual const QPen& rightBorderPen() const;
-    virtual const QPen& bottomBorderPen() const;
+    const QPen& rightBorderPen() const;
+    const QPen& bottomBorderPen() const;
 
     /**
-     * Default implementation does nothing.
+     * If a cell is set, marks its Layout and TextFormat as dirty.
      */
-    virtual void formatChanged();
+    void formatChanged();
 
     /**
-     * Default implementation returns 0.
+     * If a cell is set, it returns the row format of the sheet. Otherwise, it returns 0.
      */
-    virtual KSpreadFormat* fallbackFormat( int col, int row );
+    virtual Format* fallbackFormat( int col, int row );
     /**
-     * Default implementation returns 0.
+     * If a cell is set, it returns the row format of the sheet. Otherwise, it returns 0.
      */
-    virtual const KSpreadFormat* fallbackFormat( int col, int row ) const;
+    virtual const Format* fallbackFormat( int col, int row ) const;
 
     /**
-     * Default implementation returns TRUE.
+     * Default implementation returns true.
      */
     virtual bool isDefault() const;
 
-    KSpreadSheet * m_pSheet;
-    KSpreadStyle * m_pStyle;
+    Sheet * m_pSheet;
+    Style * m_pStyle;
 
     uint m_mask;
 
@@ -425,35 +447,37 @@ private:
     const QBrush & backGroundBrush() const;
     const QFont textFont() const;
     const QPen  & textPen() const;
+
+    Cell* m_pCell;
 };
 
 /**
  */
-class KSPREAD_EXPORT RowFormat : public KSpreadFormat
+class KSPREAD_EXPORT RowFormat : public Format
 {
 public:
-    RowFormat( KSpreadSheet * _sheet, int _row );
+    RowFormat( Sheet * _sheet, int _row );
     ~RowFormat();
 
-    virtual DCOPObject* dcopObject();
+    DCOPObject* dcopObject();
 
-    virtual QDomElement save( QDomDocument&, int yshift = 0, bool copy = false ) const;
-    virtual bool load( const QDomElement& row, int yshift = 0, PasteMode sp = Normal, bool paste = false );
-    virtual bool loadOasis( const QDomElement& row, QDomElement * rowStyle );
+    QDomElement save( QDomDocument&, int yshift = 0, bool copy = false ) const;
+    bool load( const QDomElement& row, int yshift = 0, Paste::Mode sp = Paste::Normal, bool paste = false );
+    bool loadOasis( const QDomElement& row, QDomElement * rowStyle );
 
     /**
      * @param _canvas is needed to get information about the zooming factor.
      *
      * @return the height in zoomed pixels as integer value.
      */
-    int height( const KSpreadCanvas *_canvas = 0L ) const;
+    int height( const Canvas *_canvas = 0L ) const;
     /**
      * @param _canvas is needed to get information about the zooming factor.
      *
      * @return the height in zoomed pixels as double value.
      * Use this function, if you want to work with height without having rounding problems.
      */
-    double dblHeight( const KSpreadCanvas *_canvas = 0L ) const;
+    double dblHeight( const Canvas *_canvas = 0L ) const;
     /**
      * @return the height in millimeters.
      */
@@ -464,7 +488,7 @@ public:
      * @param _h is calculated in display pixels as integer value. The function cares for zooming.
      * @param _canvas is needed to get information about the zooming factor.
      */
-    void setHeight( int _h, const KSpreadCanvas *_canvas = 0L );
+    void setHeight( int _h, const Canvas *_canvas = 0L );
     /**
      * Sets the height to _h zoomed pixels.
      *
@@ -472,7 +496,7 @@ public:
      * Use this function when setting the height, to not get rounding problems.
      * @param _canvas is needed to get information about the zooming factor.
      */
-    void setDblHeight( double _h, const KSpreadCanvas *_canvas = 0L );
+    void setDblHeight( double _h, const Canvas *_canvas = 0L );
     /**
      * Sets the height.
      *
@@ -483,11 +507,11 @@ public:
     /**
      * Use this function to tell this format that it is the default format.
      */
-    void setDefault() { m_bDefault = TRUE; }
+    void setDefault() { m_bDefault = true; }
     /**
      * @reimp
      */
-    bool isDefault() const;
+    virtual bool isDefault() const;
 
     /**
      * @return the row for this RowFormat. May be 0 if this is the default format.
@@ -526,18 +550,18 @@ public:
     /**
      * Sets the hide flag
      */
-    void setHide( bool _hide );
+    void setHide( bool _hide, bool repaint = true );
     bool isHide()const { return m_bHide;}
 
 protected:
     /**
      * @reimp
      */
-    KSpreadFormat* fallbackFormat( int col, int row );
+    virtual Format* fallbackFormat( int col, int row );
     /**
      * @reimp
      */
-    const KSpreadFormat* fallbackFormat( int col, int row ) const;
+    virtual const Format* fallbackFormat( int col, int row ) const;
 
     /**
      * Width of the cell in unzoomed points.
@@ -568,22 +592,22 @@ protected:
 
 /**
  */
-class KSPREAD_EXPORT ColumnFormat : public KSpreadFormat
+class KSPREAD_EXPORT ColumnFormat : public Format
 {
 public:
-    ColumnFormat( KSpreadSheet *_sheet, int _column );
+    ColumnFormat( Sheet *_sheet, int _column );
     ~ColumnFormat();
 
-    virtual QDomElement save( QDomDocument&, int xshift = 0, bool copy = false ) const;
-    virtual bool load( const QDomElement& row, int xshift = 0,PasteMode sp = Normal, bool paste = false );
-    virtual DCOPObject* dcopObject();
+    QDomElement save( QDomDocument&, int xshift = 0, bool copy = false ) const;
+    bool load( const QDomElement& row, int xshift = 0,Paste::Mode sp = Paste::Normal, bool paste = false );
+    DCOPObject* dcopObject();
 
     /**
      * @param _canvas is needed to get information about the zooming factor.
      *
      * @return the width in zoomed pixels as integer.
      */
-    int width( const KSpreadCanvas *_canvas = 0L ) const;
+    int width( const Canvas *_canvas = 0L ) const;
     /**
      * @param _canvas is needed to get information about the zooming factor.
      *
@@ -591,7 +615,7 @@ public:
      * Use this function, if you want to use the width and later restore it back,
      * so you don't get rounding problems
      */
-    double dblWidth( const KSpreadCanvas *_canvas = 0L ) const;
+    double dblWidth( const Canvas *_canvas = 0L ) const;
     /**
      * @return the width in millimeters.
      */
@@ -603,7 +627,7 @@ public:
      *           zooming.
      * @param _canvas is needed to get information about the zooming factor.
      */
-    void setWidth( int _w, const KSpreadCanvas *_canvas = 0L );
+    void setWidth( int _w, const Canvas *_canvas = 0L );
     /**
      * Sets the width to _w zoomed pixels as double value.
      * Use this function to set the width without getting rounding problems.
@@ -612,7 +636,7 @@ public:
      *           zooming.
      * @param _canvas is needed to get information about the zooming factor.
      */
-    void setDblWidth( double _w, const KSpreadCanvas *_canvas = 0L );
+    void setDblWidth( double _w, const Canvas *_canvas = 0L );
     /**
      * Sets the width.
      *
@@ -623,11 +647,11 @@ public:
     /**
      * Use this function to tell this format that it is the default format.
      */
-    void setDefault() { m_bDefault = TRUE; }
+    void setDefault() { m_bDefault = true; }
     /**
      * @reimp
      */
-    bool isDefault() const;
+    virtual bool isDefault() const;
 
     /**
      * @return the column of this ColumnFormat. May be 0 if this is the default format.
@@ -649,19 +673,19 @@ public:
     /**
      * @reimp
      */
-    const QPen& rightBorderPen( int col, int row ) const;
+    virtual const QPen& rightBorderPen( int col, int row ) const;
     /**
      * @reimp
      */
-    void setRightBorderPen( const QPen& p );
+    virtual void setRightBorderPen( const QPen& p );
     /**
      * @reimp
      */
-    const QPen& leftBorderPen( int col, int row ) const;
+    virtual const QPen& leftBorderPen( int col, int row ) const;
     /**
      * @reimp
      */
-    void setLeftBorderPen( const QPen& p );
+    virtual void setLeftBorderPen( const QPen& p );
 
     void setHide( bool _hide );
     bool isHide()const { return m_bHide;}
@@ -671,11 +695,11 @@ protected:
     /**
      * @reimp
      */
-    KSpreadFormat* fallbackFormat( int col, int row );
+    Format* fallbackFormat( int col, int row );
     /**
      * @reimp
      */
-    const KSpreadFormat* fallbackFormat( int col, int row ) const;
+    const Format* fallbackFormat( int col, int row ) const;
 
     /**
      * Width of the cells in unzoomed pixels.
@@ -706,22 +730,22 @@ protected:
     DCOPObject*m_dcop;
 };
 
-class KSPREAD_EXPORT KSpreadCurrency
+class KSPREAD_EXPORT Currency
 {
  public:
 
   enum currencyFormat { Gnumeric, OpenCalc, ApplixSpread,
                         GobeProductiveSpread, HancomSheet };
 
-  KSpreadCurrency();
-  ~KSpreadCurrency();
+  Currency();
+  ~Currency();
 
-  KSpreadCurrency(int index);
+  Currency(int index);
 
   /**
    * If code doesn't fit to index the index gets ignored
    */
-  KSpreadCurrency(int index, QString const & code);
+  Currency(int index, QString const & code);
 
   /**
    * code: e.g. EUR, USD,..
@@ -729,10 +753,10 @@ class KSPREAD_EXPORT KSpreadCurrency
    * currencyFormat: in Gnumeric the code is: [$EUR]
    *                 saves some work in the filter...
    */
-  KSpreadCurrency(QString const & code, currencyFormat format);
-  KSpreadCurrency & operator=(int type);
-  KSpreadCurrency & operator=(char const * code);
-  bool operator==(KSpreadCurrency const & cur) const;
+  Currency(QString const & code, currencyFormat format);
+  Currency & operator=(int type);
+  Currency & operator=(char const * code);
+  bool operator==(Currency const & cur) const;
   bool operator==(int type) const;
   operator int() const;
 
@@ -756,4 +780,6 @@ class KSPREAD_EXPORT KSpreadCurrency
   QString m_code;
 };
 
+} // namespace KSpread
+
 #endif
