@@ -8,6 +8,27 @@
 #include <qdatetime.h>
 #include <kdebug.h>
 
+
+#define KSBarIcon( x ) BarIcon( x, Factory::global() )
+#define colWidth 60.0
+#define heightOfRow 20.0
+
+/* Definition of maximal supported rows - please check kspread_util (columnName) and kspread_cluster also */
+#define KS_rowMax 0x7FFF
+/* Definition of maximal supported columns - please check kspread_util (columnName) and kspread_cluster also */
+#define KS_colMax 0x7FFF
+
+/* Currently used by StyleCluster for the Quad tree - question is, if we can make the spreadsheet this big in general?
+   We need this value to be the smallest number which is a power of 2 and more than both KS_rowMax and KS_colMax*/
+#define KS_Max_Quad 0x8000
+
+/* Definition of maximal supported columns/rows, which can be merged */
+#define KS_spanMax 0xFFF
+
+
+namespace KSpread
+{
+
 class ElapsedTime
 {
  public:
@@ -42,34 +63,76 @@ class ElapsedTime
   QString m_name;
 };
 
+/**
+ * This namespace collects enumerations related to
+ * pasting operations.
+ */
+namespace Paste
+{
+  /**
+   * The pasted content
+   */
+  enum Mode
+  {
+    Normal /** Everything */,
+    Text /** Text only */,
+    Format /** Format only */,
+    NoBorder /** not the borders */,
+    Comment /** Comment only */,
+    Result /** Result only, no formula */,
+    NormalAndTranspose /** */,
+    TextAndTranspose /** */,
+    FormatAndTranspose /** */,
+    NoBorderAndTranspose /** */
+  };
+  /**
+   * The current cell value treatment.
+   */
+  enum Operation
+  {
+    OverWrite /** Overwrite */,
+    Add /** Add */,
+    Mul /** Multiply */,
+    Sub /** Substract */,
+    Div /** Divide */
+  };
+} // namespace Paste
 
-#define KSBarIcon( x ) BarIcon( x, KSpreadFactory::global() )
-#define colWidth 60.0
-#define heightOfRow 20.0
+/**
+ * This namespace collects enumerations related to
+ * cell content restrictions.
+ */
+namespace Restriction
+{
+  /**
+   * The type of the restriction
+   */
+  enum Type
+  {
+    None /** No restriction */,
+    Number /** Restrict to numbers */,
+    Text /** Restrict to texts */,
+    Time /** Restrict to times */,
+    Date /** Restrict to dates */,
+    Integer /** Restrict to integers  */,
+    TextLength /** Restrict text length */,
+    List /** Restrict to lists */
+  };
+} // namespace Restriction
 
-/* Definition of maximal supported rows - please check kspread_util (columnName) and kspread_cluster also */
-#define KS_rowMax 0x7FFF
-/* Definition of maximal supported columns - please check kspread_util (columnName) and kspread_cluster also */
-#define KS_colMax 0x7FFF
-
-/* Currently used by StyleCluster for the Quad tree - question is, if we can make the spreadsheet this big in general?
-   We need this value to be the smallest number which is a power of 2 and more than both KS_rowMax and KS_colMax*/
-#define KS_Max_Quad 0x8000
-
-/* Definition of maximal supported columns/rows, which can be merged */
-#define KS_spanMax 0xFFF
-
-enum PasteMode { Normal, Text, Format, NoBorder, Comment, Result, NormalAndTranspose, TextAndTranspoase,
-		 FormatAndTranspose, NoBorderAndTranspose };
-enum Operation { OverWrite, Add, Mul, Sub, Div };
-enum Conditional { None, Equal,Superior,Inferior,SuperiorEqual,InferiorEqual,Between,Different, DifferentTo};
-enum Action { Stop, Warning, Information};
-enum Allow { Allow_All, Allow_Number, Allow_Text,Allow_Time,Allow_Date,Allow_Integer,Allow_TextLength, Allow_List};
-enum Series { Column,Row,Linear,Geometric };
-namespace KSpread
-{  // necessary due to QDock* enums (Werner)
-  enum MoveTo { Bottom, Left, Top, Right, BottomFirst };
+namespace Action
+{
+  enum Type
+  {
+    Stop,
+    Warning,
+    Information
+  };
 }
+
+enum Series { Column,Row,Linear,Geometric };
+// necessary due to QDock* enums (Werner)
+enum MoveTo { Bottom, Left, Top, Right, BottomFirst };
 enum MethodOfCalc { SumOfNumber, Min, Max, Average, Count, NoneCalc, CountA };
 
 enum FormatType {
@@ -90,7 +153,57 @@ enum FormatType {
     date_format17=216,date_format18=217,date_format19=218,date_format20=219,
     date_format21=220,date_format22=221,date_format23=222,date_format24=223,
     date_format25=224,date_format26=225,
-    Custom_format = 300
+    Custom_format = 300, No_format = 400
 };
 
+enum ModifyType {
+  MT_NONE = 0,
+  MT_MOVE,
+  MT_RESIZE_UP,
+  MT_RESIZE_DN,
+  MT_RESIZE_LF,
+  MT_RESIZE_RT,
+  MT_RESIZE_LU,
+  MT_RESIZE_LD,
+  MT_RESIZE_RU,
+  MT_RESIZE_RD
+};
+
+enum PropValue {
+    STATE_ON = 0,
+    STATE_OFF = 1,
+    STATE_UNDEF = 2
+};
+
+//TODO Implement all objects :)
+enum ToolEditMode {
+    TEM_MOUSE = 0,
+    //INS_RECT = 1,
+    //INS_ELLIPSE = 2,
+    //INS_TEXT = 3,
+    //INS_PIE = 4,
+    INS_OBJECT = 5,
+    //INS_LINE = 6,
+    INS_DIAGRAMM = 7,
+    //INS_TABLE = 8,
+    //INS_FORMULA = 9,
+    //INS_AUTOFORM = 10,
+    //INS_FREEHAND = 11,
+    //INS_POLYLINE = 12,
+    //INS_QUADRICBEZIERCURVE = 13,
+    //INS_CUBICBEZIERCURVE = 14,
+    //INS_POLYGON = 15,
+    INS_PICTURE = 16
+    //INS_CLIPART = 17,
+    //TEM_ROTATE = 18,
+    //TEM_ZOOM = 19,
+    //INS_CLOSED_FREEHAND = 20,
+    //INS_CLOSED_POLYLINE = 21,
+    //INS_CLOSED_QUADRICBEZIERCURVE = 22,
+    //INS_CLOSED_CUBICBEZIERCURVE = 23
+};
+
+
+} // namespace KSpread
+
 #endif
