@@ -14,28 +14,37 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef __kspread_sheetprint_h__
 #define __kspread_sheetprint_h__
 
-class KSpreadDoc;
-class KSpreadPrintNewPageEntry;
-
 #include <qobject.h>
 
-#include "kspread_sheet.h"
+#include <koffice_export.h>
+#include <KoUnit.h>
 
-class KSpreadSheetPrint : public QObject
+class KoGenStyles;
+
+namespace KSpread
+{
+class Doc;
+class Selection;
+class PrintNewPageEntry;
+class PrintObject;
+
+class KSPREAD_EXPORT SheetPrint : public QObject
 {
     Q_OBJECT
 
 public:
 
-    KSpreadSheetPrint( KSpreadSheet *sheet );
-    ~KSpreadSheetPrint();
+    SheetPrint( Sheet *sheet );
+    ~SheetPrint();
+
+    QString saveOasisSheetStyleLayout( KoGenStyles &mainStyles );
 
     /**
      * @return false if nothing to print.
@@ -43,24 +52,24 @@ public:
     bool print( QPainter &painter, KPrinter *_printer );
 
     /**
-     * @return the printable width of the paper in millimeters.
+     * @return the prinsheet width of the paper in millimeters.
      */
-    float printableWidth()const { return m_paperWidth - m_leftBorder - m_rightBorder; }
+    float prinsheetWidth()const { return m_paperWidth - m_leftBorder - m_rightBorder; }
 
     /**
-     * @return the printable width of the paper in zoomed points.
+     * @return the prinsheet width of the paper in zoomed points.
      */
-    float printableWidthPts()const { return MM_TO_POINT( printableWidth() / m_dZoom ); }
+    float prinsheetWidthPts()const { return MM_TO_POINT( prinsheetWidth() / m_dZoom ); }
 
     /**
-     * @return the printable height of the paper in millimeters.
+     * @return the prinsheet height of the paper in millimeters.
      */
-    float printableHeight()const { return m_paperHeight - m_topBorder - m_bottomBorder; }
+    float prinsheetHeight()const { return m_paperHeight - m_topBorder - m_bottomBorder; }
 
     /**
-     * @return the printable height of the paper in zoomed points.
+     * @return the prinsheet height of the paper in zoomed points.
      */
-    float printableHeightPts()const { return MM_TO_POINT( printableHeight() / m_dZoom ); }
+    float prinsheetHeightPts()const { return MM_TO_POINT( prinsheetHeight() / m_dZoom ); }
 
     /**
      * @return the height of the paper in millimeters.
@@ -143,7 +152,7 @@ public:
     KoPageLayout paperLayout() const;
 
      /**
-     * Changes the paper layout and repaints the currently displayed KSpreadSheet.
+     * Changes the paper layout and repaints the currently displayed Sheet.
      */
     void setPaperLayout( float _leftBorder, float _topBorder, float _rightBorder, float _bottomBoder,
                          KoFormat _paper, KoOrientation orientation );
@@ -182,19 +191,19 @@ public:
      * Sets the print range.
      * Set it to ( QPoint (1, 1), QPoint(KS_colMax, KS_rowMax) ) to undefine it
      */
-    void setPrintRange( QRect _printRange );
+    void setPrintRange( const QRect &_printRange );
 
     /**
      * Return the page limit in X direction.
      * 0 means no limit
      */
-    int pageLimitX() { return m_iPageLimitX; }
+    int pageLimitX() const { return m_iPageLimitX; }
 
     /**
      * Return the page limit in Y direction.
      * 0 means no limit
      */
-    int pageLimitY() { return m_iPageLimitY; }
+    int pageLimitY() const { return m_iPageLimitY; }
 
     /**
      * Sets the page limit in X direction. The zoom factor will be adjusted,
@@ -315,6 +324,36 @@ public:
      */
     void setPrintGrid( bool _printGrid );
 
+    /**
+     * Returns, if the objects shall be shown on printouts
+     */
+    bool printObjects() const { return m_bPrintObjects; }
+
+    /**
+     * Sets, if the objects shall be shown on printouts
+     */
+    void setPrintObjects( bool _printObjects );
+
+    /**
+     * Returns, if the charts shall be shown on printouts
+     */
+    bool printCharts() const { return m_bPrintCharts; }
+
+    /**
+     * Sets, if the charts shall be shown on printouts
+     */
+    void setPrintCharts( bool _printCharts );
+
+    /**
+     * Returns, if the graphics shall be shown on printouts
+     */
+    bool printGraphics() const { return m_bPrintGraphics; }
+
+    /**
+     * Sets, if the graphics shall be shown on printouts
+     */
+    void setPrintGraphics( bool _printGraphics );
+
     /**
      * Returns, if the comment rect shall be shown on printouts
      */
@@ -348,7 +387,7 @@ public:
     /**
      * Define the print range with the current selection
      */
-    void definePrintRange(KSpreadSelection* selectionInfo);
+    void definePrintRange(Selection* selectionInfo);
     /**
      * Reset the print range to the standard definition (whole sheet)
      */
@@ -381,7 +420,7 @@ public:
     /**
      * Returns the zoom level of the printout as double
      */
-    double zoom(){ return m_dZoom; }
+    double zoom() const { return m_dZoom; }
 
     /**
      * Checks wether the page has content to print
@@ -389,12 +428,12 @@ public:
     bool pageNeedsPrinting( QRect& page_range );
 
 signals:
-    void sig_updateView( KSpreadSheet *_table );
+    void sig_updateView( Sheet *_sheet );
 
 private:
 
-    KSpreadSheet * m_pSheet;
-    KSpreadDoc * m_pDoc;
+    Sheet * m_pSheet;
+    Doc * m_pDoc;
 
     /**
      * Prints the page specified by 'page_range'.
@@ -402,12 +441,14 @@ private:
      *
      * @return the last vertical line which was printed plus one.
      *
-     * @param _page_range QRect defines a rectangle of cells which should be painted
-     *                    to the device 'prn'.
-     * @view KoRect defines the sourrounding rectangle which is the printing frame.
+     * @param _page_range QRect defines a rectangle of cells which should be
+     *                    painted to the device 'prn'.
      *
-     * @param _childOffset KoPoint used to calculate the correct position of children,
-     *                    if there are repeated columns/rows
+     * @param view KoRect defines the sourrounding rectangle which is
+     *             the printing frame.
+     *
+     * @param _childOffset KoPoint used to calculate the correct position of
+     *                     children, if there are repeated columns/rows.
      */
     void printPage( QPainter &_painter, const QRect& page_range,
                     const KoRect& view, const KoPoint _childOffset );
@@ -432,21 +473,21 @@ private:
     /**
      * Returns the iterator for the column in the newPage list for columns
      */
-    QValueList<KSpreadPrintNewPageEntry>::iterator findNewPageColumn( int col );
+    QValueList<PrintNewPageEntry>::iterator findNewPageColumn( int col );
 
     /**
      * Returns the iterator for the row in the newPage list for rows
      */
-    QValueList<KSpreadPrintNewPageEntry>::iterator findNewPageRow( int row );
+    QValueList<PrintNewPageEntry>::iterator findNewPageRow( int row );
 
     /**
      * Replaces macros like <name>, <file>, <date> etc. in the string and
      * returns the modified one.
      *
      * @param _page is the page number for which the heading is produced.
-     * @param _KSpreadSheet is the name of the KSpreadSheet for which we generate the headings.
+     * @param _Sheet is the name of the Sheet for which we generate the headings.
      */
-    QString completeHeading( const QString &_data, int _page, const QString &_table ) const ;
+    QString completeHeading( const QString &_data, int _page, const QString &_sheet ) const ;
 
     /**
      * Returns a rect, which contains the cols and rows to be printed.
@@ -457,12 +498,12 @@ private:
     /**
      * Returns the numbers of pages in x direction
      */
-    int pagesX( QRect& cellsPrintRange );
+    int pagesX( const QRect& cellsPrintRange );
 
     /**
      * Returns the numbers of pages in y direction
      */
-    int pagesY( QRect& cellsPrintRange );
+    int pagesY( const QRect& cellsPrintRange );
 
     /**
      * The orientation of the paper.
@@ -536,7 +577,7 @@ private:
     uint m_uprintPages;
 
     /**
-     * Defined printable area
+     * Defined prinsheet area
      */
     QRect m_printRange;
 
@@ -555,6 +596,21 @@ private:
      */
     bool m_bPrintGrid;
 
+    /**
+     * Show the objects when making printout
+     */
+    bool m_bPrintObjects;
+
+    /**
+     * Show the charts when making printout
+     */
+    bool m_bPrintCharts;
+
+    /**
+     * Show the graphics when making printout
+     */
+    bool m_bPrintGraphics;
+
     /**
      * Show the formula rect when making printout
      */
@@ -577,12 +633,12 @@ private:
     /**
      * Stores the new page columns
      */
-     QValueList<KSpreadPrintNewPageEntry> m_lnewPageListX;
+     QValueList<PrintNewPageEntry> m_lnewPageListX;
 
     /**
      * Stores the new page columns
      */
-     QValueList<KSpreadPrintNewPageEntry> m_lnewPageListY;
+     QValueList<PrintNewPageEntry> m_lnewPageListY;
 
     /**
      * Stores internally the maximum column that was checked already
@@ -608,34 +664,36 @@ private:
      * Limit of pages in Y direction. 0 means no limit
      */
     int m_iPageLimitY;
+
+    QValueList<PrintObject*> m_printObjects;
 };
 
 
-class KSpreadPrintNewPageEntry
+class PrintNewPageEntry
 {
 public:
-    KSpreadPrintNewPageEntry() :
+    PrintNewPageEntry() :
         m_iStartItem( 0 ), m_iEndItem( 0 ), m_dSize( 0 ),
         m_dOffset( 0 ){}
 
-    KSpreadPrintNewPageEntry( int startItem, int endItem = 0, double size = 0,
+    PrintNewPageEntry( int startItem, int endItem = 0, double size = 0,
                               double offset = 0 ) :
         m_iStartItem( startItem ), m_iEndItem( endItem ), m_dSize( size ),
         m_dOffset( offset ) {}
 
-    int startItem() { return m_iStartItem; } const
+    int startItem() const { return m_iStartItem; }
     void setStartItem( int startItem ) { m_iStartItem = startItem; }
 
-    int endItem() { return m_iEndItem; } const
+    int endItem() const { return m_iEndItem; }
     void setEndItem( int endItem ) { m_iEndItem = endItem; }
 
-    double size() { return m_dSize; } const
+    double size() const { return m_dSize; }
     void setSize( double size ) { m_dSize = size; }
 
-    double offset() { return m_dOffset; }
+    double offset() const { return m_dOffset; }
     void setOffset( double offset ) { m_dOffset = offset; }
 
-    bool operator==( KSpreadPrintNewPageEntry const & entry ) const;
+    bool operator==( PrintNewPageEntry const & entry ) const;
 
 
 private:
@@ -645,5 +703,16 @@ private:
     double m_dOffset;
 };
 
+class PrintObject
+{
+  public:
+    PrintObject() : obj(0), p(0) {}
+
+    EmbeddedObject *obj;
+    QPixmap *p;
+};
+
+} // namespace KSpread
+
 #endif
 
