@@ -13,36 +13,83 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include "kspread_value.h"
 
+#include <kdebug.h>
+
+#include <qstring.h>
+#include <qtextstream.h>
+
 #include <float.h>
 #include <math.h>
 #include <limits.h>
 
-#include <qstring.h>
-#include <qtextstream.h>
+using namespace KSpread;
 
+// helper struct for array implementation
+// this struct holds one piece of an array of size CHUNK_COLS x CHUNK_ROWS
+// or less
+struct arrayChunk {
+  arrayChunk (int c, int r) {
+    cols = c; rows = r;
+    ptr = new Value* [c*r];
+    for (int i = 0; i < c*r; ++i) ptr[i] = 0;
+  }
+  
+  ~arrayChunk () {
+    if (!ptr) return;
+    unsigned count = cols * rows;
+    for (unsigned i = 0; i < count; i++)
+      delete ptr[i];
+    delete [] ptr;
+  }
 
+  arrayChunk( const arrayChunk& ac )
+  {
+    operator=( ac );
+  }
+  
+  arrayChunk& operator= ( const arrayChunk& ac )
+  {
+    cols = ac.cols; rows = ac.rows;
+    ptr = new Value* [cols*rows];
+    unsigned count = cols * rows;
+    for( unsigned i = 0; i < count; i++ )
+      if( ac.ptr[i] )
+        ptr[i] = new Value( *ac.ptr[i] );
+      else
+        ptr[i] = 0;
+    return *this;
+  }
+  
+  Value **ptr;
+  unsigned cols, rows;
+};
+
+#define CHUNK_COLS 128
+#define CHUNK_ROWS 128
 // helper class for array implementation
 class ValueArray
 {
 public:
-  KSpreadValue** ptr;
+  arrayChunk **chunks;
   unsigned columns;
   unsigned rows;
+  unsigned chunkCols, chunkRows;
   
-  ValueArray(): ptr(0), columns(0), rows(0) {};
+  ValueArray(): chunks(0), columns(0), rows(0), chunkCols(0), chunkRows(0) {};
   
   ~ValueArray()
   { 
     clear();
-   };  
+  };
    
   ValueArray( const ValueArray& va )
+    : chunks(0), columns(0), rows(0), chunkCols(0), chunkRows(0)
   {
     operator=( va );
   }
@@ -50,65 +97,112 @@ public:
   ValueArray& operator= ( const ValueArray& va )
   {
     init( va.columns, va.rows );
-    unsigned count = columns * rows;
+    unsigned count = chunkCols * chunkRows;
     for( unsigned i = 0; i < count; i++ )
-      if( va.ptr[i] )
-        ptr[i] = new KSpreadValue( *va.ptr[i] );
+      if( va.chunks[i] )
+        chunks[i] = new arrayChunk (*va.chunks[i]);
+      else
+        chunks[i] = 0;
     return *this;
   }
    
   void clear()
   {
-    if( !ptr ) return;
-    unsigned count = columns * rows;
+    int c = columns / CHUNK_COLS;
+    int r = rows / CHUNK_ROWS;
+    if (columns % CHUNK_COLS != 0) c++;
+    if (rows % CHUNK_ROWS != 0) r++;
+    if( !chunks ) return;
+    unsigned count = c*r;
     if( !count ) return;
-    for( unsigned i = 0; i < count; i++ )
-      delete ptr[i];
-    delete [] ptr;
-    ptr = 0;
+    for (unsigned i = 0; i < count; i++)
+      delete chunks[i];
+    delete [] chunks;
+    chunks = 0;
+    columns = rows = chunkCols = chunkRows = 0;
   }
   
   void init( unsigned c, unsigned r )
   {
-    if( ptr ) clear();
+    if (chunks) clear();
     columns = c; rows = r;
-    unsigned count = columns * rows;
-    ptr = new KSpreadValue* [count];
+    int cc = columns / CHUNK_COLS;
+    int rr = rows / CHUNK_ROWS;
+    if (columns % CHUNK_COLS != 0) cc++;
+    if (rows % CHUNK_ROWS != 0) rr++;
+    chunkCols = cc;
+    chunkRows = rr;
+    unsigned count = cc*rr;
+    chunks = new arrayChunk* [count];
     for( unsigned i = 0; i < count; i++ )
-      ptr[i] = (KSpreadValue*)0;
+      chunks[i] = 0;
   }
   
-  KSpreadValue* at( unsigned c, unsigned r )
+  Value* at( unsigned c, unsigned r ) const
   {
-    if( !ptr ) return 0;
+    if( !chunks ) return 0;
     if( c >= columns ) return 0;
     if( r >= rows ) return 0;
-    return ptr[r*columns+c];
+    
+    int col = c / CHUNK_COLS;
+    int row = r / CHUNK_ROWS;
+    int cpos = c % CHUNK_COLS;
+    int rpos = r % CHUNK_ROWS;
+    arrayChunk *chunk = chunks[row * chunkCols + col];
+    if (!chunk) return 0;
+    return chunk->ptr[rpos * chunk->cols + cpos];
   };
   
-  void set( unsigned c, unsigned r, KSpreadValue* v )
+  void set( unsigned c, unsigned r, Value* v )
   {
-    if( !ptr ) return;
+    if (!chunks) return;
     if( c >= columns ) return;
     if( r >= rows ) return;
-    delete ptr[r*columns+c];
-    ptr[r*columns+c] = v;
+    unsigned col = c / CHUNK_COLS;
+    unsigned row = r / CHUNK_ROWS;
+    unsigned cpos = c % CHUNK_COLS;
+    unsigned rpos = r % CHUNK_ROWS;
+    arrayChunk *chunk = chunks[row * chunkCols + col];
+    if (!chunk) {
+      unsigned cc = (col==chunkCols-1) ? (columns % CHUNK_COLS) : CHUNK_COLS;
+      unsigned rr = (row==chunkRows-1) ? (rows % CHUNK_ROWS) : CHUNK_ROWS;
+      chunk = new arrayChunk (cc, rr);
+      chunks[row * chunkCols + col] = chunk;
+    }
+    delete chunk->ptr[rpos * chunk->cols + cpos];
+    chunk->ptr[rpos * chunk->cols + cpos] = v;
   }
-  
+
+  bool operator==( const ValueArray& other ) const
+  {
+    if ( columns != other.columns || rows != other.rows )
+      return false;
+    for ( unsigned r = 0; r < rows; ++r )
+      for ( unsigned c = 0; c < columns; ++c ) {
+        Value* v1 = at( c, r );
+        Value* v2 = other.at( c, r );
+        if ( ( v1 && !v2 ) || ( !v1 && v2 ) )
+            return false;
+        if ( !( v1 && v2 && *v1 == *v2 ) )
+            return false;
+    }
+    return true;
+  }
+
 };
 
 
-// helper class for KSpreadValue
-class KSpreadValueData
+// helper class for Value
+class KSpread::ValueData
 {
   public:
 
-    KSpreadValue::Type type:4;
-    KSpreadValue::Format format:4;
+    Value::Type type:4;
+    Value::Format format:4;
 
     // reference count, at least one when object exists
     unsigned int count:24;
-    
+
     union
     {
       bool b;
@@ -117,22 +211,22 @@ class KSpreadValueData
       QString* ps;
       ValueArray* pa;
     };
-    
+
     // create empty data
-    KSpreadValueData(): type( KSpreadValue::Empty ),
-      format (KSpreadValue::fmt_None), count( 1 ), ps( 0 ) { };
+    ValueData(): type( Value::Empty ),
+      format (Value::fmt_None), count( 1 ), ps( 0 ) { };
 
     // destroys data
-    ~KSpreadValueData(){ if( this == s_null ) s_null = 0;
-       if( type == KSpreadValue::Array ) delete pa;
-       if( type == KSpreadValue::String ) delete ps;
-       if( type == KSpreadValue::Error ) delete ps;
+    ~ValueData(){ if( this == s_null ) s_null = 0;
+       if( type == Value::Array ) delete pa;
+       if( type == Value::String ) delete ps;
+       if( type == Value::Error ) delete ps;
      }
 
     // static empty data to be shared
-    static KSpreadValueData* null()
-      { if( !s_null) s_null = new KSpreadValueData; else s_null->ref(); return s_null; }
-      
+    static ValueData* null()
+      { if( !s_null) s_null = new ValueData; else s_null->ref(); return s_null; }
+
     // increase reference count
     void ref() { count++; }
 
@@ -145,148 +239,176 @@ class KSpreadValueData
 
     /** set most probable formatting based on the type */
     void setFormatByType ();
-    
+
   private:
 
-    static KSpreadValueData* s_null;
+    static ValueData* s_null;
 };
 
-void KSpreadValueData::setFormatByType ()
+void KSpread::ValueData::setFormatByType ()
 {
   switch (type) {
-    case KSpreadValue::Empty:
-      format = KSpreadValue::fmt_None;
+    case Value::Empty:
+      format = Value::fmt_None;
     break;
-    case KSpreadValue::Boolean:
-      format = KSpreadValue::fmt_Boolean;
+    case Value::Boolean:
+      format = Value::fmt_Boolean;
     break;
-    case KSpreadValue::Integer:
-      format = KSpreadValue::fmt_Number;
+    case Value::Integer:
+      format = Value::fmt_Number;
     break;
-    case KSpreadValue::Float:
-      format = KSpreadValue::fmt_Number;
+    case Value::Float:
+      format = Value::fmt_Number;
     break;
-    case KSpreadValue::String:
-      format = KSpreadValue::fmt_String;
+    case Value::String:
+      format = Value::fmt_String;
     break;
-    case KSpreadValue::Array:
-      format = KSpreadValue::fmt_None;
+    case Value::Array:
+      format = Value::fmt_None;
     break;
-    case KSpreadValue::CellRange:
-      format = KSpreadValue::fmt_None;
+    case Value::CellRange:
+      format = Value::fmt_None;
     break;
-    case KSpreadValue::Error:
-      format = KSpreadValue::fmt_String;
+    case Value::Error:
+      format = Value::fmt_String;
     break;
   };
 }
 
 // to be shared between all empty value
-KSpreadValueData* KSpreadValueData::s_null = 0;
+ValueData* ValueData::s_null = 0;
 
 // static things
-KSpreadValue ks_value_empty;
-KSpreadValue ks_error_div0;
-KSpreadValue ks_error_na;
-KSpreadValue ks_error_name;
-KSpreadValue ks_error_null;
-KSpreadValue ks_error_num;
-KSpreadValue ks_error_ref;
-KSpreadValue ks_error_value;
+Value ks_value_empty;
+Value ks_error_div0;
+Value ks_error_na;
+Value ks_error_name;
+Value ks_error_null;
+Value ks_error_num;
+Value ks_error_ref;
+Value ks_error_value;
 
 // create an empty value
-KSpreadValue::KSpreadValue()
+Value::Value()
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
 }
 
 // destructor
-KSpreadValue::~KSpreadValue()
+Value::~Value()
 {
   d->unref();
 }
 
 // create value of certain type
-KSpreadValue::KSpreadValue( KSpreadValue::Type _type )
+Value::Value( Value::Type _type )
 {
-  d = new KSpreadValueData;
+  d = new ValueData;
   d->type = _type;
   d->setFormatByType ();
 }
 
 // copy constructor
-KSpreadValue::KSpreadValue( const KSpreadValue& _value )
+Value::Value( const Value& _value )
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
   assign( _value );
 }
 
 // assignment operator
-KSpreadValue& KSpreadValue::operator=( const KSpreadValue& _value )
+Value& Value::operator=( const Value& _value )
 {
   return assign( _value );
 }
 
-// create a boolean value
-KSpreadValue::KSpreadValue( bool b )
+// comparison operator - returns true only if strictly identical, unlike equal()/compare()
+bool Value::operator==( const Value& v ) const
 {
-  d = KSpreadValueData::null();
+  const ValueData* n = v.d;
+  if ( d->type != n->type )
+    return false;
+  switch( d->type )
+  {
+    case Empty: return true;
+    case Boolean: return n->b == d->b;
+    case Integer: return n->i == d->i;
+    case Float:   return compare( n->f, d->f ) == 0;
+    case String:  return *n->ps == *d->ps;
+    case Array:   return *n->pa == *d->pa;
+    case Error:   return *n->ps == *d->ps;
+    default: break;
+  }
+  kdWarning() << "Unhandled type in Value::operator==: " << d->type << endl;
+  return false;
+}
+
+// create a boolean value
+Value::Value( bool b )
+{
+  d = ValueData::null();
   setValue( b );
 }
 
 // create an integer value
-KSpreadValue::KSpreadValue( long i )
+Value::Value( long i )
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
   setValue ( i );
 }
 
 // create an integer value
-KSpreadValue::KSpreadValue( int i )
+Value::Value( int i )
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
   setValue ( i );
 }
 
 // create a floating-point value
-KSpreadValue::KSpreadValue( double f )
+Value::Value( double f )
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
   setValue( f );
 }
 
 // create a string value
-KSpreadValue::KSpreadValue( const QString& s )
+Value::Value( const QString& s )
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
   setValue( s );
 }
 
-// create a floating-point value from date/time
-KSpreadValue::KSpreadValue( const QDateTime& dt )
+// create a string value
+Value::Value (const char *s)
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
+  setValue (QString (s));
+}
+
+// create a floating-point value from date/time
+Value::Value( const QDateTime& dt )
+{
+  d = ValueData::null();
   setValue( dt );
 }
 
 // create a floating-point value from time
-KSpreadValue::KSpreadValue( const QTime& dt )
+Value::Value( const QTime& dt )
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
   setValue( dt );
 }
 
 // create a floating-point value from date
-KSpreadValue::KSpreadValue( const QDate& dt )
+Value::Value( const QDate& dt )
 {
-  d = KSpreadValueData::null();
+  d = ValueData::null();
   setValue( dt );
 }
 
 // create an array value
-KSpreadValue::KSpreadValue( unsigned columns, unsigned rows )
+Value::Value( unsigned columns, unsigned rows )
 {
-  d = new KSpreadValueData;
+  d = new ValueData;
   d->type = Array;
   d->format = fmt_None;
   d->pa = new ValueArray;
@@ -295,7 +417,7 @@ KSpreadValue::KSpreadValue( unsigned columns, unsigned rows )
 
 // assign value from other
 // shallow copy: only copy the data pointer
-KSpreadValue& KSpreadValue::assign( const KSpreadValue& _value )
+Value& Value::assign( const Value& _value )
 {
   d->unref();
   d = _value.d;
@@ -304,13 +426,13 @@ KSpreadValue& KSpreadValue::assign( const KSpreadValue& _value )
 }
 
 // return type of the value
-KSpreadValue::Type KSpreadValue::type() const
+Value::Type Value::type() const
 {
   return d ? d->type : Empty;
 }
 
 // set the value to boolean
-void KSpreadValue::setValue( bool b )
+void Value::setValue( bool b )
 {
   detach();
   d->type = Boolean;
@@ -319,18 +441,18 @@ void KSpreadValue::setValue( bool b )
 }
 
 // get the value as boolean
-bool KSpreadValue::asBoolean() const
+bool Value::asBoolean() const
 {
   bool result = false;
 
-  if( type() == KSpreadValue::Boolean )
+  if( type() == Value::Boolean )
     result = d->b;
 
   return result;
 }
 
 // set the value to integer
-void KSpreadValue::setValue( long i )
+void Value::setValue( long i )
 {
   detach();
   d->type = Integer;
@@ -339,7 +461,7 @@ void KSpreadValue::setValue( long i )
 }
 
 // set the value to integer
-void KSpreadValue::setValue( int i )
+void Value::setValue( int i )
 {
   detach();
   d->type = Integer;
@@ -348,26 +470,26 @@ void KSpreadValue::setValue( int i )
 }
 
 // get the value as integer
-long KSpreadValue::asInteger() const
+long Value::asInteger() const
 {
   long result = 0;
 
-  if( type() == KSpreadValue::Integer )
+  if( type() == Value::Integer )
     result = d->i;
 
-  if( type() == KSpreadValue::Float )
+  if( type() == Value::Float )
     result = static_cast<int>(d->f);
 
   return result;
 }
 
-void KSpreadValue::setValue( const KSpreadValue& v )
+void Value::setValue( const Value& v )
 {
   assign( v );
 }
 
 // set the value as floating-point
-void KSpreadValue::setValue( double f )
+void Value::setValue( double f )
 {
   detach();
   d->type = Float;
@@ -376,21 +498,21 @@ void KSpreadValue::setValue( double f )
 }
 
 // get the value as floating-point
-double KSpreadValue::asFloat() const
+double Value::asFloat() const
 {
   double result = 0.0;
 
-  if( type() == KSpreadValue::Float )
+  if( type() == Value::Float )
     result = d->f;
 
-  if( type() == KSpreadValue::Integer )
+  if( type() == Value::Integer )
     result = static_cast<double>(d->i);
 
   return result;
 }
 
 // set the value as string
-void KSpreadValue::setValue( const QString& s )
+void Value::setValue( const QString& s )
 {
   detach();
   d->type = String;
@@ -399,11 +521,11 @@ void KSpreadValue::setValue( const QString& s )
 }
 
 // get the value as string
-QString KSpreadValue::asString() const
+QString Value::asString() const
 {
   QString result;
 
-  if( type() == KSpreadValue::String )
+  if( type() == Value::String )
   if( d->ps )
     result = QString( *d->ps );
 
@@ -411,7 +533,7 @@ QString KSpreadValue::asString() const
 }
 
 // set error message
-void KSpreadValue::setError( const QString& msg )
+void Value::setError( const QString& msg )
 {
   detach();
   d->type = Error;
@@ -419,11 +541,11 @@ void KSpreadValue::setError( const QString& msg )
 }
 
 // get error message
-QString KSpreadValue::errorMessage() const
+QString Value::errorMessage() const
 {
   QString result;
 
-  if( type() == KSpreadValue::Error )
+  if( type() == Value::Error )
   if( d->ps )
     result = QString( *d->ps );
 
@@ -433,115 +555,115 @@ QString KSpreadValue::errorMessage() const
 // set the value as date/time
 // NOTE: date/time is stored as serial number
 // Day 61 means 1st of March, 1900
-void KSpreadValue::setValue( const QDateTime& dt )
+void Value::setValue( const QDateTime& dt )
 {
   // reference is 31 Dec, 1899 midnight
   QDate refDate( 1899, 12, 31 );
   QTime refTime( 0, 0 );
 
-  double f = refDate.daysTo( dt.date() ) + 1.0;
-  f += refTime.secsTo( dt.time() ) / 86400.0;
+  int i = refDate.daysTo( dt.date() ) + 1;
+  i += refTime.secsTo( dt.time() ) / 86400;
 
-  setValue( f );
+  setValue( i );
   d->format = fmt_DateTime;
 }
 
-void KSpreadValue::setValue( const QTime& time )
+void Value::setValue( const QTime& time )
 {
   // reference time is midnight
   QTime refTime( 0, 0 );
-  double f = refTime.msecsTo( time ) / 86400000.0;
+  int i = refTime.msecsTo( time ) /* / 86400000 */;
 
-  setValue( f );
+  setValue( i );
   d->format = fmt_Time;
 }
 
-void KSpreadValue::setValue( const QDate& date )
+void Value::setValue( const QDate& date )
 {
   // reference date is 31 Dec, 1899
   QDate refDate = QDate( 1899, 12, 31 );
-  double f = refDate.daysTo( date ) + 1.0;
+  int i = refDate.daysTo( date ) + 1;
 
-  setValue( f );
+  setValue( i );
   d->format = fmt_Date;
 }
 
 // get the value as date/time
-QDateTime KSpreadValue::asDateTime() const
+QDateTime Value::asDateTime() const
 {
   return QDateTime( asDate(), asTime() );
 }
 
 // get the value as date
-QDate KSpreadValue::asDate() const
+QDate Value::asDate() const
 {
   QDate dt( 1899, 12, 30 );
 
-  double f = asFloat();
-  dt = dt.addDays( (int) f );
+  int i = asInteger();
+  dt = dt.addDays( i );
   
   return dt;
 }
 
 // get the value as time
-QTime KSpreadValue::asTime() const
+QTime Value::asTime() const
 {
   QTime dt;
   
-  double f = asFloat();
-  dt = dt.addMSecs( qRound( (f-(int)f) * 86400 * 1000 ) );
+  int i = asInteger();
+  dt = dt.addMSecs(i) /*( f * 86400 * 1000  )*/;
   
   return dt;
 }
 
-KSpreadValue::Format KSpreadValue::format() const
+Value::Format Value::format() const
 {
   return d ? d->format : fmt_None;
 }
 
-void KSpreadValue::setFormat (KSpreadValue::Format fmt)
+void Value::setFormat (Value::Format fmt)
 {
   d->format = fmt;
 }
 
-KSpreadValue KSpreadValue::element( unsigned column, unsigned row ) const
+Value Value::element( unsigned column, unsigned row ) const
 {
-  if( d->type != Array ) return empty();
-  if( !d->pa ) return empty();
-  KSpreadValue* v = d->pa->at( column, row );
-  return v ? KSpreadValue( *v ) : empty();
+  if( d->type != Array ) return *this;
+  if( !d->pa ) return *this;
+  Value* v = d->pa->at (column % columns(), row % rows());
+  return v ? Value( *v ) : empty();
 }
 
-void KSpreadValue::setElement( unsigned column, unsigned row, const KSpreadValue& v )
+void Value::setElement( unsigned column, unsigned row, const Value& v )
 {
   if( d->type != Array ) return;
   if( !d->pa ) return;
   detach();
-  d->pa->set( column, row, new KSpreadValue( v ) );
+  d->pa->set( column, row, new Value( v ) );
 }
 
-unsigned KSpreadValue::columns() const
+unsigned Value::columns() const
 {
-  if( d->type != Array ) return 0;
-  if( !d->pa ) return 0;
+  if( d->type != Array ) return 1;
+  if( !d->pa ) return 1;
   return d->pa->columns;
 }
 
-unsigned KSpreadValue::rows() const
+unsigned Value::rows() const
 {
-  if( d->type != Array ) return 0;
-  if( !d->pa ) return 0;
+  if( d->type != Array ) return 1;
+  if( !d->pa ) return 1;
   return d->pa->rows;
 }
 
 // reference to empty value
-const KSpreadValue& KSpreadValue::empty()
+const Value& Value::empty()
 {
   return ks_value_empty;
 }
 
 // reference to #DIV/0! error
-const KSpreadValue& KSpreadValue::errorDIV0()
+const Value& Value::errorDIV0()
 {
   if( !ks_error_div0.isError() )
     ks_error_div0.setError( "#DIV/0!" );
@@ -549,7 +671,7 @@ const KSpreadValue& KSpreadValue::errorDIV0()
 }
 
 // reference to #N/A error
-const KSpreadValue& KSpreadValue::errorNA()
+const Value& Value::errorNA()
 {
   if( !ks_error_na.isError() )
     ks_error_na.setError( "#N/A" );
@@ -557,7 +679,7 @@ const KSpreadValue& KSpreadValue::errorNA()
 }
 
 // reference to #NAME? error
-const KSpreadValue& KSpreadValue::errorNAME()
+const Value& Value::errorNAME()
 {
   if( !ks_error_name.isError() )
     ks_error_name.setError( "#NAME?" );
@@ -565,7 +687,7 @@ const KSpreadValue& KSpreadValue::errorNAME()
 }
 
 // reference to #NUM! error
-const KSpreadValue& KSpreadValue::errorNUM()
+const Value& Value::errorNUM()
 {
   if( !ks_error_num.isError() )
     ks_error_num.setError( "#NUM!" );
@@ -573,7 +695,7 @@ const KSpreadValue& KSpreadValue::errorNUM()
 }
 
 // reference to #NULL! error
-const KSpreadValue& KSpreadValue::errorNULL()
+const Value& Value::errorNULL()
 {
   if( !ks_error_null.isError() )
     ks_error_null.setError( "#NULL!" );
@@ -581,7 +703,7 @@ const KSpreadValue& KSpreadValue::errorNULL()
 }
 
 // reference to #REF! error
-const KSpreadValue& KSpreadValue::errorREF()
+const Value& Value::errorREF()
 {
   if( !ks_error_ref.isError() )
     ks_error_ref.setError( "#REF!" );
@@ -589,20 +711,20 @@ const KSpreadValue& KSpreadValue::errorREF()
 }
 
 // reference to #VALUE! error
-const KSpreadValue& KSpreadValue::errorVALUE()
+const Value& Value::errorVALUE()
 {
   if( !ks_error_value.isError() )
     ks_error_value.setError( "#VALUE!" );
   return ks_error_value;
 }
 
-// detach, create deep copy of KSpreadValueData
-void KSpreadValue::detach()
+// detach, create deep copy of ValueData
+void Value::detach()
 {
   if( d->isNull() || ( d->count > 1 ) )
   {
-    KSpreadValueData* n;
-    n = new KSpreadValueData;
+    ValueData* n;
+    n = new ValueData;
 
     n->type = d->type;
     switch( n->type )
@@ -622,7 +744,7 @@ void KSpreadValue::detach()
   }
 }
 
-int KSpreadValue::compare( double v1, double v2 )
+int Value::compare( double v1, double v2 )
 {
   double v3 = v1 - v2;
   if( v3 > DBL_EPSILON ) return 1;
@@ -630,21 +752,21 @@ int KSpreadValue::compare( double v1, double v2 )
   return 0;
 }
 
-bool KSpreadValue::isZero( double v )
+bool Value::isZero( double v )
 {
   return fabs( v ) < DBL_EPSILON;
 }
 
-bool KSpreadValue::isZero() const
+bool Value::isZero() const
 {
   if( !isNumber() ) return false;
   return isZero( asFloat() );
 }
 
-bool KSpreadValue::allowComparison( const KSpreadValue& v ) const
+bool Value::allowComparison( const Value& v ) const
 {
-  KSpreadValue::Type t1 = d->type;
-  KSpreadValue::Type t2 = v.type();
+  Value::Type t1 = d->type;
+  Value::Type t2 = v.type();
   
   if( ( t1 == Empty ) && ( t2 == Empty ) ) return true;
   if( ( t1 == Empty ) && ( t2 == String ) ) return true;
@@ -670,14 +792,17 @@ bool KSpreadValue::allowComparison( const KSpreadValue& v ) const
   if( ( t1 == String ) && ( t2 == Float ) ) return true;
   if( ( t1 == String ) && ( t2 == String ) ) return true;
   
+  // errors can be compared too ...
+  if ((t1 == Error) && (t2 == Error)) return true;
+  
   return false;
 }
 
 // compare values. looks strange in order to be compatible with Excel
-int KSpreadValue::compare( const KSpreadValue& v ) const 
+int Value::compare( const Value& v ) const 
 {
-  KSpreadValue::Type t1 = d->type;
-  KSpreadValue::Type t2 = v.type();
+  Value::Type t1 = d->type;
+  Value::Type t2 = v.type();
   
   // errors always less than everything else
   if( ( t1 == Error ) && ( t2 != Error ) )
@@ -780,59 +905,62 @@ int KSpreadValue::compare( const KSpreadValue& v ) const
   return 0;
 }
 
-bool KSpreadValue::equal( const KSpreadValue& v ) const
+bool Value::equal( const Value& v ) const
 {
+  if (!allowComparison (v)) return false;
   return compare( v ) == 0;
 }
 
-bool KSpreadValue::less( const KSpreadValue& v ) const
+bool Value::less( const Value& v ) const
 {
+  if (!allowComparison (v)) return false;
   return compare( v ) < 0;
 }
 
-bool KSpreadValue::greater( const KSpreadValue& v ) const
+bool Value::greater( const Value& v ) const
 {
+  if (!allowComparison (v)) return false;
   return compare( v ) > 0;
 }
 
-QTextStream& operator<<( QTextStream& ts, KSpreadValue::Type type ) 
+QTextStream& operator<<( QTextStream& ts, Value::Type type ) 
 {
   switch( type )
   {
-    case KSpreadValue::Empty:   ts << "Empty"; break;
-    case KSpreadValue::Boolean: ts << "Boolean"; break;
-    case KSpreadValue::Integer: ts << "Integer"; break;
-    case KSpreadValue::Float:   ts << "Float"; break;
-    case KSpreadValue::String:  ts << "String"; break;
-    case KSpreadValue::Array:   ts << "Array"; break;
-    case KSpreadValue::Error:   ts << "Error"; break;
+    case Value::Empty:   ts << "Empty"; break;
+    case Value::Boolean: ts << "Boolean"; break;
+    case Value::Integer: ts << "Integer"; break;
+    case Value::Float:   ts << "Float"; break;
+    case Value::String:  ts << "String"; break;
+    case Value::Array:   ts << "Array"; break;
+    case Value::Error:   ts << "Error"; break;
     default: ts << "Unknown!"; break;
   };
   return ts;
 }
 
-QTextStream& operator<<( QTextStream& ts, KSpreadValue value ) 
+QTextStream& operator<<( QTextStream& ts, Value value ) 
 {
   ts << value.type();
   switch( value.type() )
   {
-    case KSpreadValue::Empty:   break;
+    case Value::Empty:   break;
     
-    case KSpreadValue::Boolean: 
+    case Value::Boolean: 
       ts << ": "; 
       if (value.asBoolean()) ts << "TRUE"; 
       else ts << "FALSE"; break;
       
-    case KSpreadValue::Integer: 
+    case Value::Integer: 
       ts << ": " << value.asInteger(); break;
       
-    case KSpreadValue::Float:   
+    case Value::Float:   
       ts << ": " << value.asFloat(); break;
       
-    case KSpreadValue::String:  
+    case Value::String:  
       ts << ": " << value.asString(); break;
       
-    case KSpreadValue::Error: 
+    case Value::Error: 
       ts << "(" << value.errorMessage() << ")"; break;
     
     default: break;
