@@ -1,5 +1,5 @@
 /*
-    $Id: kcalc_core.cpp 399247 2005-03-20 16:49:54Z johnflux $
+    $Id: kcalc_core.cpp 466447 2005-10-02 17:54:10Z zander $
 
     kCalculator, a scientific calculator for the X window system using the
     Qt widget libraries, available at no cost at http://www.troll.no
@@ -24,7 +24,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 
 */
 
