@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
    Copyright 2004 Tomas Mecir <mecirt@gmail.com>
-   Copyright (C) 1998-2004 KSpread Team <koffice-devel@mail.kde.org>
+   Copyright (C) 1998-2004 KSpread Team <koffice-devel@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include "valueformatter.h"
@@ -38,31 +38,34 @@ ValueFormatter::ValueFormatter (ValueConverter *conv) : converter( conv )
 {
 }
 
-QString ValueFormatter::formatText (KSpreadCell *cell, FormatType fmtType)
+QString ValueFormatter::formatText (Cell *cell, FormatType fmtType)
 {
   if (cell->hasError ())
     return errorFormat (cell);
 
   QString str;
-  
-  KSpreadFormat::FloatFormat floatFormat =
-      cell->floatFormat (cell->column(), cell->row());
-  int precision = cell->precision (cell->column(), cell->row());
-  QString prefix = cell->prefix (cell->column(), cell->row());
-  QString postfix = cell->postfix (cell->column(), cell->row());
+
+  Format::FloatFormat floatFormat =
+      cell->format()->floatFormat (cell->column(), cell->row());
+  int precision = cell->format()->precision (cell->column(), cell->row());
+  QString prefix = cell->format()->prefix (cell->column(), cell->row());
+  QString postfix = cell->format()->postfix (cell->column(), cell->row());
+  Format::Currency currency;
+  bool valid = cell->format()->currencyInfo(currency);
+  QString currencySymbol = valid ? currency.symbol : QString::null;
 
   return formatText (cell->value(), fmtType, precision,
-      floatFormat, prefix, postfix);
+      floatFormat, prefix, postfix, currencySymbol);
 }
 
-QString ValueFormatter::formatText (const KSpreadValue &value,
-    FormatType fmtType, int precision, KSpreadFormat::FloatFormat floatFormat,
-    const QString &prefix, const QString &postfix)
+QString ValueFormatter::formatText (const Value &value,
+    FormatType fmtType, int precision, Format::FloatFormat floatFormat,
+    const QString &prefix, const QString &postfix, const QString &currencySymbol)
 {
   //if we have an array, use its first element
   if (value.isArray())
     return formatText (value.element (0, 0), fmtType, precision,
-        floatFormat, prefix, postfix);
+        floatFormat, prefix, postfix, currencySymbol);
 
   QString str;
   
@@ -92,6 +95,15 @@ QString ValueFormatter::formatText (const KSpreadValue &value,
     str = fractionFormat (value.asFloat(), fmtType);
 
   //another
+  else if (value.isInteger())
+  {
+	long v = value.asInteger();
+	// Always unsigned ?
+    if ((floatFormat == Format::AlwaysUnsigned) && (v < 0))
+      v *= -1;
+    str = createNumberFormat (v, fmtType,
+        (floatFormat == Format::AlwaysSigned), currencySymbol);
+  }
   else
   {
     QChar decimal_point = converter->locale()->decimalSymbol()[0];
@@ -102,35 +114,35 @@ QString ValueFormatter::formatText (const KSpreadValue &value,
     double v = converter->asFloat (value).asFloat();
 
     // Always unsigned ?
-    if ((floatFormat == KSpreadCell::AlwaysUnsigned) && (v < 0.0))
+    if ((floatFormat == Format::AlwaysUnsigned) && (v < 0.0))
       v *= -1.0;
 
     // Make a string out of it.
     str = createNumberFormat (v, precision, fmtType,
-        (floatFormat == KSpreadCell::AlwaysSigned));
+        (floatFormat == Format::AlwaysSigned), currencySymbol);
 
     // Remove trailing zeros and the decimal point if necessary
     // unless the number has no decimal point
     if (precision == -1)
       removeTrailingZeros (str, decimal_point);
   }
-    
+
   if (!prefix.isEmpty())
     str = prefix + ' ' + str;
 
   if( !postfix.isEmpty())
     str += ' ' + postfix;
-  
+
   //kdDebug() << "ValueFormatter says: " << str << endl;
   return str;
 }
 
-FormatType ValueFormatter::determineFormatting (const KSpreadValue &value,
+FormatType ValueFormatter::determineFormatting (const Value &value,
     FormatType fmtType)
 {
   //if the cell value is a string, then we want to display it as-is,
   //no matter what, same if the cell is empty
-  if (value.isString () || (value.format() == KSpreadValue::fmt_None))
+  if (value.isString () || (value.format() == Value::fmt_None))
     return Text_format;
   //same if we're supposed to display string, no matter what we actually got
   if (fmtType == Text_format)
@@ -140,36 +152,36 @@ FormatType ValueFormatter::determineFormatting (const KSpreadValue &value,
   if (fmtType == Generic_format)
   {
     //here we decide based on value's format...
-    KSpreadValue::Format fmt = value.format();
+    Value::Format fmt = value.format();
     switch (fmt) {
-      case KSpreadValue::fmt_None:
+      case Value::fmt_None:
         fmtType = Text_format;
       break;
-      case KSpreadValue::fmt_Boolean:
+      case Value::fmt_Boolean:
         fmtType = Text_format;
       break;
-      case KSpreadValue::fmt_Number:
+      case Value::fmt_Number:
         if (value.asFloat() > 1e+10)
           fmtType = Scientific_format;
         else
           fmtType = Number_format;
       break;
-      case KSpreadValue::fmt_Percent:
+      case Value::fmt_Percent:
         fmtType = Percentage_format;
       break;
-      case KSpreadValue::fmt_Money:
+      case Value::fmt_Money:
         fmtType = Money_format;
       break;
-      case KSpreadValue::fmt_DateTime:
+      case Value::fmt_DateTime:
         fmtType = TextDate_format;
       break;
-      case KSpreadValue::fmt_Date:
+      case Value::fmt_Date:
         fmtType = ShortDate_format;
       break;
-      case KSpreadValue::fmt_Time:
+      case Value::fmt_Time:
         fmtType = Time_format;
       break;
-      case KSpreadValue::fmt_String:
+      case Value::fmt_String:
         //this should never happen
         fmtType = Text_format;
       break;
@@ -213,7 +225,7 @@ void ValueFormatter::removeTrailingZeros (QString &str, QChar decimal_point)
     start = 0;
 
   int i = str.length() - start;
-  bool bFinished = FALSE;
+  bool bFinished = false;
   while ( !bFinished && i > 0 )
   {
     QChar ch = str[i - 1];
@@ -221,22 +233,59 @@ void ValueFormatter::removeTrailingZeros (QString &str, QChar decimal_point)
       str.remove (--i,1);
     else
     {
-      bFinished = TRUE;
+      bFinished = true;
       if (ch == decimal_point)
         str.remove (--i, 1);
     }
   }
 }
 
+
+QString ValueFormatter::createNumberFormat ( long value, FormatType fmt, 
+	 bool alwaysSigned, const QString& currencySymbol)
+{
+  QString number;
+
+  //multiply value by 100 for percentage format
+  if (fmt == Percentage_format)
+    value *= 100;
+
+  switch (fmt)
+  {
+    case Number_format:
+    case Scientific_format:
+      number = QString::number(value);
+      break;
+    case Percentage_format:
+      number = QString::number(value) + " %";
+      break;
+    case Money_format:
+      number = converter->locale()->formatMoney (value, currencySymbol.isEmpty() ? converter->locale()->currencySymbol() : currencySymbol);
+      break;
+    default :
+      //other formatting?
+      // This happens with Custom_format...
+      kdDebug(36001)<<"Wrong usage of ValueFormatter::createNumberFormat fmt=" << fmt << "\n";
+      break;
+  }
+
+  //prepend positive sign if needed
+  if (alwaysSigned && value >= 0 )
+    if (converter->locale()->positiveSign().isEmpty())
+      number='+'+number;
+
+  return number;
+}
+
 QString ValueFormatter::createNumberFormat ( double value, int precision,
-    FormatType fmt, bool alwaysSigned)
+    FormatType fmt, bool alwaysSigned, const QString& currencySymbol)
 {
   // if precision is -1, ask for a huge number of decimals, we'll remove
   // the zeros later. Is 8 ok ?
   int p = (precision == -1) ? 8 : precision;
   QString localizedNumber;
   int pos = 0;
-  
+
   //multiply value by 100 for percentage format
   if (fmt == Percentage_format)
     value *= 100;
@@ -265,8 +314,8 @@ QString ValueFormatter::createNumberFormat ( double value, int precision,
       localizedNumber = converter->locale()->formatNumber (value, p)+ " %";
       break;
     case Money_format:
-      localizedNumber = converter->locale()->formatMoney (value, 
-          converter->locale()->currencySymbol(), p );
+      localizedNumber = converter->locale()->formatMoney (value,
+        currencySymbol.isEmpty() ? converter->locale()->currencySymbol() : currencySymbol, p );
       break;
     case Scientific_format:
       decimal_point = converter->locale()->decimalSymbol()[0];
@@ -276,7 +325,8 @@ QString ValueFormatter::createNumberFormat ( double value, int precision,
       break;
     default :
       //other formatting?
-      kdDebug(36001)<<"Wrong usage of ValueFormatter::createNumberFormat\n";
+      // This happens with Custom_format...
+      kdDebug(36001)<<"Wrong usage of ValueFormatter::createNumberFormat fmt=" << fmt << "\n";
       break;
   }
 
@@ -623,14 +673,14 @@ QString ValueFormatter::dateFormat (const QDate &date, FormatType fmtType)
   return tmp;
 }
 
-QString ValueFormatter::errorFormat (KSpreadCell *cell)
+QString ValueFormatter::errorFormat (Cell *cell)
 {
   QString err;
-  if (cell->testFlag (KSpreadCell::Flag_ParseError))
+  if (cell->testFlag (Cell::Flag_ParseError))
     err = "#" + i18n ("Parse") + "!";
-  else if ( cell->testFlag (KSpreadCell::Flag_CircularCalculation))
+  else if ( cell->testFlag (Cell::Flag_CircularCalculation))
     err = "#" + i18n ("Circle") + "!";
-  else if ( cell->testFlag (KSpreadCell::Flag_DependancyError))
+  else if ( cell->testFlag (Cell::Flag_DependancyError))
     err = "#" + i18n ("Depend") + "!";
   else
   {
