@@ -7,7 +7,7 @@
 #define _KUGAR_FACTORY_H
 
 
-#include <koFactory.h>
+#include <KoFactory.h>
 #include <kaboutdata.h>
 
 class KInstance;
@@ -21,8 +21,8 @@ public:
     ~KugarFactory();
 
     virtual KParts::Part *createPartObject( QWidget *parentWidget = 0, const char *widgetName = 0,
-                QObject *parent = 0, const char *name = 0, const char *classname = "KoDocument",
-                const QStringList &args = QStringList() );
+                                            QObject *parent = 0, const char *name = 0, const char *classname = "KoDocument",
+                                            const QStringList &args = QStringList() );
 
     static KInstance* global();
 
@@ -33,4 +33,4 @@ private:
     static KAboutData* s_aboutdata;
 };
 
-#endif 
+#endif
