@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qstring.h>
@@ -174,7 +174,7 @@ QDomElement BasicElement::getElementDom( QDomDocument& doc)
 }
 
 
-void BasicElement::writeMathML( QDomDocument& doc, QDomNode parent, bool oasisFormat )
+void BasicElement::writeMathML( QDomDocument& doc, QDomNode parent, bool /*oasisFormat*/ )
 {
     parent.appendChild( doc.createComment( QString( "MathML Error in %1" )
                                            .arg( getTagName() ) ) );
