@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef BASICELEMENT_H
@@ -64,7 +64,7 @@ class SequenceElement;
  *
  * To build a tree an element must own children. If there are children
  * there must be a main child. This is the child that might be used to
- * replace the element. See @ref getMainChild and @ref setMainChild .
+ * replace the element. See @ref getMainChild().
  *
  * If there can be children you might want to @ref insert and @ref remove
  * them. After a removal the element might be senseless.
@@ -364,12 +364,12 @@ public:
      * children, and as attribute the attribute of this
      * element.
      */
-    QDomElement getElementDom(QDomDocument& doc);
+    QDomElement getElementDom( QDomDocument& doc);
 
     /**
      * Same as above, just MathML.
      */
-    virtual void writeMathML( QDomDocument doc, QDomNode parent );
+    virtual void writeMathML( QDomDocument& doc, QDomNode parent, bool oasisFormat = false );
 
     /**
      * Set this element attribute, build children and
