@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #include <qptrlist.h>
@@ -28,7 +28,7 @@
 #include <ksimpleconfig.h>
 #include <kstandarddirs.h>
 
-#include <koDocument.h>
+#include <KoDocument.h>
 
 #include "contextstyle.h"
 #include "creationstrategy.h"
@@ -326,11 +326,11 @@ void Document::updateConfig()
 }
 
 
-void Document::introduceWrapper( DocumentWrapper* wrapper )
+void Document::introduceWrapper( DocumentWrapper* wrapper, bool init )
 {
     m_wrapper = wrapper;
-    m_contextStyle->readConfig( wrapper->config() );
-    m_contextStyle->init();
+    m_contextStyle->readConfig( wrapper->config(), init );
+    m_contextStyle->init( init );
 }
 
 
@@ -368,10 +368,10 @@ DocumentWrapper::~DocumentWrapper()
 }
 
 
-void DocumentWrapper::document( Document* document )
+void DocumentWrapper::document( Document* document, bool init )
 {
     m_document = document;
-    m_document->introduceWrapper( this );
+    m_document->introduceWrapper( this, init );
     initSymbolNamesAction();
     m_config->setGroup("General");
     if ( m_hasActions )
