@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (c) 2001 David Faure <faure@kde.org>
 
-   $Id: KoMainWindowIface.cc 161643 2002-06-17 15:54:56Z kervel $
+   $Id: KoMainWindowIface.cc 508787 2006-02-12 18:28:12Z ingwa $
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -15,12 +15,12 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 #include "KoMainWindowIface.h"
 
-#include "koMainWindow.h"
+#include "KoMainWindow.h"
 
 #include <kapplication.h>
 #include <dcopclient.h>
