@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef _KOFFICE_VERSION_H_
@@ -29,10 +29,10 @@
 // - karbon/karbon_aboutdata.h
 //
 
-#define KOFFICE_VERSION_STRING "1.4.1"
+#define KOFFICE_VERSION_STRING "1.5.0"
 #define KOFFICE_VERSION_MAJOR 1
-#define KOFFICE_VERSION_MINOR 4
-#define KOFFICE_VERSION_RELEASE 1
+#define KOFFICE_VERSION_MINOR 5
+#define KOFFICE_VERSION_RELEASE 0
 #define KOFFICE_MAKE_VERSION( a,b,c ) (((a) << 16) | ((b) << 8) | (c))
 
 #define KOFFICE_VERSION \
